/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package controls;

import java.awt.event.MouseEvent;
import java.awt.image.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.TooManyListenersException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.*;

/**
 *
 * @author stefano
 */
public class ImagePanel extends JPanel implements MouseListener {
	private Image image = null;
	private static Color normal = new Color(238, 238, 238);
	private static Color dnd = new Color(200, 200, 200);
	private ActionListener actionListener = null;
	private JPopupMenu pm;
	
	public ImagePanel() {
		
		this.setDropTarget(new DropTargetImpl(this));
		this.addMouseListener(this);

		pm = new JPopupMenu();
		JMenuItem mi = new JMenuItem("Select image...");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				FileDialog fd = new FileDialog((JFrame) SwingUtilities.getWindowAncestor(ImagePanel.this));
				fd.setVisible(true);

				if (fd.getFile() != null) {
					File f = new File(fd.getDirectory() + File.separator + fd.getFile());
					ImagePanel.this.load(f);
				}
			}
		});
		pm.add(mi);
		
		pm.addSeparator();
		mi = new JMenuItem("Clear");
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ImagePanel.this.setImage(null);
			}
		});
		pm.add(mi);
	}

	@Override
	public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		if (image == null) {
			g2.setBackground(getBackground());
			//g2.clearRect(0, 0, getWidth(), getHeight());
			g2.setFont(getFont().deriveFont(9f));
			g2.drawString( "Drag an image.", 15, (int) (0.5 * getHeight()));
		} else {
			g2.setBackground(getBackground());
			//g2.clearRect(0, 0, getWidth(), getHeight());
			int width = getWidth() - 20;
			int height = getHeight() - 20;
			if (width > image.getWidth(null))
				width = image.getWidth(null);
			if (height > image.getHeight(null))
				height = image.getHeight(null);
			int iwidth = image.getWidth(null);
			int iheight =  image.getHeight(null);
			if (iwidth > iheight)
				height = width * (iheight) / iwidth;
			else
				width = height * (iwidth) / iheight;

			g2.drawImage(image, (int)(0.5 * (getWidth() - width)),
					(int)(0.5 * (getHeight() - height)), width, height, null);
		}

		if (in) {
			int cx = 15;
			int cy = getHeight() - 15;
			g2.setColor(new Color(255, 255, 255, 200));
			g2.fillOval(cx - 10, cy - 10, 15, 15);
			g2.setColor(Color.BLACK);
			g2.drawOval(cx - 10, cy - 10, 15, 15);
			int[] x = new int[] { cx - 6, cx, cx - 3 };
			int[] y = new int[] { cy - 4, cy - 4, cy };
			g2.fillPolygon(x, y, 3);
			
		}
		
	}

	public byte[] toBytes() {
		if (image == null)
			return null;

		try {
			BufferedImage im = new BufferedImage(image.getWidth(null), image.getHeight(null),
					BufferedImage.TYPE_INT_ARGB);
			im.createGraphics().drawImage(image, 0, 0, null);
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			ImageIO.write(im, "png", bos);
			return bos.toByteArray();
		} catch (IOException ex) {
			ex.printStackTrace();
			return null;
		}
	}

	public void fromBytes(byte[] arr) {
		try {
			if (arr == null) {
				return;
			}
			ByteArrayInputStream bis = new ByteArrayInputStream(arr);
			image = ImageIO.read(bis);
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	public void dragEnter(DropTargetDragEvent e) {
		DataFlavor[] dataFlavours = e.getCurrentDataFlavors();
		for(int i=0; i<dataFlavours.length; i++) {
			if(dataFlavours[i].isFlavorJavaFileListType()) {
				e.acceptDrag(DnDConstants.ACTION_COPY);
			} else
				e.rejectDrag();
		}
	}

	public void setImage(Image image) {
		if (image == this.image)
			return;

		this.image = image;
		if (actionListener != null)
			actionListener.actionPerformed(new ActionEvent(this, 0, ""));
		repaint();

	}

	public void setSelectedState(boolean selected) {
		if (selected)
			setBorder(UIManager.getBorder("InsetBorder.aquaVariant"));
		else
			setBorder(null);
	}

	public void setImageActionListener(ActionListener al) {
		actionListener = al;
	}

	public Image getImage() {
		return image;
	}

	public void load(File f) {
		try {
			setImage(ImageIO.read(f));
		} catch (IOException ex) {}
	}

	public void mouseClicked(MouseEvent e) {
		pm.show(this, 0, getHeight());
	}

	public void mousePressed(MouseEvent e) {}

	public void mouseReleased(MouseEvent e) {}

	boolean in = false;
	public void mouseEntered(MouseEvent e) {
		in = true;
		repaint();
	}

	public void mouseExited(MouseEvent e) {
		in = false;
		repaint();
	}



	private static class DropTargetImpl extends DropTarget {
		public DropTargetImpl(final ImagePanel ip) {
			super();
			try {
				addDropTargetListener(new DropTargetListener() {
						public void dragEnter(DropTargetDragEvent e) {
							DataFlavor[] dataFlavours = e.getCurrentDataFlavors();
							for(int i=0; i<dataFlavours.length; i++) {
								if(dataFlavours[i].isFlavorJavaFileListType()) {
									e.acceptDrag(DnDConstants.ACTION_COPY);
								}
							}
						}


						public void dragOver(DropTargetDragEvent dtde) {
							ip.setCursor((Cursor)Toolkit.getDefaultToolkit().getDesktopProperty("DnD.Cursor.CopyDrop"));
							ip.setSelectedState(true);
						}

						public void dropActionChanged(DropTargetDragEvent dtde) {

						}

						public void dragExit(DropTargetEvent dte) {
							ip.setCursor(Cursor.getDefaultCursor());
							ip.setSelectedState(false);
						}

						public void drop(DropTargetDropEvent dtde) {
							try {
								dtde.acceptDrop(DnDConstants.ACTION_COPY);
								ip.setCursor(Cursor.getDefaultCursor());
								ip.setSelectedState(false);
								Transferable tr = dtde.getTransferable();
								java.util.List l = (java.util.List) tr.getTransferData(DataFlavor.javaFileListFlavor);
								File f = (File) l.get(0);
								ip.load(f);
							} catch (Exception e) {
								e.printStackTrace();
								Toolkit.getDefaultToolkit().beep();
							}
						}
				});
			} catch (TooManyListenersException ex) {

			}
		}

	}
}
