/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package controls;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 *
 * @author stefano
 */
public class SmallSlider extends JPanel implements MouseMotionListener, MouseListener {
    private double min;
    private double max;
    
    private boolean log;
    private double val;
    private int knobSize = -1;
    private ActionListener al;
    private static ActionEvent moved = new ActionEvent("", 0, "moved");
    private static ActionEvent released = new ActionEvent("", 0, "released");
    
    private Color c1 = new Color(91, 165, 219);
    private Color c2 = new Color(54, 136, 205).darker();
    
    private SmallSlider() {};
    
    public SmallSlider(double min, double max, double val, boolean log, ActionListener l) {
        al = l;
        this.min = min;
        this.max = max;
        this.val = val;
        this.log = log;
        
        this.addMouseMotionListener(this);
        this.addMouseListener(this);
    }
    
    private int getKnobSize() {
        int maxSize = getWidth() - 4;
        
        if (log) {
            return (int)((Math.log10(val) - Math.log10(min))/(Math.log10(max) - Math.log10(min)) * maxSize);
        } else {
            return (int)((val - min) / (max-min) * maxSize);
        }
    }
    
    public void setVal(double val) {
        this.val = val;
        knobSize = -1;
        repaint();
    }
    
    public double getVal() {
        return val;
    }
    
    @Override
    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        
        g2.setBackground(getBackground());
        g2.clearRect(0, 0, getWidth(), getHeight());
        
        
        
        g2.setPaint(new GradientPaint(0, 0, c1, getWidth(), 0, c2));
        
        knobSize = (knobSize == -1  ? getKnobSize() : knobSize);
        g2.fillRoundRect(2, 2, knobSize, getHeight()-4, 8, 8);
        g2.setColor(getForeground());
        g2.drawRoundRect(2, 2, getWidth()-4, getHeight()-4, 8, 8);
        
    }

    private void handleMouse(MouseEvent e) {
        int x = e.getX();
        if (x < 0) {
            setVal(min);
            al.actionPerformed(moved);
            repaint();
            return;
        }
        if (x > getWidth()) {
            setVal(min);
            al.actionPerformed(moved);
            repaint();
            return;
        }
        
        if (log) {
            setVal(Math.pow(10, (double) x / (double) getWidth() * (Math.log10(max) - Math.log10(min)) + Math.log10(min)));
        } else {
            setVal((double)x / (double)getWidth() * (max - min) + min);
        }
        
        
        al.actionPerformed(moved);
    }
    
    public void mouseDragged(MouseEvent e) {
        handleMouse(e);
    }

    public void mouseMoved(MouseEvent e) {
        
    }

    public void mouseClicked(MouseEvent e) {
        //handleMouse(e);
        //al.actionPerformed(released);
    }

    public void mousePressed(MouseEvent e) {
     
    }

    public void mouseReleased(MouseEvent e) {
        handleMouse(e);
        al.actionPerformed(released);
    }

    public void mouseEntered(MouseEvent e) {
     
    }

    public void mouseExited(MouseEvent e) {
     
    }


    
}
