
package numal;

import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import utils.Utils;
import static java.lang.Math.*;

public class NumUtils {

    // puts an angle in the 0-360 degrees range
    public static double normAngle(double angle) {
        while (angle < 0) {
            angle += 360.;
        }
        return angle % 360.;
    }

    public static double normAngleR(double angle) {
        while (angle < 0) {
            angle += 2. * PI;
        }
        return angle % (2 * PI);
    }
    public static double[] unwrap(double[] v, double diff) {
        if ((v == null) || v.length == 0) {
            return null;
        }

        double adding = 0.;
        double[] ret = new double[v.length];
        ret[0] = v[0];

        for (int k = 1; k < v.length; k++) {
            ret[k] = v[k] + adding;
            int trials = 0;

            while (Math.abs(ret[k] - ret[k - 1]) > diff) {
                double adding2 = 0;

                double diff0 = Math.abs(ret[k] - ret[k - 1]) % 360.;
                double diff1 = Math.abs(ret[k] - ret[k - 1] + 360) % 360.;
                double diff2 = Math.abs(ret[k] - ret[k - 1] - 360) % 360.;

                if ((diff1 >= diff0) && (diff2 >= diff0)) {
                    break;
                }

                if (diff2 > diff1) {
                    adding2 = 360;
                } else {
                    adding2 = -360;
                }

                /*if (Math.abs(ret[k] % 360. - ret[k - 1] % 360. + 360) <
                Math.abs(ret[k] % 360. - ret[k - 1] % 360. - 360)) {
                adding2 = 360;
                } else {
                adding2 = -360;
                }
                 */

                adding += adding2;
                ret[k] += adding2;

                trials++;
                if (trials > 20) {
                    System.out.println(ret[k - 1] + " " + ret[k] + "  " + adding + " " + diff0 + " " + diff1 + " " + diff2);
                    System.out.println("this should not happen.");
                    System.exit(0);
                }
            }

        }
        return ret;
    }

    public final static double average(final double[] v) {
        double avg = v[0];

        for (int i = 1; i < v.length; i++) {
            avg += v[i];
        }

        return avg / (double) v.length;
    }

    public final static double median(final double[] v) {
        double[] sorted = Utils.copy(v);
        Arrays.sort(sorted);
        int idx = (int) (((double) sorted.length)/2.);
        return sorted[idx];
    }

    public static double average(double[] v, int avgtype) {
        if (avgtype == 0)
            return average(v);
        else if (avgtype == 1)
            return median(v);
        else 
            return Double.NaN;
    }


    public static double rms(double[] v) {
        double avg = v[0];

        for (int i = 1; i < v.length; i++) {
            avg += v[i] * v[i];
        }

        return sqrt(avg / (double) v.length);
    }

    public static final double average_angle(final double[] v, boolean isRadians, int avgType) {
        if (avgType == 0)
            return average_angle(v, isRadians);
        else if (avgType == 1)
            return median_angle(v, isRadians);
        else
            return Double.NaN;
    }

    public static final double average_angle(final double[] v, boolean isRadians) {
        double cos_avg = cos((isRadians ? v[0] : toRadians(v[0])));
        double sin_avg = sin((isRadians ? v[0] : toRadians(v[0])));

        for (int i = 1; i < v.length; i++) {
            cos_avg += cos((isRadians ? v[i] : toRadians(v[i])));
            sin_avg += sin((isRadians ? v[i] : toRadians(v[i])));
        }

        double avg = atan2(sin_avg / (double) v.length, cos_avg / (double) v.length);

        if (avg < 0) {
            avg += 2 * PI;
        }

        if (!isRadians) {
            avg = toDegrees(avg);
        }

        return avg;
    }

    public static final double median_angle(final double[] v, boolean isRadians) {
        double[] cos_avg = new double[v.length];
        double[] sin_avg = new double[v.length];

        for (int i = 0; i < v.length; i++) {
            cos_avg[i] = cos((isRadians ? v[i] : toRadians(v[i])));
            sin_avg[i] = sin((isRadians ? v[i] : toRadians(v[i])));
        }

        double avg = atan2(median(sin_avg), median(cos_avg));

        if (avg < 0) {
            avg += 2 * PI;
        }

        if (!isRadians) {
            avg = toDegrees(avg);
        }

        return avg;
    }


        public final static double stdDev(final double[] v) {
        return stdDev(v, 0);
    }

    public final static double stdDev(final double[] v, int avgType) {
        return sqrt(variance(v, avgType));
    }

    public final static double variance(final double[] v) {
        return variance(v, 0);
    }

    public final static double variance(final double[] v, int avgType) {
        if (v.length <= 1) {
            return 0;
        }

        double avg = average(v, avgType);
        double dev = 0;

        for (int i = 0; i < v.length; i++) {
            dev += (v[i] - avg) * (v[i] - avg);
        }

        return (1 / (double) (v.length - 1) * dev);
    }

    public static final double stdDev_angle(final double[] v, boolean isRadians) {
        return stdDev_angle(v, isRadians, 0);
    }

    public static final double stdDev_angle(final double[] v, boolean isRadians, int avgType)  {
        if (v.length <= 1) {
            return 0;
        }

        double avg = average_angle(v, isRadians, avgType);
        double avg_r = (isRadians ? avg : toRadians(avg));
        
        double dev = 0;

        for (int i = 0; i < v.length; i++) {
            double val = (isRadians ? v[i] : toRadians(v[i]));
            val = val % (2 * PI);
            if (val < 0)
                val += 2 * PI;

            double diff = Utils.amin(new double[] { normAngleR(abs(val - avg_r)), normAngleR(abs(val - avg_r + 2*PI)),
                normAngleR(abs(val - avg_r - 2*PI)) });
            dev += diff * diff;
        }

        dev = sqrt(1 / (double) (v.length) * dev);
        

        if (!isRadians) {
            dev = toDegrees(dev);
        }
        
        
        return dev;
    }

    public static final double percentile(final double[] v, double perc, int avgType) {
        double[] v2 = Utils.copy(v);
        double avg = average(v2, avgType);

        for (int i = 0; i < v2.length; i++) {
            v2[i] = Math.abs(v2[i] - avg);
        }

        Arrays.sort(v2);
        int pidx = (int) (perc * (double) v.length);

        return v2[pidx];
    }


    public static double percentile_angle(double[] v, double perc, boolean isRadians, int avgType) {


        double[] v2 = new double[v.length];
        

        double avg = average_angle(v, isRadians, avgType);
        double avg_r = isRadians ? avg : toRadians(avg);
        
        for (int i = 0; i < v.length; i++) {
            double val = (isRadians ? v[i] : toRadians(v[i]));
            val = val % (2 * PI);
            if (val < 0)
                val += 2 * PI;

            double diff = Utils.amin(new double[] { normAngleR(abs(val - avg_r)), normAngleR(abs(val - avg_r + 2*PI)),
                normAngleR(abs(val - avg_r - 2*PI)) });
            v2[i] = diff;
        }

        
        Arrays.sort(v2);
        int pidx = (int) (perc * (double) v.length);
        double ret =  v2[pidx];

        
        
        return (isRadians ? ret : toDegrees(ret));
    }

    public static double betai(double a, double b, double x) {
        double bt, result;
        if (x < 0.0) {
            result = 0.0;
        }
        if (x > 1.0) {
            result = 1.0;
        }
        if (x == 0.0 || x == 1.0) {
            bt = 0.0;
        } else {
            bt = Math.exp(gammln(a + b)
                    - gammln(a)
                    - gammln(b)
                    + a * Math.log(x)
                    + b * Math.log(1.0 - x));
        }
        if (x < (a + 1.0) / (a + b + 2.0)) {
            result = bt * betacf(a, b, x) / a;
        } else {
            result = 1.0 - bt * betacf(b, a, 1.0 - x) / b;
        }
        return result;
    }

    public static double betacf(double a, double b, double x) {
        int maxit = 100;
        double eps = 3.0 * Math.pow(10, -7);
        double fpmin = 1.0 * Math.pow(10, -30);
        double result;
        int m2;
        double aa, c, d, del, h, qab, qam, qap;
        qab = a + b;
        qap = a + 1.0;
        qam = a - 1.0;
        c = 1.0;
        d = 1.0 - qab * x / qap;
        if (Math.abs(d) < fpmin) {
            d = fpmin;
        }
        d = 1.0 / d;
        h = d;
        for (int m = 1; m <= maxit; m++) {
            m2 = 2 * m;
            aa = m * (b - m) * x / ((qam + m2) * (a + m2));
            d = 1.0 + aa * d;
            if (Math.abs(d) < fpmin) {
                d = fpmin;
            }
            c = 1.0 + aa / c;
            if (Math.abs(c) < fpmin) {
                c = fpmin;
            }
            d = 1.0 / d;
            h = h * d * c;
            aa = -(a + m) * (qab + m) * x / ((a + m2) * (qap + m2));
            d = 1.0 + aa * d;
            if (Math.abs(d) < fpmin) {
                d = fpmin;
            }
            c = 1.0 + aa / c;
            if (Math.abs(c) < fpmin) {
                c = fpmin;
            }
            d = 1.0 / d;
            del = d * c;
            h = h * del;
            if (Math.abs(del - 1.0) < eps) {
                break;
            }
        }
        result = h;
        return result;
    }

    static public double gammln(double xx) {
        double result;
        double ser, tmp, x, y;
        double stp = 2.5066282746310005;
        double[] cof = {76.18009172947146,
            -86.50532032941677,
            24.01409824083091,
            -1.231739572450155,
            .1208650973866179 * Math.pow(10, -2),
            -.5395239384953 * Math.pow(10, -5)};
        x = xx;
        y = x;
        tmp = x + 5.5;
        tmp = (x + 0.5) * Math.log(tmp) - tmp;
        ser = 1.000000000190015;
        for (int j = 0; j < cof.length; j++) {
            y = y + 1.0;
            ser = ser + cof[j] / y;
        }
        result = tmp + Math.log(stp * ser / x);
        return result;
    }

    static public double[] avevar(double[] in) {
        double[] av = new double[2];
        double sum = 0.0;
        for (int i = 0; i < in.length; i++) {
            sum += in[i];
            //System.out.println(" " + in[i] + " " + sum);
        }
        av[0] = sum / in.length;
        //System.out.println("av[0]: " + av[0]);
        sum = 0.0;
        for (int i = 0; i < in.length; i++) {
            sum += (in[i] - av[0]) * (in[i] - av[0]);
            //System.out.println(" " + in[i] + " " + sum);
        }
        av[1] = sum / (in.length - 1);
        //System.out.println("av[1]: " + av[1]);
        return av;
    }

    static public double[] ftest(double[] data1, double[] data2) {
        double f, df1, df2, prob;
        double av1, av2;

        av1 = avevar(data1)[1];
        av2 = avevar(data2)[1];
        if (av1 >= av2) {
            f = av1 / av2;
            df1 = data1.length;
            df2 = data2.length;
        } else {
            f = av2 / av1;
            df1 = data2.length;
            df2 = data1.length;
        }
        prob = 2.0 * betai(0.5 * df2, 0.5 * df1, df2 / (df2 + df1 * f));
        if (prob > 1.0) {
            prob = 2.0 - prob;
        }

        return new double[]{f, prob, av1, av2};
    }



    public static double calcRms(double[][] actual, double[][] fit) {
	double out = 0;
	for (int i = 0; i < actual.length; i++) {
	    out += (actual[i][1] - fit[i][1]) * (actual[i][1] - fit[i][1]);
	}
	out = Math.sqrt(out / actual.length);
	return out;
    }

    public static double reqJitter(double[][] actual, double rmsV) {
	double out = 0;
	for (int i = 0; i < actual.length; i++) {
	    out += actual[i][2] * actual[i][2];
	}
	out = out / actual.length;
	out = Math.sqrt(rmsV * rmsV - out);
	return out;
    }


    public static final double dist(double x1, double y1, double x2, double y2) {
        return Math.sqrt(Math.pow(x1 - x2, 2) + Math.pow(y1 - y2, 2));
    }
    
    public static final void rotate(double[] x, double[] X, double w, double W, double i) {   
        X[0] = (cos(w)*x[0]-sin(w)*x[1])*cos(W)-(cos(i)*(cos(w)*x[1]+sin(w)*x[0])-sin(i)*x[2])*sin(W);
        X[1] = (cos(w)*x[0]-sin(w)*x[1])*sin(W)+(cos(i)*(cos(w)*x[1]+sin(w)*x[0])-sin(i)*x[2])*cos(W);
        X[2] = cos(i)*x[2]+sin(i)*(cos(w)*x[1]+sin(w)*x[0]);
    }
    
    
}
