/*
 * FoldPlot.java
 *
 * Created on March 31, 2008, 1:47 PM
 */
package plotting;

import java.awt.*;
import java.awt.print.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.*;
import javax.imageio.ImageIO;
import javax.swing.*;
import numal.NumUtils;
import palette.RangeField;
import palette.inputParameters;
import structs.Range;
import utils.*;
import static java.lang.Math.*;

public class FoldPlot extends javax.swing.JComponent implements ActionListener, Printable, ComponentListener {

	int x1, x2, y1, y2;
	ArrayList<double[][]> plotData;
	ArrayList<double[][]> data;
	ArrayList<double[][]> curves;
	ArrayList<DataProp> props;
	ArrayList<ArrayList<double[]>> xyCurves;
	ArrayList<CurveProp> xyCurvesProps;
	ArrayList<double[]> xy;
	Comparator pointComparator;
	boolean printing = false;
	double ypad = 0.05;
	double[] xBounds;
	double[] yBounds;
	double[] xRange;
	double[] yRange;
	double hzoom = 1;
	double vzoom = 1;
	double hv = 1;
	double vv = 1;
	double xn = 10;
	double yn = 10;
	double xx1;
	double xx2;
	double yy1;
	double yy2;
	double aspectRatio = 0;
	boolean mouseIn;
	int xmouse;
	int ymouse;
	String xlabel;
	String ylabel;
	String warning;
	Font fontLabel;
	Font fontAxis;
	double foldPeriod;
	double customYScale;
	double customXScale;
	int highlight = 0;
	int xdivs = 5;
	int ydivs = 6;
	LayoutManager oldLayout;
	public static final int FT_NONE = 0;
	public static final int FT_CUSTOM = 1;
	public static final int FT_CONTROLLED = 2;
	int foldType = FT_NONE;
	double phase = 0.;
	PhaseWindow phaseDialog;
	public boolean hideData;
	public boolean hideTransits;
	public boolean hideLine;
	FoldPlotListener listener;
	boolean locked;
	double drWidth;
	double drHeight;
	double hpos;
	double vpos;
	double hs;
	double vs;
	double dx;
	double dy;
	Color background;
	Color boxColor;
	boolean symmetric = true;
	boolean showDate = false;
	boolean xlog = false;
	boolean ylog = false;
	boolean errorBars = true;
	FoldPlot synchronizeVertical;
	double beam = -1;
	ArrayList<Component> customMenu;
	JInternalFrame infoWindow;
	static public Integer FOLD = 0;
	static public Integer PRINT = 1;
	static public Integer SCALE = 2;
	static public Integer DATA = 3;
	static public Integer ZOOM = 4;
	private boolean zeroLine = true;
	ArrayList<Integer> caps;
	boolean doUpdateCurves = true;
	RenderingHints renderHints;
	public String plotClass;
	private BasicStroke boxStroke;
	private BasicStroke tickStroke;
	private BasicStroke pointStroke;
	private int smallTics = 5;
	private boolean intX;
	private boolean intY;
	private JFrame legendForm;
	private String curPos;
	private int[] annotationMouseXY;
	private double[] annotationXY;
	private JFrame anFrame;
	private RangeField rangeField;
	private ArrayList<LabelProp> labels;
	private double xshift = 0;
	private boolean niceTics = true;
    private int valueColumn = 1;

    private int brMargin = 10;
    
    private String noDataWarning = "No data selected.";
    private static ImageIcon warningIcon = new ImageIcon(FoldPlot.class.getResource("/plotting/warning.png")); 
    
    private boolean mouseDrag = false;
    private int mouseDragX = 0;
    private int mouseDragY = 0;
    private boolean draggable = true;
    
    private FoldPlotPainter painter;

    public FoldPlotPainter getPainter() {
        return painter;
    }

    public void setPainter(FoldPlotPainter painter) {
        this.painter = painter;
    }
            
    public boolean isDraggable() {
        return draggable;
    }

    public void setDraggable(boolean draggable) {
        this.draggable = draggable;
    }
    
	public FoldPlot() {
		init("default");
	}

	public FoldPlot(String pclass) {
		init(pclass);
	}

	private void init(String pclass) {
		plotClass = pclass;
		setOpaque(true);
        
        //setDoubleBuffered(true);
		plotData = new ArrayList<double[][]>();

		pointComparator = new Comparator() {

			public final int compare(final Object a, final Object b) {
				return signum(((double[]) a)[0] - ((double[]) b)[0]);

			}

			public final int signum(double diff) {
				if (diff > 0) {
					return 1;
				}
				if (diff < 0) {
					return -1;
				} else {
					return 0;
				}
			}
		};


		props = new ArrayList<DataProp>();


		xBounds = new double[2];
		yBounds = new double[2];

		initComponents();

		foldPeriod = -1;
		customYScale = -1;
		customXScale = -1;

		curves = new ArrayList<double[][]>();
		xyCurves = new ArrayList<ArrayList<double[]>>();
		xyCurvesProps = new ArrayList<CurveProp>();
		labels = new ArrayList<LabelProp>();

		caps = new ArrayList<Integer>();
		setCaps(FOLD, PRINT, SCALE, DATA, ZOOM);

		hideData = false;
		hideTransits = false;
		hideLine = false;
		xlabel = "";
		ylabel = "";
		locked = false;

		customMenu = new ArrayList<Component>();

		fontAxis = new Font("Helvetica", Font.PLAIN, 11);
		fontLabel = new Font("Helvetica", Font.BOLD, 13);
		boxStroke = new BasicStroke(2f);
		tickStroke = new BasicStroke(1f);
		pointStroke = new BasicStroke(1f, BasicStroke.CAP_BUTT, BasicStroke.CAP_SQUARE, 1f, new float[]{1f, 1f}, 0f);

		background = Color.WHITE;
		boxColor = Color.BLACK;

		renderHints =
				new RenderingHints(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		renderHints.put(RenderingHints.KEY_RENDERING,
				RenderingHints.VALUE_RENDER_QUALITY);

		renderHints.put(RenderingHints.KEY_INTERPOLATION,
				RenderingHints.VALUE_INTERPOLATION_BILINEAR);

		rangeField = new RangeField("", 4);
		rangeField.setVisible(false);
		rangeField.setToolTipText("Sets the xmin-xmax and ymin-ymax range; press ENTER to confirm.");
		rangeField.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				double[] fields = rangeField.getValues();
				setXRange(new double[]{fields[0], fields[1]});
				setYRange(new double[]{fields[2], fields[3]});
			}
		});

		setHZoom(1);
		setVZoom(1);
		addComponentListener(this);
        hSb.setVisible(false);
        vSb.setVisible(false);
		toolbar.setVisible(false);

	}

	public static FoldPlot histogram(double[] data, int bins, double min, double max) {
		FoldPlot hist = new FoldPlot("histogram");

		Double minv = min;
		Double maxv = max;

		if (min > max) {
			minv = null;
			maxv = null;
		}

		double[][] curve = Utils.bin(data, minv, maxv, bins, null, null);

		hist.setXRange(curve[0][0], curve[curve.length - 1][0]);
		hist.clearCurves();
		hist.addHistogram(curve);

		hist.setSymmetric(false);
		hist.updateCurves();
		hist.setWarning("");
		return hist;
	}

/*	public static PlotAggregator xyPlotWindow(double[][] data, String xlabel, String ylabel, PlotAggregator pa) {
                FoldPlot plot = null;
                if (pa == null) {
                    plot = new FoldPlot();
                    pa = new PlotAggregator(plot);
                    pa.setVisible(true);                    
                } else {
                    plot = pa.getPlotter();
                    plot.clear();
                }
                plot.setXLabel(xlabel);
                plot.setYLabel(ylabel);
                plot.setSymmetric(false);
                if (data != null)
                    plot.addData(data);
		return pa;
	}
    * 
    */
/*
        
        public static PlotAggregator xyCurveWindow(double[][] data, String xlabel, String ylabel, PlotAggregator pa) {
                FoldPlot plot = null;
                if (pa == null) {
                    plot = new FoldPlot();
                    pa = new PlotAggregator(plot);
                    pa.setVisible(true);                    
                } else {
                    plot = pa.getPlotter();
                    plot.clear();
                }
		
                plot.setXLabel(xlabel);
                plot.setYLabel(ylabel);
                plot.setSymmetric(false);
                if (data != null)
                    plot.addCurve(data);
                
		return pa;
	}
*/
/*        public static PlotAggregator colorPlotWindow(double[][] data, String xlabel, String ylabel, PlotAggregator pa) {
                FoldPlot plot = null;
                if (pa == null) {
                    plot = new FoldPlot();
                    pa = new PlotAggregator(plot);
                } else {
                    plot = pa.getPlotter();
                    pa.setVisible(true);                    
                    plot.clear();
                }
                plot.setXLabel(xlabel);
                plot.setYLabel(ylabel);
                plot.addData(data);
                plot.setSymmetric(false);
                double min = Utils.amin(data, 2);
                double max = Utils.amax(data, 2);
                
                for (int i = 0; i < data.length; i++) {
                    int cv = (int)((data[i][2] - min)/ (max - min) * 254.);
                    Color color = new Color(cv, cv, cv);
                    data[i][2] = 0.;
                    DataProp dp = plot.addData(new double[][] { data[i] });
                    dp.color = color;
                    dp.size = 30;
                };
                
                return pa;
        }
        
        */
	public void setXLog(boolean xl) {
		xlog = xl;
		doUpdateCurves = true;
		findBounds();
		repaint();
	}

	public boolean getXLog() {
		return xlog;
	}

	public void setYLog(boolean xl) {
		ylog = xl;
		doUpdateCurves = true;
		findBounds();
		repaint();
	}

	public boolean getYLog() {
		return ylog;
	}

	public void setErrorBars(boolean eb) {
		errorBars = eb;
		repaint();
	}

	public boolean getErrorBars() {
		return errorBars;
	}

	public boolean isIntX() {
		return intX;
	}

	public void setIntX(boolean intX) {
		this.intX = intX;
		findBounds();
		updateCurves();
		updateBuf();
		repaint();
	}

	public boolean isIntY() {
		return intY;
	}

	public void setIntY(boolean intY) {
		this.intY = intY;
		findBounds();
		updateCurves();
		updateBuf();
		repaint();
	}

	public void reduce(int what, int idx, int fac) {

		if (what == 0) {
			if (curves == null) {
				return;
			}
			if (idx >= curves.size()) {
				return;
			}

			double[][] curve = curves.get(idx);

			ArrayList<double[]> redcurve = new ArrayList<double[]>();
			for (int i = 0; i < curve.length; i += fac) {
				redcurve.add(curve[i]);
			}
			curves.set(idx, Utils.toMatrix(redcurve));

			findBounds();
			updateCurves();
			updateBuf();
			repaint();
		}

	}

	public void setToDefaultAppearance() {
		fontAxis = new Font("Helvetica", Font.PLAIN, 12);
		fontLabel = new Font("Helvetica", Font.BOLD, 14);

		background = Color.WHITE;
		boxColor = Color.BLACK;

		for (int i = 0; i < props.size(); i++) {
			props.get(i).size = 6;
		}

		for (int i = 0; i < xyCurvesProps.size(); i++) {
			xyCurvesProps.get(i).setStrokeWidth(1f);
		}

		aspectRatio = 0;
		hideLine = false;
		hideData = false;
		hideTransits = false;
		setHighlight(highlight);
		findBounds();
		updateCurves();
		repaint();
	}

	public void setAxisFont(Font f) {
		fontAxis = f;
	}

	public void setLabelFont(Font f) {
		fontLabel = f;
	}

	public Font getAxisFont() {
		return fontAxis;
	}

	public Font getLabelFont() {
		return fontLabel;
	}

	public void setCaps(Integer... gc) {
		caps.clear();

		for (int i = 0; i < gc.length; i++) {
			caps.add(gc[i]);
		}

	}


	public void setYTics(int tics) {
		ydivs = tics;
		doUpdateCurves = true;
		repaint();
	}

	public void setXTics(int tics) {
		xdivs = tics;
		doUpdateCurves = true;
		repaint();
	}

	public int getXTics() {
		return xdivs;
	}

	public int getYTics() {
		return ydivs;
	}

	public double getFoldPeriod() {
		return foldPeriod;
	}

	public ArrayList<Component> getCustomMenu() {
		return customMenu;
	}

	public ArrayList<DataProp> getDataProps() {
		return props;
	}

	public ArrayList<CurveProp> getCurvesProps() {
		return xyCurvesProps;
	}

	public void setDataProps(ArrayList<DataProp> pr) {
		props = pr;
	}

	public void setSynchronizeVertical(FoldPlot sv) {
		//synchronizeVertical = sv;
		//doUpdateCurves = true;
		//repaint();
	}

    public void setValueColumn(int c) {
        valueColumn = c;
        doUpdateCurves = true;
        findBounds();
        repaint();
    }
    
    public void setNoDataWarning(String str) {
        noDataWarning = str;
        repaint();
    }
    
	public void setFoldPeriod(double p) {
		if (p == foldPeriod) {
			return;
		}

		doUpdateCurves = true;
		findBounds();

		foldPeriod = p;

		if (foldPeriod < 0) {
			foldType = FT_NONE;
			hidePhaseDialog();
		}

		fold();
		findBounds();
		updateCurves();
		repaint();

		if (listener != null) {
			listener.foldUpdated();
		}

	}

	public void setFoldType(int type) {
		doUpdateCurves = true;
		if (type == FT_NONE) {
			setFoldPeriod(-1);
			hidePhaseDialog();
		} else if (type == FT_CUSTOM) {
			showPhaseDialog();
		} else if (type == FT_CONTROLLED) {
			if (phaseDialog != null) {
				hidePhaseDialog();
			}
		}
	}

	public LabelProp addLabel(String text, String position) {
		LabelProp l = new LabelProp(text, position);
		labels.add(l);

		findBounds();
		repaint();

		return l;
	}

	public LabelProp addLabel(String text, String position, boolean highlight) {
		LabelProp l = new LabelProp(text, position);
		labels.add(l);
		if (highlight) {
			l.setColor(UIUtils.getColor(labels.size() - 1));
		}

		findBounds();
		repaint();

		return l;
	}

	public LabelProp addLabel(String text, double x, double y) {
		LabelProp l = new LabelProp(text, x, y);
		labels.add(l);

		findBounds();
		repaint();

		return l;
	}

	public LabelProp addLabel(String text, double x, double y, boolean highlight) {
		LabelProp l = new LabelProp(text, x, y);
		labels.add(l);
		if (highlight) {
			l.setColor(UIUtils.getColor(labels.size() - 1));
		}

		findBounds();
		repaint();

		return l;
	}

    public JButton addButtonToToolbar(Icon icon, String tooltip) {
        JButton btn = new JButton(icon);
        btn.setBorder(null);
        btn.setBorderPainted(false);
        btn.setToolTipText(tooltip);
        toolbar.add(btn, toolbar.getComponentCount()-1);
        return btn;
    }
    
    
	public void clearLabels() {
		labels.clear();
		findBounds();
		repaint();
	}

	public void setCustomScale(double xscale, double yscale) {
		customXScale = xscale;
		customYScale = yscale;
		doUpdateCurves = true;
		findBounds();
		repaint();
	}

	public void setCustomYScale(double yscale) {
		customYScale = yscale;
		doUpdateCurves = true;
		findBounds();
		repaint();
	}

	public void setListener(FoldPlotListener fpl) {
		listener = fpl;
	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        vSb = new javax.swing.JScrollBar();
        hSb = new javax.swing.JScrollBar();
        toolbar = new javax.swing.JPanel();
        resetZoom = new javax.swing.JButton();
        filler1 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 32767));

        setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
        addMouseWheelListener(new java.awt.event.MouseWheelListener() {
            public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) {
                formMouseWheelMoved(evt);
            }
        });
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                formMousePressed(evt);
            }
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                formMouseReleased(evt);
            }
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                formMouseClicked(evt);
            }
            public void mouseExited(java.awt.event.MouseEvent evt) {
                formMouseExited(evt);
            }
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                formMouseEntered(evt);
            }
        });
        addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                formMouseMoved(evt);
            }
            public void mouseDragged(java.awt.event.MouseEvent evt) {
                formMouseDragged(evt);
            }
        });
        setLayout(new java.awt.BorderLayout());

        vSb.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
            public void adjustmentValueChanged(java.awt.event.AdjustmentEvent evt) {
                vSbAdjustmentValueChanged(evt);
            }
        });
        add(vSb, java.awt.BorderLayout.WEST);

        hSb.setOrientation(javax.swing.JScrollBar.HORIZONTAL);
        hSb.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
            public void adjustmentValueChanged(java.awt.event.AdjustmentEvent evt) {
                hSbAdjustmentValueChanged(evt);
            }
        });
        add(hSb, java.awt.BorderLayout.SOUTH);

        toolbar.setBackground(new java.awt.Color(240, 240, 240));
        toolbar.setOpaque(false);
        toolbar.setLayout(new javax.swing.BoxLayout(toolbar, javax.swing.BoxLayout.Y_AXIS));

        resetZoom.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/restorezoom.png"))); // NOI18N
        resetZoom.setToolTipText("Zoom to 100%");
        resetZoom.setBorder(javax.swing.BorderFactory.createEmptyBorder(3, 3, 3, 3));
        resetZoom.setBorderPainted(false);
        resetZoom.setFocusable(false);
        resetZoom.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetZoomActionPerformed(evt);
            }
        });
        toolbar.add(resetZoom);
        toolbar.add(filler1);

        add(toolbar, java.awt.BorderLayout.LINE_END);
    }// </editor-fold>//GEN-END:initComponents

    private void hSbAdjustmentValueChanged(java.awt.event.AdjustmentEvent evt) {//GEN-FIRST:event_hSbAdjustmentValueChanged
		doUpdateCurves = true;
		repaint();
    }//GEN-LAST:event_hSbAdjustmentValueChanged

    private void vSbAdjustmentValueChanged(java.awt.event.AdjustmentEvent evt) {//GEN-FIRST:event_vSbAdjustmentValueChanged
		doUpdateCurves = true;
		repaint();
    }//GEN-LAST:event_vSbAdjustmentValueChanged

    private void formMouseMoved(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseMoved
		xmouse = evt.getX();
		ymouse = evt.getY();

		mouseIn = ((xmouse >= x1 + xn) && (xmouse <= x2 - xn) && (ymouse >= y1 + yn) && (ymouse <= y2 - yn));
		if (mouseIn) {
			repaint();
		}
    }//GEN-LAST:event_formMouseMoved

    private void formMouseExited(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseExited
		mouseIn = false;
        if (evt.getX() >= getWidth() || evt.getY() >= getHeight() || evt.getX() <= 0 || evt.getY() <= 0)
            toolbar.setVisible(false);
		repaint();
    }//GEN-LAST:event_formMouseExited

	private JPopupMenu createGearPopupMenu() {
		JPopupMenu foldPop = new JPopupMenu();

		JMenuItem mi = new JMenuItem("");
		JCheckBoxMenuItem mi2;
		Font boldFont = new Font(mi.getFont().getFamily(), Font.BOLD, mi.getFont().getSize());

		if (caps.indexOf(FOLD) != -1) {

			mi = new JMenuItem("Folding");
			mi.setEnabled(false);
			mi.setFont(boldFont);
			foldPop.add(mi);

			mi = new JMenuItem("    Period/phase...");
			if (foldPeriod < 0) {
				mi.setEnabled(false);
			}
			mi.setActionCommand("phase");
			mi.addActionListener(this);
			foldPop.add(mi);
		}

		if (caps.indexOf(DATA) != -1) {

			mi = new JMenuItem("Coloring");
			mi.setEnabled(false);


			mi.setFont(boldFont);
			foldPop.add(mi);

			mi2 = new JCheckBoxMenuItem("    None");
			mi2.addActionListener(this);
			mi2.setState(highlight == 0);
			mi2.setActionCommand("nohighlight");
			foldPop.add(mi2);

			mi2 = new JCheckBoxMenuItem("    Datasets");
			mi2.addActionListener(this);
			mi2.setState(highlight == 1);
			mi2.setActionCommand("highlight");
			foldPop.add(mi2);

			mi = new JMenuItem("Show");
			mi.setEnabled(false);
			mi.setFont(boldFont);
			foldPop.add(mi);

			mi2 = new JCheckBoxMenuItem("    RV Data");
			mi2.addActionListener(this);
			mi2.setState(!this.hideData);
			mi2.setActionCommand("norv");
			foldPop.add(mi2);

			mi2 = new JCheckBoxMenuItem("    Transits");
			mi2.addActionListener(this);
			mi2.setState(!this.hideTransits);
			mi2.setActionCommand("notr");
			foldPop.add(mi2);

			mi2 = new JCheckBoxMenuItem("    Curve");
			mi2.addActionListener(this);
			mi2.setState(!this.hideLine);
			mi2.setActionCommand("noli");
			foldPop.add(mi2);

			mi2 = new JCheckBoxMenuItem("    Show calendar date");
			mi2.setState(showDate);
			mi2.setActionCommand("showdate");
			mi2.addActionListener(this);
			foldPop.add(mi2);


		}

		if (caps.indexOf(SCALE) != -1) {
			mi = new JMenuItem("Y Axis Scaling");
			mi.setEnabled(false);
			mi.setFont(boldFont);
			foldPop.add(mi);

			mi2 = new JCheckBoxMenuItem("    Autoscale");
			mi2.addActionListener(this);
			mi2.setState(customYScale < 0);
			mi2.setActionCommand("noscale");
			foldPop.add(mi2);

			mi2 = new JCheckBoxMenuItem("    10 m/s");
			mi2.addActionListener(this);
			mi2.setState(customYScale == 10);
			mi2.setActionCommand("scale_010");
			foldPop.add(mi2);

			mi2 = new JCheckBoxMenuItem("    50 m/s");
			mi2.addActionListener(this);
			mi2.setState(customYScale == 50);
			mi2.setActionCommand("scale_050");
			foldPop.add(mi2);

			mi2 = new JCheckBoxMenuItem("    100 m/s");
			mi2.addActionListener(this);
			mi2.setState(customYScale == 100);
			mi2.setActionCommand("scale_100");
			foldPop.add(mi2);

			mi2 = new JCheckBoxMenuItem("    Custom...");
			mi2.addActionListener(this);
			mi2.setState((customYScale > 0) && (customYScale != 100) && (customYScale != 50) && (customYScale != 10));
			mi2.setActionCommand("customscale");
			foldPop.add(mi2);
			foldPop.addSeparator();


			mi2 = new JCheckBoxMenuItem("    Toggle Y symmetry");
			mi2.addActionListener(this);
			mi2.setActionCommand("ysym");
			foldPop.add(mi2);
			foldPop.addSeparator();
		}

		if (caps.indexOf(PRINT) != -1) {
			mi = new JMenuItem("Print/PDF/Image...");
			mi.addActionListener(this);
			mi.setActionCommand("print");
			foldPop.add(mi);
			foldPop.addSeparator();

			mi = new JMenuItem("Export data...");
			mi.addActionListener(this);
			mi.setActionCommand("export");
			foldPop.add(mi);

			mi = new JMenuItem("Merge to...");
			mi.addActionListener(this);
			mi.setActionCommand("merge");
			foldPop.add(mi);
		}

		for (int i = 0; i < customMenu.size(); i++) {
			if (customMenu.get(i) != null) {
				foldPop.add(customMenu.get(i));
			}
		}

		return foldPop;
	}

private void formMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseClicked
}//GEN-LAST:event_formMouseClicked

private void formMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMousePressed
	if (!evt.isPopupTrigger()) {
		if (!evt.isShiftDown()) {
			return;
		}

		xmouse = evt.getX();
		ymouse = evt.getY();
	}
}//GEN-LAST:event_formMousePressed
	private boolean zoom = true;

    private void formMouseWheelMoved(java.awt.event.MouseWheelEvent evt) {//GEN-FIRST:event_formMouseWheelMoved
        if (! evt.isMetaDown()) {
            if (evt.isShiftDown()) {
                hSb.setValue(hSb.getValue() + evt.getWheelRotation());
            } else {
                vSb.setValue(vSb.getValue() + evt.getWheelRotation());
            }
        } else {
            if (evt.isShiftDown()) {
                setHZoom(Math.min(Math.max(hzoom + evt.getWheelRotation(), 1.), 50));
            } else {
                setVZoom(Math.min(Math.max(vzoom + evt.getWheelRotation(), 1.), 50));
            }
        }
    }//GEN-LAST:event_formMouseWheelMoved

    
    private void drawMouseDrag(Graphics2D g2) {
        g2.setColor(new Color(0, 0, 0, 150));
        g2.fillRect(xmouse, ymouse, mouseDragX - xmouse, mouseDragY - ymouse);
    }
    
    private void formMouseDragged(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseDragged
        if (!draggable) {
            mouseDrag = false;
            return;
        }
            
        mouseDrag = mouseIn;
        repaint();
        mouseDragX = evt.getX();
        mouseDragY = evt.getY();
    }//GEN-LAST:event_formMouseDragged

    private void formMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseReleased
        if (mouseDrag) {
            double dx = (double)Math.abs(mouseDragX - xmouse)/(x2-x1) * (xBounds[1] - xBounds[0]);
            double dy = (double)Math.abs(mouseDragY - ymouse)/(y2-y1) * (yBounds[1] - yBounds[0]);
            
            double x = (double)Math.min(xmouse - x1, mouseDragX) /(x2-x1) * (xBounds[1] - xBounds[0]) + xBounds[0];
            double y = (double)Math.min(ymouse - y1, mouseDragY) /(y2-y1) * (yBounds[1] - yBounds[0]) + yBounds[0];
            
            setXRange(x, x+dx);
            setYRange(y, y+dy);
            System.out.println(x + " " + dx + " " + y + " " + dy + " " + xBounds[1] + " " + xBounds[0] + " " + x1 + " " + x2);
        }
        mouseDrag = false;
        
    }//GEN-LAST:event_formMouseReleased

    private void formMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseEntered
        toolbar.setVisible(true);
    }//GEN-LAST:event_formMouseEntered

    private void resetZoomActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resetZoomActionPerformed
        setXRange(null);
        setYRange(null);
    }//GEN-LAST:event_resetZoomActionPerformed

	public void setShowDate(boolean show) {
		showDate = show;
		repaint();
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();

		if (command.equals("nohighlight")) {
			setHighlight(0);

		} else if (command.equals("ysym")) {
			setSymmetric(!getSymmetric());
		} else if (command.equals("noscale")) {
			customYScale = -1;
			doUpdateCurves = true;
			findBounds();
			repaint();
		} else if (command.startsWith("scale_")) {
			double scale = Double.parseDouble(command.substring(command.length() - 3));
			customYScale = scale;
			doUpdateCurves = true;
			findBounds();
			repaint();
		} else if (command.equals("print")) {
			doUpdateCurves = true;
			//doPrint();
		} else if (command.equals("phase")) {
			doUpdateCurves = true;
			showPhaseDialog();
		} else if (command.equals("norv")) {
			hideData = !hideData;
			doUpdateCurves = true;
			repaint();
		} else if (command.equals("notr")) {
			hideTransits = !hideTransits;
			doUpdateCurves = true;
			repaint();
		} else if (command.equals("noli")) {
			hideLine = !hideLine;
			doUpdateCurves = true;
			repaint();
		} else if (command.equals("showdate")) {
			showDate = !showDate;
			doUpdateCurves = true;
			repaint();
		} else if (command.equals("showlegend")) {
			showLegend();
		} else if (command.equals("customscale")) {

			try {
				String ret = JOptionPane.showInputDialog("Enter Y scale: ");
				if (ret.equals("")) {
					return;
				}

				double yscale = Double.parseDouble(ret);
				if (yscale <= 0) {
					throw (new Exception());
				}
				customYScale = yscale;
				findBounds();
				repaint();
			} catch (Exception e2) {
				JOptionPane.showMessageDialog(this, "Error parsing number.");
			}

		} else if (command.equals("highlight")) {
			setHighlight(1);

			showLegend();
		} else if (command.equals("export")) {

			inputParameters ip = new inputParameters("Export options", true);
			ip.addRow("Output file: ", null, File.class);
			ip.addRow("Numerical format: ", "%15.8e ", String.class);
			ip.addRow("Comment character: ", "#", String.class);
			ip.addRow("Export datapoints", true, Boolean.class);
			ip.addRow("Export curves", false, Boolean.class);
			ip.addRow("Export transits", false, Boolean.class);

			ip.setVisible(true);
			ArrayList<Object> values = ip.getValues();

			if (values != null) {
				String fmt = (String) values.get(1);
				if (fmt.indexOf(" ") == -1) {
					fmt = fmt + " ";
				}

				String com = (String) values.get(2);

				try {
					PrintWriter pw = new PrintWriter((File) values.get(0));

					if ((curves != null) && ((Boolean) values.get(4))) {
						for (int i = 0; i < curves.size(); i++) {
							double[][] curve = curves.get(i);

							pw.println(com + " Curve " + i + " [" + this.xyCurvesProps.get(i).getName() + "] ");
							pw.println(com + " " + curve.length + " points");

							for (int j = 0; j < curve.length; j++) {
								for (int k = 0; k < curve[j].length; k++) {
									pw.format(fmt, curve[j][k]);
								}
								pw.println();
							}

							pw.println();
							pw.println();

						}
					}

					if ((data != null) && ((Boolean) values.get(3))) {
						for (int i = 0; i < data.size(); i++) {

							double[][] curve = data.get(i);


							pw.println(com + " Dataset " + i + " [" + this.props.get(i).name + "] ");
							pw.println(com + " " + curve.length + " points");

							for (int j = 0; j < curve.length; j++) {
								for (int k = 0; k < curve[j].length; k++) {
									pw.format(fmt, curve[j][k]);
								}
								pw.println();
							}

							pw.println();
							pw.println();

						}
					}

					pw.close();

				} catch (Exception ex) {
					JOptionPane.showMessageDialog(this, "Error saving data.");
				}

			}
		} 
        
        /*else if (command.equals("merge")) {
			System.gc();
			ArrayList<String> choices = new ArrayList<String>();
			choices.add("New window...");

			ArrayList<WeakReference<PlotAggregator>> list = PlotAggregator.all;
			for (int i = 0; i < list.size(); i++) {
				if (list.get(i).get() != null) {
					choices.add(list.get(i).get().getTitle());
				}
			}


			int choice = JOptionPane.showOptionDialog(this, "Choose the destination of the data:", "Merge dialog",
					0, 0, null, choices.toArray(), choices.get(0));

			if (choice == JOptionPane.CLOSED_OPTION) {
				return;
			} else if (choice == 0) {
				doPrint();
			} else if (choice == 1) {
				choice = choice - 1;
				int j = 0;
				for (int i = 0; i < list.size(); i++) {
					if (list.get(i).get() != null) {
						if (j == choice) {
							list.get(i).get().mergePlotter(this);
							list.get(i).get().toFront();
							break;
						}
						j++;
					}
				}
			}
		}*/
	}

	public void hidePhaseDialog() {

		if (phaseDialog != null) {
			phaseDialog.dispose();
		}

		phaseDialog = null;
	}

	public void showLegend() {
		if (legendForm == null) {
			legendForm = new JFrame("Legend");
		} else {
			legendForm.getContentPane().removeAll();
		}

		legendForm.setLocationRelativeTo(this);
		JPanel pan = new JPanel();
		pan.setLayout(new BoxLayout(pan, BoxLayout.Y_AXIS));
		legendForm.add(pan);
		doUpdateCurves = true;

		for (int i = 0; i < props.size(); i++) {
			String rgb = Integer.toHexString(props.get(i).color.getRGB());
			rgb = rgb.substring(2, rgb.length());

			JLabel lab = new JLabel("<html><font size=+2 color=\"#" + rgb + "\">&#8226;</font> " + props.get(i).name);
			pan.add(lab);
			legendForm.setSize(200, 60 + legendForm.getHeight() + lab.getHeight());
		}

		for (int i = 0; i < xyCurvesProps.size(); i++) {
			Color c = xyCurvesProps.get(i).getColor();
			String rgb = Integer.toHexString(c.getRGB());
			rgb = rgb.substring(2, rgb.length());

			JLabel lab = new JLabel("<html><font color=\"#" + rgb + "\"><font size=+1>-</font> " + xyCurvesProps.get(i).getName());
			pan.add(lab);
			legendForm.setSize(200, 60 + legendForm.getHeight() + lab.getHeight());
		}

		legendForm.setAlwaysOnTop(true);
		legendForm.setVisible(true);

	}

	public void showPhaseDialog() {
		showPhaseDialog("");
	}

	public void addFromFile(String fn) throws FileNotFoundException, IOException {
		BufferedReader br = new BufferedReader(new FileReader(fn));
		String line = br.readLine();
		while (line != null) {
			if (line.startsWith("# Curve")) {
				// curve
				String name = line.substring(line.indexOf('['), line.lastIndexOf(']'));

				line = br.readLine().replaceAll("# ", "").replaceAll(" points", "").trim();
				int n = Integer.valueOf(line);
				double[][] buf = FileUtils.readTable(br, n);
				addCurve(buf, name);
			} else if (line.startsWith("# Dataset")) {
				String name = line.substring(line.indexOf('['), line.lastIndexOf(']'));

				line = br.readLine().replaceAll("# ", "").replaceAll(" points", "").trim();
				int n = Integer.valueOf(line);
				double[][] buf = FileUtils.readTable(br, n);
				addData(buf, name);
			}

			line = br.readLine();
		}

	}

	public void setInfoWindow(String text, String title) {
		if (title == null) {
			if (infoWindow != null) {
				remove(infoWindow);
				infoWindow.dispose();
				infoWindow = null;
			}
			return;
		}



		if (infoWindow == null) {
			infoWindow = new JInternalFrame(title, true, false, false, false);
			add(infoWindow);
			infoWindow.getContentPane().setLayout(new BoxLayout(infoWindow.getContentPane(), BoxLayout.LINE_AXIS));
			JLabel labTitle = new JLabel(text);
			infoWindow.getContentPane().add(labTitle);

			infoWindow.setSize(160, 100);
			infoWindow.setLocation(getWidth() - 180, 10);

			infoWindow.setVisible(true);
			infoWindow.pack();
		} else {
			((JLabel) infoWindow.getContentPane().getComponent(0)).setText(text);
		}

		infoWindow.setTitle(title);
	}

	public void showPhaseDialog(String title) {

		if (phaseDialog == null) {
			phaseDialog = new PhaseWindow(null, false);
			phaseDialog.setListener(this);
			phaseDialog.setPeriod(foldPeriod);
			phaseDialog.setLocationByPlatform(true);
			phaseDialog.setPhase(phase);
			phaseDialog.setVisible(true);
			phaseDialog.setAlwaysOnTop(true);
			phaseDialog.setSize(300, 150);
		}

		if (foldType == FT_CONTROLLED) {
			phaseDialog.setPeriodVisible(false);
		} else {
			phaseDialog.setPeriodVisible(true);
		}

		if (!title.equals("")) {
			phaseDialog.setTitle("Phase window - " + title);
		}
	}

	public void setHighlight(int high) {
		highlight = high;

		if (highlight == 0) {
			for (int i = 0; i < props.size(); i++) {
				props.get(i).color = Color.BLACK;
			}
		} else if (highlight == 1) {
            
            
			for (int i = 0; i < props.size(); i++) {
				props.get(i).color = UIUtils.getColor(i);
			}
		} else if (highlight == 2) {
                    	for (int i = 0; i < xyCurvesProps.size(); i++) {
				xyCurvesProps.get(i).setColor(UIUtils.getColor(i));
			}
        }
		doUpdateCurves = true;
		repaint();
	}


	public void setXLabel(String xl) {
		xlabel = xl;
		repaint();
	}

	public void setYLabel(String yl) {
		ylabel = yl;
		repaint();
	}

	public String getXLabel() {
		return xlabel;
	}

	public String getYLabel() {
		return ylabel;
	}

	public void beginUpdate() {
		locked = true;
	}

	public boolean isUpdating() {
		return locked;
	}

	public void endUpdate() {
		locked = false;
		doUpdateCurves = true;
		repaint();
	}

	public void setHZoom(double val) {
		hzoom = val;
		hSb.setMinimum(0);
		hSb.setMaximum(100);

		if (hSb.getValue() + (int) (100 / val) > 100) {
			hSb.setValue(100 - (int) (100 / val));
		}


		hSb.setVisibleAmount((int) (100 / val));

		hv = 100. / val;
		doUpdateCurves = true;
		repaint();
	}

	public void setVZoom(double val) {
		vzoom = val;

		vSb.setMinimum(0);
		vSb.setMaximum(100);

		if (symmetric) {
			vSb.setValue(50 - (int) (100. / val / 2));
			vSb.setVisibleAmount((int) (100 / val));
			vSb.setValue(50 - (int) (100. / val / 2));
		} else {
			vSb.setValue(50 - (int) (100. / val / 2));
			vSb.setVisibleAmount((int) (100 / val));
			vSb.setValue(50 - (int) (100. / val / 2));
		}

		doUpdateCurves = true;
		repaint();
	}

	public int getCurvesSize() {
		return curves.size();
	}

	public void clearData() {
		plotData.clear();
		trend = 0;
		props.clear();
		doUpdateCurves = true;
		updateBuf();
		repaint();
	}


	public void setAspectRatio(double ar) {
		aspectRatio = ar;
	}

	public double getAspectRatio() {
		return aspectRatio;
	}

	public DataProp getDataProperty(String name) {
		for (int i = 0; i < props.size(); i++) {
			if (props.get(i).name.equals(name)) {
				return props.get(i);
			}
		}
		return null;
	}

	public void setVShift(String name, double vshift) {
		DataProp prop = getDataProperty(name);
		if (prop != null) {
			prop.vshift = vshift;

			fold();
			doUpdateCurves = true;
			findBounds();
			repaint();
		}
	}

	public void setShift(double xshift) {
		this.xshift = xshift;
		fold();
		findBounds();
		repaint();
	}

	public double getShift() {
		return xshift;
	}

	public double getPhase() {
		return phase;
	}

	public void setPhase(double ph) {
		phase = ph;
		if (listener != null) {
			listener.phaseUpdated();
		}
		doUpdateCurves = true;
		fold();
		findBounds();
		repaint();
	}

	public void setWarning(String warn) {
		warning = warn;
	}

	public FoldPlot clone() {
		FoldPlot plot = new FoldPlot(plotClass);

		if (data != null) {
			for (int i = 0; i < data.size(); i++) {
				plot.addData(Utils.copy(plotData.get(i)), new DataProp(props.get(i)));
			}
		}

		if (curves != null) {
			for (int i = 0; i < curves.size(); i++) {
				plot.addCurve(Utils.copy(curves.get(i)), new CurveProp(xyCurvesProps.get(i)));
			}
		}

		if (labels != null) {
			for (int i = 0; i < labels.size(); i++) {
				plot.addLabel(labels.get(i).getText(), labels.get(i).getPosition());
			}
		}

		plot.setXLabel(xlabel);
		plot.setYLabel(ylabel);
		plot.setXRange(xRange);
		plot.setYRange(yRange);
		plot.setSymmetric(symmetric);
		plot.setFoldType(foldType);
		plot.setFoldPeriod(foldPeriod);
		plot.setZeroLine(zeroLine);
		plot.setShift(xshift);
		plot.setXLog(xlog);
		plot.updateCurves();
		return plot;
	}

	public void configure() {
	}

	public void doPDF(final String fn) {
		UIUtils.runInEDT(new Runnable() {

			public void run() {
				try {
					printing = true;
					//createPDF(fn);
					printing = false;
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}, true);
	}

	public void doFile(final String fn, final String type, final int width) {
		UIUtils.runInEDT(new Runnable() {

			public void run() {
				try {
					printing = true;
					double w = (int) width;

					double h = (w / (double) getWidth() * (double) getHeight());
					if (aspectRatio != 0) {
						h = w / aspectRatio;
					}

					BufferedImage currentImage = new BufferedImage((int) w, (int) h, BufferedImage.TYPE_INT_ARGB);
					mouseIn = false;
					doUpdateCurves = true;
					findBounds();
					updateCurves();
					paintComponent(currentImage.createGraphics(), new Dimension((int) w, (int) h));
					ImageIO.write(currentImage, type, new File(fn));
					printing = false;
					doUpdateCurves = true;
					repaint();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}, true);
	}
/*
	private void createPDF(String fn) throws IOException {
		setDoubleBuffered(false);
		float w = PageSize.A4.getWidth();
		double thisAspectRatio = aspectRatio;
		if (aspectRatio == 0.) {
			thisAspectRatio = (double) getWidth() / (double) getHeight();
		}

		float h = (float) (w / thisAspectRatio);


		Document document = new Document(new com.lowagie.text.Rectangle(0, 0, w, h), 0, 0, 0, 0);

		try {
			PdfWriter writer = PdfWriter.getInstance(document, new FileOutputStream(fn));
			document.open();
			PdfContentByte cb = writer.getDirectContent();

			Graphics2D g2 = cb.createGraphicsShapes(w, h);

			mousein = false;
			doUpdateCurves = true;
			findBounds();
			updateCurves();

			paintComponent((Graphics) g2, new Dimension((int) w, (int) h));
			g2.dispose();
			cb.sanityCheck();
			document.close();
		} catch (Exception e) {
			e.printStackTrace();
			throw new IOException(e.toString());
		}

		doUpdateCurves = true;
		setDoubleBuffered(true);
	}

	public void doPrint() {
		PlotAggregator pa = new PlotAggregator(clone());
		pa.setTitle(pa.getTitle() + " - Print/Export preview");
		pa.setVisible(true);
	}
*/
	public void startPrint() {
		try {
			PrinterJob job = PrinterJob.getPrinterJob();
			job.setPrintable(this);
			doUpdateCurves = true;
			mouseIn = false;
			boolean doPrint = job.printDialog();
			if (doPrint) {
				job.print();
			}

			doUpdateCurves = true;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public int print(Graphics g, PageFormat pf, int page) throws PrinterException {
		if (page > 0) {
			return NO_SUCH_PAGE;
		}

		setDoubleBuffered(false);
		double thisAspectRatio = aspectRatio;

		if (aspectRatio == 0.) {
			thisAspectRatio = (double) getWidth() / (double) getHeight();
		}

		Graphics2D g2d = (Graphics2D) g;

		g2d.translate(pf.getImageableX(), pf.getImageableY());
		System.out.println((int) pf.getImageableWidth() + " " + (int) (pf.getImageableWidth() / thisAspectRatio));
		findBounds();
		updateCurves();
		paintComponent(g, new Dimension((int) pf.getImageableWidth(), (int) (pf.getImageableWidth() / thisAspectRatio)));
		setDoubleBuffered(true);
		return PAGE_EXISTS;
	}

	public int getDataSize() {
		return data.size();
	}

	public double[][] getData(String name) {
		if (props == null) {
			return null;
		}
		if (plotData == null) {
			return null;
		}

		for (int i = 0; i < props.size(); i++) {
			if (props.get(i).name.equals(name)) {
				return plotData.get(i);
			}
		}

		return null;
	}

	public double[][] getData(int data) {
		return plotData.get(data);
	}

	public final double[][] getCurve(int curve) {
		return curves.get(curve);
	}

	public final double[][] getCurve(String nam) {
		if (xyCurvesProps == null) {
			return null;
		} else {
			for (int i = 0; i < xyCurvesProps.size(); i++) {
				if ((xyCurvesProps.get(i) != null) && (xyCurvesProps.get(i).getName().equals(nam))) {
					return curves.get(i);
				}
			}
		}

		return null;
	}

	public double[][] copyData(String name) {
		for (int i = 0; i < props.size(); i++) {
			if (props.get(i).name.equals(name)) {

				double[][] source = plotData.get(i);
				if (source == null) {
					return null;
				}

				double[][] copy = new double[source.length][source[0].length];

				for (int j = 0; j < source.length; j++) {
					for (int k = 0; k < source[i].length; k++) {
						copy[j][k] = source[j][k];
					}
				}

				return copy;
			}
		}


		return null;
	}

	public DataProp addData(ArrayList<double[]> dataPoints, String name) {
		if ((dataPoints == null) || (dataPoints.size() == 0)) {
			return null;
		}

		double[][] dataP = new double[dataPoints.size()][dataPoints.get(0).length];
		for (int i = 0; i < dataPoints.size(); i++) {
			for (int j = 0; j < dataPoints.get(i).length; j++) {
				dataP[i][j] = dataPoints.get(i)[j];
			}
		}

		DataProp prop = addData(dataP, name);
		doUpdateCurves = true;
		return prop;
	}

	public DataProp addData(double[][] dataPoints) {
		return addData(dataPoints, "");
	}

	public DataProp addData(double[][] dataPoints, String name) {
		try {
            if (dataPoints == null)
                return null;
            
			plotData.add(dataPoints);

			DataProp prop = new DataProp(name);
			props.add(prop);

			updateBuf();

			findBounds();
			mouseIn = false;

			fold();
			doUpdateCurves = true;
			findBounds();
			repaint();
			return prop;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}
    
    public void addData(double[][] dataPoints, String name, int column) {
		try {
            props.clear();
            
			plotData.add(dataPoints);

            int maxSets = 0;
            for (int i = 0; i < dataPoints.length; i++)
                maxSets = Math.max(maxSets, (int) dataPoints[i][column]);
            
            for (int i = 0; i <= maxSets; i++) {
                DataProp prop = new DataProp(name + "_" + i);
                props.add(prop);
            }
            
			updateBuf();

			findBounds();
			mouseIn = false;

			fold();
			doUpdateCurves = true;
			findBounds();
			repaint();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public DataProp addData(double[][] dataPoints, DataProp prop) {
		plotData.add(dataPoints);
		props.add(prop);

		updateBuf();

		findBounds();
		mouseIn = false;

		fold();
		doUpdateCurves = true;
		findBounds();
		repaint();

		return prop;
	}

	public CurveProp addEllipse(double[] pars) {
		if (pars == null) {
			return null;
		}

		int points = 1000;
		double extra = 0.01;
		double tstep = (2 * Math.PI + extra) / (double) points;
		double[][] curve = new double[points][2];

		double xc = pars[0];
		double yc = pars[1];
		double a = pars[2];
		double b = pars[3];
		double phi = pars[4];
		int i = 0;
		for (double t = 0; t < 2 * Math.PI + extra; t += tstep) {
			double x = a * cos(t);
			double y = b * sin(t);
			curve[i][0] = xc + x * cos(phi) - y * sin(phi);
			curve[i][1] = yc + x * sin(phi) + y * cos(phi);
			i++;
		}

		return addCurve(curve);
	}

	public CurveProp addCurve(double[][] curve) {
		return addCurve(curve, null, null, null, 1f, CurveProp.SOLID);
	}

	public CurveProp addCurve(double[][] curve, String name) {
		return addCurve(curve, name, null, null, 1f, CurveProp.SOLID);
	}

	public CurveProp addCurve(double[][] curve, String name, float width) {
		return addCurve(curve, name, null, null, width, CurveProp.SOLID);
	}

	public CurveProp addCurve(double[][] curve, String name, Color color) {
		return addCurve(curve, name, color, null, 1f, CurveProp.SOLID);
	}

	public CurveProp addCurve(double[][] curve, String name, Color color, Color cycleTo, float strokeWidth, int strokeType) {
		return addCurve(curve, new CurveProp(name, color, cycleTo, strokeWidth, strokeType));
	}

	public CurveProp addCurve(double[][] curve, CurveProp props) {
                curve = Utils.copy(curve);
		for (int i = 0; i < curve.length; i++) {
			curve[i][0] -= xshift;
                        if (xlog)
                            curve[i][0] = Math.log10(curve[i][0]);
		}

		curves.add(curve);

		xyCurvesProps.add(props);
		doUpdateCurves = true;

		updateBuf();

		findBounds();
		resetZoom();
		mouseIn = false;

		fold();
		doUpdateCurves = true;
		findBounds();
		repaint();

		return props;
	}

	private void resetZoom() {
		//setHZoom(1);
		//setVZoom(1);
	}

    public void reset() {
        clear();
        setXRange(null);
        setYRange(null);
    }
    
	public CurveProp addHistogram(double[][] curve) {
		return addHistogram(curve, null, null, null, 1f, CurveProp.SOLID);
	}

	public CurveProp addHistogram(double[][] curve, String name, Color color, Color cycleTo, float strokeWidth, int strokeType) {

		double[][] curve2 = new double[curve.length * 2][3];
		double bind = curve[1][0] - curve[0][0];

		for (int i = 0; i < curve.length; i++) {
			curve2[i * 2][0] = curve[i][0];
			curve2[i * 2][1] = curve[i][1];
			curve2[i * 2][2] = curve[i][2];

			curve2[i * 2 + 1][0] = curve[i][0] + bind;
			curve2[i * 2 + 1][1] = curve[i][1];
			curve2[i * 2 + 1][2] = curve[i][2];
		}



		return addCurve(curve2, name, color, cycleTo, strokeWidth, strokeType);

	}

	public void clear() {
		clearCurves();
		clearData();
		clearLabels();
	}

	public void clearCurves() {
		curves.clear();
		xyCurves.clear();
		xyCurvesProps.clear();
		beam = -1;
		doUpdateCurves = true;
		repaint();
	}

	public void updateCurves() {
		xyCurves.clear();

		for (int i = 0; i < curves.size(); i++) {
			xyCurves.add(getCurve(i, curves.get(i), xyCurvesProps.get(i)));
		}
	}

	public void updateBuf() {
        
		// avoid unnecessary allocation for data
		if (data == null) {
			data = new ArrayList<double[][]>();
		}
		data.clear();

        int maxNumProps = 0;
        
		for (int i = 0; i < plotData.size(); i++) {
			double[][] ds = new double[plotData.get(i).length][4];
			for (int j = 0; j < plotData.get(i).length; j++) {
				ds[j][0] = plotData.get(i)[j][0];
				ds[j][1] = plotData.get(i)[j][valueColumn];
				if (plotData.get(i)[j].length > 2) {
					ds[j][2] = plotData.get(i)[j][2];
				}
                if (plotData.get(i)[j].length >= 9) {
                    ds[j][3] = plotData.get(i)[j][9];
                } else
                    ds[j][3] = -1;
			}
            
            
			data.add(ds);
        }
        
        if (data.size() > 0)
            System.out.println(this.plotClass + " " + data.size() + " " + data.get(0)[0][1]);
	}

	public void setProperty(String property, String value) {
		property = property.trim();
		value = value.trim();
		if (property.equals("XLABEL")) {
			setXLabel(value);
		} else if (property.equals("YLABEL")) {
			setYLabel(value);
		} else if (property.equals("FONTNAME")) {
			fontAxis = new Font(value, Font.PLAIN, fontAxis.getSize());
			fontLabel = new Font(value, Font.BOLD, fontLabel.getSize());
		} else if (property.equals("XTICS")) {
			setXTics(Integer.valueOf(value));
		} else if (property.equals("YTICS")) {
			setYTics(Integer.valueOf(value));
		} else if (property.equals("AXISFONT")) {
			int fontAxisSize = Integer.valueOf(value);
			fontAxis = new Font(fontAxis.getFontName(), Font.PLAIN, fontAxisSize);
		} else if (property.equals("LABELFONT")) {
			int labelSize = Integer.valueOf(value);
			fontLabel = new Font(fontLabel.getFontName(), Font.BOLD, labelSize);
		} else if (property.equals("XRANGE")) {
			setXRange(Range.valueOf(value));
		} else if (property.equals("YRANGE")) {
			setYRange(Range.valueOf(value));
		} else if (property.equals("ASPECTRATIO")) {
			setAspectRatio(Double.valueOf(value));
		} else if (property.equals("BACKGROUND")) {
			setBackground(Color.decode("#" + value));
		} else if (property.equals("BOXCOLOR")) {
			setBoxColor(Color.decode("#" + value));
		} else if (property.equals("SYMMETRICY")) {
			setSymmetric(Boolean.valueOf(value));
		} else if (property.equals("ZEROLINE")) {
			setZeroLine(Boolean.valueOf(value));
		} else if (property.equals("XLOG")) {
			setXLog(Boolean.valueOf(value));
		} else if (property.equals("SHIFT")) {
			setShift(Double.valueOf(value));
		}

		if (data != null) {
			for (int i = 0; i < data.size(); i++) {
				String name = "DATA" + i;

				if (property.equals(name + ".COLOR")) {
					props.get(i).color = Color.decode("#" + value);
				}

				if (property.equals(name + ".SIZE")) {
					props.get(i).size = Integer.valueOf(value);
				}

				System.out.println(this.getClassString());

			}
		}

		if (xyCurvesProps != null) {
			for (int i = 0; i < xyCurvesProps.size(); i++) {
				String name = "CURVE" + i;
				CurveProp cp = xyCurvesProps.get(i);

				if (property.equals(name + ".COLOR")) {
					cp.setColor(Color.decode("#" + value));
				} else if (property.equals(name + ".WIDTH")) {
					cp.setStrokeWidth(Float.valueOf(value));
				} else if (property.equals(name + ".STROKE")) {
					cp.setStrokeType(value);
				} else if (property.equals(name + ".YSCALE")) {
					cp.setScaling(Double.valueOf(value));
				}

			}
		}

	}

	public String getClassString() {
		return plotClass;
	}

	public void setClassString(String pc) {
		plotClass = pc;
	}

	public void readParameters(String fn) throws FileNotFoundException, IOException {
		readParameters(new File(fn));
	}

	public void readParameters(File f) throws FileNotFoundException, IOException {
		if (!f.exists()) {
			return;
		}
		System.out.println(f.getName());
		BufferedReader br = new BufferedReader(new FileReader(f));
		String line;

		beginUpdate();
		HashMap<String, String> map = new HashMap<String, String>();
		while ((line = br.readLine()) != null) {
			line = line.trim();
			String value = br.readLine().trim();
			map.put(line, value);
		}
		br.close();
		try {
			for (String key : map.keySet()) {
				setProperty(key, map.get(key));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		endUpdate();
		repaint();
	}
	private double trend;

	public void setTrend(double trend) {
		this.trend = trend;
		updateBuf();
		fold();
		updateCurves();
		repaint();
	}

	public double getTrend() {
		return trend;
	}

	public void fold() {
		double tmin = 1e10;

		for (int i = 0; i < plotData.size(); i++) {
			for (int j = 0; j < plotData.get(i).length; j++) {
				tmin = Math.min(tmin, plotData.get(i)[j][0]);
			}
		}
		if (foldPeriod < 0) {
			for (int i = 0; i < plotData.size(); i++) {
				for (int j = 0; j < plotData.get(i).length; j++) {
					double time = plotData.get(i)[j][0];
					data.get(i)[j][0] = time - xshift;
					data.get(i)[j][1] = plotData.get(i)[j][valueColumn] + props.get(i).vshift
							- trend * (time - tmin);
				}
			}
		} else {

			double minT = min(plotData);
			double maxT = max(plotData);

			for (int i = 0; i < plotData.size(); i++) {

				for (int j = 0; j < plotData.get(i).length; j++) {
					double Tc = phase * (maxT - minT) + minT;

					double t = (plotData.get(i)[j][0] - Tc) % foldPeriod;
					if (t < -0.) {
						t += foldPeriod;
					} else if (t > foldPeriod) {
						t -= foldPeriod;
					}
					double time = plotData.get(i)[j][0];
					data.get(i)[j][0] = t / foldPeriod * 100.;
					data.get(i)[j][1] = plotData.get(i)[j][valueColumn] + props.get(i).vshift
							- trend * (time - tmin);
				}

			}
		}

	}


	@Override
	public void setBackground(Color bg) {
		background = bg;
	}

	@Override
	public Color getBackground() {
		return background;
	}

	public void setBoxColor(Color bg) {
		boxColor = bg;
	}

	public Color getBoxColor() {
		return boxColor;
	}

	@Override
	public void paintComponent(Graphics g) {
		paintComponent(g, null);
	}

	public void paintComponent(Graphics g, Dimension d) {
		if (locked == true) {
			return;
		}

		if (doUpdateCurves) {
			updateCurves();
		}

		Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHints(renderHints);
		super.paintComponent(g);


		if (d == null) {
			d = getSize();
		}

		if (aspectRatio != 0.) {
			d = new Dimension(d.width, (int) (d.width / aspectRatio));
		}

		g2.setColor(background);
		g2.fillRect(0, 0, d.width, d.height);


		
		g2.setFont(fontAxis);

		
		if (!printing) {
                        x1 = (int) (g2.getFontMetrics().stringWidth("00000000000") + 15);
			x2 = (int) (d.width - 3 * brMargin - 10);
		} else {
			x2 = (int) d.width - (int) (0.05 * d.width);
                        x1 = (int) (g2.getFontMetrics().stringWidth("0000000000"));
		}

		g2.setFont(fontLabel);
		y1 = (int) 5;
		if (!printing) {
			y2 = (int) (d.height - brMargin - g2.getFontMetrics().getHeight() * 2);
		} else {
			y2 = (int) (d.height - g2.getFontMetrics().getHeight() * 3);
		}

		g2.setFont(fontAxis);
        if (locked == true) {
			return;
		}

		drWidth = (x2 - x1) - 2 * xn;
		drHeight = (y2 - y1) - 2 * yn;

		if (((data == null) || (data.size() == 0)) && ((curves == null) || (curves.size() == 0))) {
			drawWarning(g2, noDataWarning);
			return;
		}

		if ((xBounds == null) || (yBounds == null)) {
			findBounds();
		}


		hpos = (double) hSb.getValue();
		vpos = (double) vSb.getValue();
		hs = (double) hSb.getVisibleAmount();
		vs = (double) vSb.getVisibleAmount();


		dx = (double) hs / 100. * (xBounds[1] - xBounds[0]);
		dy = (double) vs / 100. * (yBounds[1] - yBounds[0]);

		xx1 = (double) hpos / 100. * (xBounds[1] - xBounds[0]) + xBounds[0];
		xx2 = (double) (hpos + hs) / 100. * (xBounds[1] - xBounds[0]) + xBounds[0];
		yy1 = -(double) (vpos) / 100. * (yBounds[1] - yBounds[0]) + yBounds[1];
		yy2 = -(double) (vpos + vs) / 100. * (yBounds[1] - yBounds[0]) + yBounds[1];

//		drawTransits(g2);
		drawData(g2);
		drawCurves(g2);
		drawBox(g2);
//		drawPredTransits(g2);
		drawLabels(g2);

		if (mouseIn) {
			drawCurPos(g2);
		}

		if ((warning != null) && (!warning.equals(""))) {
			drawWarning(g2, warning);
		}
        
        if (mouseDrag && mouseIn) {
            drawMouseDrag(g2);
        }

        if (painter != null)
            painter.draw(this, g2, xBounds, yBounds, new double[] {x1, x2}, new double[] {y1, y2});
	}

	public final void drawCurPos(Graphics2D g2) {
		g2.setColor(Color.RED);
		g2.drawString(curPos, 10, y2 + 30);
	}

	public final void drawWarning(Graphics2D g2, String warning) {
        warningIcon.paintIcon(this, g2, 5, 5);
        g2.setColor(Color.BLACK);
		g2.setFont(new Font("Sans Serif", Font.BOLD, 12));
		g2.drawString(warning, 35, 20);
        
	}

	public void findBounds() {

		if (doUpdateCurves) {
			updateCurves();
		}

		xBounds[0] = Double.MAX_VALUE - 1;
		xBounds[1] = Double.MIN_VALUE;
		yBounds[0] = Double.MAX_VALUE - 1;
		yBounds[1] = Double.MIN_VALUE;

		if (data != null) {
			if (data.size() != 0) {
				xBounds[0] = data.get(0)[0][0];
				xBounds[1] = data.get(0)[0][0];
				yBounds[0] = data.get(0)[0][1];
				yBounds[1] = data.get(0)[0][1];

				for (int i = 0; i < data.size(); i++) {
					double[][] dataPoints = data.get(i);
					for (int j = 0; j < dataPoints.length; j++) {
						xBounds[0] = Math.min(dataPoints[j][0], xBounds[0]);
						yBounds[0] = Math.min(dataPoints[j][1] - dataPoints[j][2], yBounds[0]);
						xBounds[1] = Math.max(dataPoints[j][0], xBounds[1]);
						yBounds[1] = Math.max(dataPoints[j][1] + dataPoints[j][2], yBounds[1]);
					}
				}
			}
		}


		if (curves != null) {
			for (int j = 0; j < curves.size(); j++) {
				double[][] curve = curves.get(j);
				if (curve == null) {
					continue;
				}

				for (int i = 0; i < curve.length; i++) {
					if (foldPeriod < 0) {
						xBounds[0] = Math.min(curve[i][0], xBounds[0]);
						xBounds[1] = Math.max(curve[i][0], xBounds[1]);
					}
					yBounds[0] = Math.min(curve[i][1], yBounds[0]);
					yBounds[1] = Math.max(curve[i][1], yBounds[1]);
				}
			}
		}



		if (foldPeriod < 0) {
		} else {
			xBounds[0] = 0;
			xBounds[1] = 100.;
			if (xRange != null) {
				xRange[1] = 100.;
			}
		}


		if (customYScale > 0) {
			yBounds[0] = -customYScale;
			yBounds[1] = customYScale;
		}

		if (xRange != null) {
			if (!Double.isNaN(xRange[0])) {
				xBounds[0] = xRange[0];
			}
			if (!Double.isNaN(xRange[1])) {
				xBounds[1] = xRange[1];
			}
		}

		if (yRange != null) {
			if (!Double.isNaN(yRange[0])) {
				yBounds[0] = yRange[0];
			}
			if (!Double.isNaN(yRange[1])) {
				yBounds[1] = yRange[1];
			}
		}

		if (Math.abs(yBounds[1] - yBounds[0]) < 1e-10) {
			yBounds[0] -= 1;
			yBounds[1] += 1;
		}



		if (symmetric) {
			double yspan = yBounds[1] - yBounds[0];
			yBounds[0] = yBounds[0] - ypad * yspan;
			yBounds[1] = yBounds[1] + ypad * yspan;

			yBounds[0] = -Math.max(Math.abs(yBounds[0]), Math.abs(yBounds[1]));
			yBounds[1] = -yBounds[0];
		}

		if (intX) {
			xBounds[0] = Math.floor(xBounds[0]);
			xBounds[1] = Math.ceil(xBounds[1]);
		}
		if (intY) {
			yBounds[0] = Math.floor(yBounds[0]);
			yBounds[1] = Math.ceil(yBounds[1]);
		}


		if (doUpdateCurves) {
			updateCurves();
		}
	}

	public void setSymmetric(boolean sym) {
		symmetric = sym;
		doUpdateCurves = true;
		findBounds();
		repaint();
	}

	public void removeData(String dataName) {

		if (data != null) {
			for (int i = 0; i < data.size(); i++) {
				if ((props.get(i) != null) && (props.get(i).name.equals(dataName))) {
					data.remove(i);
					plotData.remove(i);
					props.remove(i);
					findBounds();
					repaint();
					return;
				}
			}
		}
	}

	public void removeData(int i) {
		if (data != null) {
			data.remove(i);
			props.remove(i);
			findBounds();
			repaint();
		}
	}

	public void removeCurve(String dataName) {
		for (int i = 0; i < curves.size(); i++) {
			if ((xyCurvesProps.get(i) != null) && (xyCurvesProps.get(i).getName().equals(dataName))) {
				curves.remove(i);
				xyCurvesProps.remove(i);
				xyCurves.remove(i);
				doUpdateCurves = true;
				findBounds();
				repaint();
				return;
			}
		}
	}

	public void removeCurve(int i) {
		if (data != null) {
			curves.remove(i);
			xyCurves.remove(i);
			xyCurvesProps.remove(i);

			findBounds();
			repaint();
		}
	}

	public boolean getZeroLine() {
		return zeroLine;
	}

	public void setZeroLine(boolean zl) {
		zeroLine = zl;
		repaint();
	}

	public boolean getSymmetric() {
		return symmetric;
	}

	public void setXRange(double x1, double x2) {
		setXRange(new double[]{Math.min(x1, x2), Math.max(x1, x2)});
	}

	public void setYRange(double y1, double y2) {
		setYRange(new double[]{Math.min(y1, y2), Math.max(y1, y2)});
	}

	public void setXRange(double[] xr) {
		xRange = xr;
		findBounds();
		resetZoom();
		doUpdateCurves = true;
		findBounds();
		repaint();
	}

	public void setYRange(double[] yr) {
		yRange = yr;
		findBounds();
		resetZoom();
		doUpdateCurves = true;
		findBounds();
		repaint();
	}

	public double[] getXRange() {
		return xRange;
	}

	public double[] getYRange() {
		return yRange;
	}

	public double min(ArrayList<double[][]> d) {
		if (d.size() <= 0) {
			return 0;
		}

		double m = d.get(0)[0][0];
		for (int i = 0; i < d.size(); i++) {
			for (int j = 0; j < d.get(i).length; j++) {
				m = Math.min(m, d.get(i)[j][0]);
			}
		}
		return m;
	}

	public double max(ArrayList<double[][]> d) {
		if (d.size() <= 0) {
			return 0;
		}


		double m = d.get(0)[0][0];
		for (int i = 0; i < d.size(); i++) {
			for (int j = 0; j < d.get(i).length; j++) {
				m = Math.max(m, d.get(i)[j][0]);
			}
		}
		return m;
	}

	private boolean range(double v, double v1, double v2) {
		return ((v <= v2) && (v >= v1));
	}

	private String formatLog(double v) {
		if (v >= 1) {
			return String.format("%.0f", v);
		} else {
			int nd = (int) -log10(v);

			return String.format("%." + (nd) + "f", v);
		}
	}

	private final void drawBox(Graphics2D g2) {

		g2.setColor(boxColor);
		g2.setStroke(new BasicStroke(2f));
		g2.drawRect(x1, y1, x2 - x1, y2 - y1);

		g2.setStroke(new BasicStroke(1f));



		double hs = (double) hSb.getVisibleAmount();
		double vs = (double) vSb.getVisibleAmount();

		double dx = (double) hs / 100 * (xBounds[1] - xBounds[0]);
		double dy = (double) vs / 100 * (yBounds[1] - yBounds[0]);

		String numOfDigitsX = "2";

		if (Math.abs((xx2 - xx1) / xdivs) > 1) {
			numOfDigitsX = "0";
		} else if (Math.abs((xx2 - xx1) / xdivs) > 1e-2) {
			numOfDigitsX = "1";
		} else if (Math.abs((xx2 - xx1) / xdivs) > 1e-2) {
			numOfDigitsX = "2";
		} else if (Math.abs((xx2 - xx1) / xdivs) > 1e-3) {
			numOfDigitsX = "3";
		} else if (Math.abs((xx2 - xx1) / xdivs) > 1e-4) {
			numOfDigitsX = "4";
		} else {
			numOfDigitsX = "6";
		}

		double startx = xBounds[0];

		ArrayList<Double> tics = new ArrayList<Double>();
		double dxlab = (xBounds[1] - xBounds[0]) / (xdivs * hzoom);
		if (intX) {
			dxlab = Math.ceil(dxlab);
			numOfDigitsX = "0";
		}

		if (niceTics) {
			double dig = Math.floor(Math.log10(dxlab));
			dxlab = Math.floor(dxlab / pow(10, dig)) * pow(10, dig);
			startx = ceil(xBounds[0] / dxlab) * dxlab;
		}

		for (int i = 0; i < 1000; i++) {

			double x = ((double) i) * dxlab + startx;

			if (xlog) {
				x = (double) i - 10;
			}

			if ((x >= xx1) && (x <= xx2)) {
				//System.out.println("-> " + x + " " + dxlab);
				double xx = (x - xx1) * (drWidth / dx) + xn + x1;
				double xlab = x; //dxlab * (double) i + xBounds[0];
				String lab = String.format("%." + numOfDigitsX + "f", xlab);

				if (xlog) {
					xlab = Math.pow(10., xlab);
					lab = formatLog(xlab);
				}
				tics.add(xx);

				int sw = g2.getFontMetrics().stringWidth(lab);

				g2.drawString(lab, (int) (xx - sw / 2), y2 + 15);

				g2.drawLine((int) xx, (int) y2, (int) xx, (int) y2 - 6);
				g2.drawLine((int) xx, (int) y1, (int) xx, (int) y1 + 6);
			}


		}
		int smallTicsX = (false ? (int) Math.max(dxlab / smallTics, 1) : smallTics);

		for (int i = 0; i < tics.size() - 1; i++) {
			for (int j = 1; j < smallTicsX; j++) {
				double xx = 0;
				if (xlog) {
					break;
				} else {
					xx = tics.get(i) + j * (double) (tics.get(i + 1) - tics.get(i)) / smallTicsX;
				}

				g2.drawLine((int) xx, (int) y2, (int) xx, (int) y2 - 3);
				g2.drawLine((int) xx, (int) y1, (int) xx, (int) y1 + 3);
			}

			if (i == 0) {
				for (int j = 1; j < smallTicsX; j++) {
					double xx = 0;
					if (xlog) {
						break;
					} else {
						xx = tics.get(i) - j * (double) (tics.get(i + 1) - tics.get(i)) / smallTicsX;
					}
					if (xx < x1) {
						break;
					}
					g2.drawLine((int) xx, (int) y2, (int) xx, (int) y2 - 3);
					g2.drawLine((int) xx, (int) y1, (int) xx, (int) y1 + 3);
				}
			}

			if (i == tics.size() - 2) {
				for (int j = 1; j < smallTicsX; j++) {
					double xx = 0;
					if (xlog) {
						break;
					} else {
						xx = tics.get(i + 1) + j * (double) (tics.get(i + 1) - tics.get(i)) / smallTicsX;
					}
					if (xx > x2) {
						break;
					}
					g2.drawLine((int) xx, (int) y2, (int) xx, (int) y2 - 3);
					g2.drawLine((int) xx, (int) y1, (int) xx, (int) y1 + 3);
				}
			}
		}



		double maxWidth = 0;


		if (symmetric) {
			double dylab = (Math.max(Math.abs(yBounds[1]), Math.abs(yBounds[0]))) / (ydivs * vzoom);

			numOfDigitsX = "2";

			if (Math.abs((yy2 - yy1) / ydivs) > 1e0) {
				numOfDigitsX = "0";
			} else if (Math.abs((yy2 - yy1) / ydivs) > 1e-2) {
				numOfDigitsX = "2";
			} else if (Math.abs((yy2 - yy1) / ydivs) > 1e-3) {
				numOfDigitsX = "3";
			} else if (Math.abs((yy2 - yy1) / ydivs) > 1e-4) {
				numOfDigitsX = "4";
			} else {
				numOfDigitsX = "6";
			}

			if (intY) {
				dylab = Math.ceil(dylab);
				numOfDigitsX = "0";
			}

			if (niceTics) {
				double dig = Math.floor(Math.log10(dylab));
				dylab = Math.round(dylab / pow(10, dig)) * pow(10, dig);

			}


			tics.clear();
			for (int i = 0; i < 1000; i++) {
				double y = ((double) i) * dylab;

				if (ylog) {
					y = (double) i - 10;
				}

				if ((y <= yy1) && (y >= yy2)) {
					//System.out.println("-> " + x + " " + dxlab);
					double yy = -(y - yy2) * (drHeight / dy) - yn + y2;
					double ylab = y;//dylab * (double) i;
					String lab = String.format("%." + numOfDigitsX + "f", ylab);
					tics.add(yy);
					if (ylog) {
						ylab = Math.pow(10., ylab);
						lab = formatLog(ylab);
					}

					int sw = g2.getFontMetrics().stringWidth(lab);
					maxWidth = Math.max(maxWidth, sw);
					g2.drawString(lab, (int) (x1 - sw - 10), (int) yy + fontAxis.getSize() / 2);

					g2.drawLine((int) x1, (int) yy, (int) x1 + 6, (int) yy);
					g2.drawLine((int) x2, (int) yy, (int) x2 - 6, (int) yy);

				}
			}

			for (int i = 0; i < 1000; i++) {
				double y = -((double) (i) * dylab);
				if (ylog) {
					y = (double) i - 10;
				}

				if ((y <= yy1) && (y >= yy2)) {
					//System.out.println("-> " + x + " " + dxlab);
					double yy = -(y - yy2) * (drHeight / dy) - yn + y2;
					tics.add(yy);
					double ylab = y;//dylab * (double) i;
					String lab = String.format("%." + numOfDigitsX + "f", ylab);

					if (ylog) {
						ylab = Math.pow(10., ylab);
						lab = formatLog(ylab);

					}

					int sw = g2.getFontMetrics().stringWidth(lab);
					maxWidth = Math.max(maxWidth, sw);
					if (i != 0) {
						g2.drawString(lab, (int) (x1 - sw - 10), (int) yy + fontAxis.getSize() / 2);
					}

					g2.drawLine((int) x1, (int) yy, (int) x1 + 6, (int) yy);
					g2.drawLine((int) x2, (int) yy, (int) x2 - 6, (int) yy);
				}

			}

			int smallTicsY = (false ? (int) Math.max(dylab / smallTics, 1) : smallTics);
			Collections.sort(tics);
			
			for (int i = 0; i < tics.size() - 1; i++) {
				for (int j = 1; j < smallTicsY; j++) {
					double yy = 0;
					if (ylog) {
						break;
					} else {
						yy = tics.get(i) + j * (double) (tics.get(i + 1) - tics.get(i)) / smallTicsY;
					}

					g2.drawLine((int) x1, (int) yy, (int) x1 + 3, (int) yy);
					g2.drawLine((int) x2, (int) yy, (int) x2 - 3, (int) yy);
				}

				if (i == 0) {
					for (int j = 1; j < smallTicsX; j++) {
						double yy = 0;
						if (xlog) {
							break;
						} else {
							yy = tics.get(i) - j * (double) (tics.get(i + 1) - tics.get(i)) / smallTicsY;
							if (yy < y2 && yy > y1) {
								g2.drawLine((int) x1, (int) yy, (int) x1 + 3, (int) yy);
								g2.drawLine((int) x2, (int) yy, (int) x2 - 3, (int) yy);
							}
						}

					}
				}

				if (i == tics.size() - 2) {
					for (int j = 1; j < smallTicsX; j++) {
						double yy = 0;
						if (xlog) {
							break;
						} else {
							yy = tics.get(i + 1) + j * (double) (tics.get(i + 1) - tics.get(i)) / smallTicsY;
							if (yy < y2 && yy > y1) {
								g2.drawLine((int) x1, (int) yy, (int) x1 + 3, (int) yy);
								g2.drawLine((int) x2, (int) yy, (int) x2 - 3, (int) yy);
							}
						}

					}
				}
			}


		} else {
			double dylab = (yBounds[1] - yBounds[0]) / (ydivs * vzoom);
			tics.clear();

			numOfDigitsX = "2";

			if (Math.abs((yy2 - yy1) / ydivs) > 1) {
				numOfDigitsX = "0";
			} else if (Math.abs((yy2 - yy1) / ydivs) > 1e-2) {
				numOfDigitsX = "2";
			} else if (Math.abs((yy2 - yy1) / ydivs) > 1e-3) {
				numOfDigitsX = "3";
			} else if (Math.abs((yy2 - yy1) / ydivs) > 1e-4) {
				numOfDigitsX = "4";
			} else {
				numOfDigitsX = "6";
			}

			if (intY) {
				dylab = Math.ceil(dylab);
				numOfDigitsX = "0";
			}

			double starty = yBounds[0];

			if (niceTics) {
				double dig = Math.floor(Math.log10(dylab));
				dylab = Math.round(dylab / pow(10, dig)) * pow(10, dig);
				starty = ceil(yBounds[0] / dylab) * dylab;
			}


			for (int i = 0; i < 1000; i++) {
				double y = ((double) i) * dylab + starty;
				if (ylog) {
					y = (double) i - 10;
				}

				if ((y <= yy1) && (y >= yy2)) {

					double yy = -(y - yy2) * (drHeight / dy) - yn + y2;
					tics.add(yy);
					double ylab = y; //dylab * (double) i + yBounds[0];
					String lab = String.format("%." + numOfDigitsX + "f", ylab);


					if (ylog) {
						ylab = Math.pow(10., ylab);
						lab = formatLog(ylab);
					}

					int sw = g2.getFontMetrics().stringWidth(lab);
					maxWidth = Math.max(maxWidth, sw);
					g2.drawString(lab, (int) (x1 - sw - 10), (int) yy + fontAxis.getSize() / 2);
					g2.drawLine((int) x1, (int) yy, (int) x1 + 6, (int) yy);
					g2.drawLine((int) x2, (int) yy, (int) x2 - 6, (int) yy);
				}
			}

			int smallTicsY = (false ? (int) Math.max(dylab / smallTics, 1) : smallTics);
			for (int i = 0; i < tics.size() - 1; i++) {
				for (int j = 1; j < smallTicsY; j++) {
					double yy = 0;
					if (ylog) {
						break;
					} else {
						yy = tics.get(i) + j * (double) (tics.get(i + 1) - tics.get(i)) / smallTicsY;
					}

					g2.drawLine((int) x1, (int) yy, (int) x1 + 3, (int) yy);
					g2.drawLine((int) x2, (int) yy, (int) x2 - 3, (int) yy);


				}
				if (i == tics.size() - 2) {
					for (int j = 1; j < smallTicsY; j++) {
						double yy = 0;
						if (ylog) {
							break;
						} else {
							yy = tics.get(i + 1) + j * (double) (tics.get(i + 1) - tics.get(i)) / smallTicsY;
							if (yy < y2 && yy > y1) {
								g2.drawLine((int) x1, (int) yy, (int) x1 + 3, (int) yy);
								g2.drawLine((int) x2, (int) yy, (int) x2 - 3, (int) yy);
							}
						}

					}
				}
				if (i == 0) {
					for (int j = 1; j < smallTicsX; j++) {
						double yy = 0;
						if (ylog) {
							break;
						} else {
							yy = tics.get(i) - j * (double) (tics.get(i + 1) - tics.get(i)) / smallTicsY;
							if (yy < y2 && yy > y1) {
								g2.drawLine((int) x1, (int) yy, (int) x1 + 3, (int) yy);
								g2.drawLine((int) x2, (int) yy, (int) x2 - 3, (int) yy);
							}
						}

					}
				}
			}

		}

		g2.setFont(fontLabel);

		if (foldPeriod > 0) {
			g2.drawString("Phase", (int) ((x2 - x1 - g2.getFontMetrics().stringWidth("Phase")) / 2 + x1), y2 + (int) (g2.getFontMetrics().getHeight() * 2.25));
		} else {
			g2.drawString(xlabel, (int) ((x2 - x1 - g2.getFontMetrics().stringWidth(xlabel)) / 2 + x1), y2 + (int) (g2.getFontMetrics().getHeight() * 2.25));
		}

		AffineTransform none = g2.getTransform();

		double xLab = Math.max(x1 - maxWidth - 25, 10);
		double yW = g2.getFontMetrics().stringWidth(ylabel) / 2;
		double yLab = (y2 - y1) / 2 + y1 + yW;

		AffineTransform at = new AffineTransform(g2.getTransform());
		at.rotate(-Math.PI / 2, xLab, yLab);

		g2.setTransform(at);
		g2.drawString(ylabel, (int) xLab, (int) (yLab));

		g2.setTransform(none);

		g2.setFont(fontAxis);
		if (mouseIn) {
			g2.setColor(Color.GRAY);
			g2.setStroke(pointStroke);
			g2.drawLine(x1, ymouse, x2, ymouse);
			g2.drawLine(xmouse, y1, xmouse, y2);

			double xx = (xmouse - x1 - xn) / drWidth * (xx2 - xx1) + xx1;
			double yy = yy1 - (ymouse - y1 - yn) / drHeight * (yy1 - yy2);

			if ((showDate) && (foldPeriod < 0)) {
				GregorianCalendar rd = numal.JulianDate.JDToCalendar((int) xx);
				curPos = String.format("%2d/%2d/%4d, %.2f", rd.get(rd.MONTH) + 1, rd.get(rd.DATE), rd.get(rd.YEAR), yy);
			} else {
				if (xlog) {
					curPos = String.format("%.2f, %.2f", Math.pow(10, xx), yy);
				} else {
					curPos = String.format("%.6f, %.6f", xx, yy);
				}
			}

			if (synchronizeVertical != null) {
				synchronizeVertical.setVerticalBeam(xx);
			}

		} else {
			if (synchronizeVertical != null) {
				synchronizeVertical.setVerticalBeam(-1);
			}

			if (beam != -1) {
				g2.setColor(Color.GRAY);
				g2.setStroke(pointStroke);
				double xbeam = (beam - xx1) * (drWidth / dx) + xn + x1;
				g2.drawLine((int) xbeam, y1, (int) xbeam, y2);

			}

			curPos = "";
		}

	}

	public void setVerticalBeam(double b) {
		beam = b;
		repaint();
	}

        public boolean isNiceTics() {
            return niceTics;
        }

        public void setNiceTics(boolean niceTics) {

            this.niceTics = niceTics;
            repaint();
        }
/*
	private final void drawPredTransits(Graphics2D g2) {
		double minT = min(plotData);
		double maxT = max(plotData);


		g2.setStroke(new BasicStroke(2f));
		for (int i = 0; i < predTransits.size(); i++) {
			double t = 0;
			if (foldPeriod < 0) {
				t = predTransits.get(i).centralTime;

			} else {
				double Tc = phase * (maxT - minT) + minT;
				t = (predTransits.get(i).centralTime - Tc) % foldPeriod;
				if (t < -0.5 * foldPeriod) {
					t += foldPeriod;
				} else if (t > 0.5 * foldPeriod) {
					t -= foldPeriod;
				}
			}

			if ((t >= xx1) && (t <= xx2)) {
				Color color = Color.RED;

				if (predTransits.get(i).flag == 2) {
					color = Color.GREEN;
				}


				double x = (t - xx1) * (drWidth / dx) + xn + x1;
				g2.setColor(color);
				g2.fillOval((int) x - 3, (int) y1 - 3, 6, 6);
				g2.setColor(Color.BLACK);
				g2.drawOval((int) x - 3, (int) y1 - 3, 6, 6);
			}
		}
		g2.setStroke(new BasicStroke(1f));
	}

	private final void drawTransits(Graphics2D g2) {

		double minT = min(plotData);
		double maxT = max(plotData);

		g2.setStroke(new BasicStroke(2f));


		if (transits != null) {

			for (int i = 0; i < transits.size(); i++) {
				double t = 0;
				if (foldPeriod < 0) {
					t = transits.get(i).centralTime;

				} else {
					double Tc = phase * (maxT - minT) + minT;
					t = (transits.get(i).centralTime - Tc) % foldPeriod;
					if (t < 0) {
						t += foldPeriod;
					} else if (t > foldPeriod) {
						t -= foldPeriod;
					}
					t = t / foldPeriod * 100;
				}

				if ((t >= xx1) && (t <= xx2)) {
					Color color = Color.RED;

					if (transits.get(i).flag == 2) {
						color = Color.GREEN;
					}

					if (transits.get(i).flag == 3) {
						color = Color.LIGHT_GRAY;
					}


					double x = (t - xx1) * (drWidth / dx) + xn + x1;
					g2.setColor(color);
					if (!hideTransits) {
						g2.drawLine((int) x, y1, (int) x, y2);
					}
				}
			}
		}

		g2.setStroke(new BasicStroke(1f));

	}*/

	private void drawLabels(Graphics2D g2) {
		g2.drawRect(x1, y1, x2 - x1, y2 - y1);

		if (data == null) {
			return;
		}

		for (int i = 0; i < labels.size(); i++) {
			LabelProp lab = labels.get(i);
			String txt = lab.getText();
			g2.setFont(lab.getFont());
			g2.setColor(lab.getColor());

			int w = g2.getFontMetrics().stringWidth(txt);

			float x = 0f;
			float y = 0f;
			float pad = lab.getPad();


			if (lab.getPosition().startsWith(LabelProp.SOUTH)) {
				x = 0.5f * (float) (x2 - x1 - w) + x1;
				y = y2 - pad;
			} else if (lab.getPosition().startsWith(LabelProp.NORTH)) {
				x = 0.5f * (float) (x2 - x1 - w) + x1;
				y = y1 + pad;
			}
			;

			if (lab.getPosition().endsWith("WEST")) {
				x = x1 + pad;
			} else if (lab.getPosition().endsWith("EAST")) {
				x = x2 - pad - w;
			}
			if (lab.getPosition().contains(":")) {
				String[] pos = lab.getPosition().split(":");
				x = Float.valueOf(pos[0]);
				y = Float.valueOf(pos[1]);
			}
			//System.out.println("POS = " + x + " " +y);
			g2.drawString(txt, x, y);
		}

	}

	private final void drawData(Graphics2D g2) {

		if (data == null) {
			return;
		}
        
		for (int i = 0; i < data.size(); i++) {
			double[][] dataPoints = data.get(i);
            
			int pointSize = props.get(i).size;

			for (int j = 0; j < dataPoints.length; j++) {
				if (bounded(dataPoints[j], xx1, xx2, yy2, yy1)) {
					double x = (dataPoints[j][0] - xx1) * (drWidth / dx) + xn + x1;
					double y = -(dataPoints[j][1] - yy2) * (drHeight / dy) - yn + y2;

					double sig1 = -(dataPoints[j][1] - dataPoints[j][2] - yy2) * (drHeight / dy) - yn + y2;
					double sig2 = -(dataPoints[j][1] + dataPoints[j][2] - yy2) * (drHeight / dy) - yn + y2;
					sig1 = Math.min(sig1, y2);
					sig2 = Math.max(sig2, y1);
                                        
                                        g2.setColor(Color.BLACK);

                    if (dataPoints[j][3] >= 0) {
                        int set = (int) dataPoints[j][3];
                        if (props.size() > set)
                            g2.setColor(props.get(set).color);
                    } else if (props.size() > i) {
						g2.setColor(props.get(i).color);
					}
                    
                    
					if (!hideData) {
						g2.fillOval((int) x - pointSize / 2, (int) y - pointSize / 2, pointSize, pointSize);
						if (errorBars) {
							g2.drawLine((int) x, (int) sig1, (int) x, (int) sig2);
						}
					}
				}
			}
		}

		if (zeroLine) {
			if ((0 >= yy2) && (0 <= yy1)) {
				g2.setColor(Color.lightGray);
				g2.setStroke(pointStroke);

				g2.drawLine((int) x1, (int) (yy2 * (drHeight / dy) - yn + y2), (int) x2, (int) (yy2 * (drHeight / dy) - yn + y2));
			}
		}
	}

	private final synchronized void drawCurves(Graphics2D g2) {
		if (doUpdateCurves) {
			updateCurves();
			doUpdateCurves = false;
		}


		g2.setColor(Color.BLACK);

		if (hideLine) {
			return;
		}

		GeneralPath path = new GeneralPath();

		for (int j = 0; j < xyCurves.size(); j++) {
			ArrayList<double[]> curve = xyCurves.get(j);
			CurveProp props = xyCurvesProps.get(j);

			if (curve == null) {
				continue;
			}

			if (!props.visible) {
				continue;
			}

			if (props.getColor() != null) {
				g2.setColor(props.getColor());
			} else {
				g2.setColor(Color.BLACK);
			}

			g2.setStroke(props.getStroke());

			int lasthole = 0;


			boolean firstPoint = true;

			for (int i = 0; i < curve.size(); i++) {
				double[] xy = curve.get(i);

				if ((xy[0] <= x2) && (xy[0] >= x1) && (!Double.isNaN(xy[1]))) {
					if (firstPoint) {
						path.moveTo((float) xy[0], (float) xy[1]);
						firstPoint = false;
					} else {
						path.lineTo((float) xy[0], (float) xy[1]);
					}
				} else {
					firstPoint = true;
				}
			}

			g2.draw(path);
			path.reset();
		}


	}
	private ArrayList[] curveBuffers;

	private final ArrayList getCurve(int idx, double[][] curve, CurveProp prop) {
		if (curveBuffers == null) {
			curveBuffers = new ArrayList[20];
		}
		curveBuffers[idx] = new ArrayList();
		//} else
		//  curveBuffers[idx].clear();

		ArrayList xy = curveBuffers[idx];

		if ((curve != null)) {
			double minT = 0;
			double maxT = 0;

			int interval = curve.length;

			if (foldPeriod > 0) {
				interval = Math.min(5000, curve.length);
				if ((plotData == null) || (plotData.size() == 0)) {
					return null;
				}

				minT = min(plotData);
				maxT = max(plotData);
			}

			double[] fP = new double[2];

			for (double j = 0; j < curve.length; j += ((double) curve.length / (double) interval)) {
				int i = (int) j;

				if (foldPeriod < 0) {

					double x = (curve[i][0] - xx1) * (drWidth / dx) + xn + x1;
					double y = -(curve[i][1] * prop.getScaling() - yy2) * (drHeight / dy) - yn + y2;


					double[] pt = new double[2];

					if (bounded(curve[i], xx1, xx2, yy2, yy1)) {
						pt[0] = x;
						pt[1] = y;

					} else {
						pt[0] = x;
						pt[1] = Double.NaN;

					}

					xy.add(pt);
				} else {


					double Tc = phase * (maxT - minT) + minT;

					double t = (curve[i][0] - Tc) % foldPeriod;
					if (t < 0) {
						t += foldPeriod;
					} else if (t > foldPeriod) {
						t -= foldPeriod;
					}


					fP[0] = t / foldPeriod * 100;
					fP[1] = curve[i][1] * prop.getScaling();
					double x = (fP[0] - xx1) * (drWidth / dx) + xn + x1;
					double y = -(fP[1] - yy2) * (drHeight / dy) - yn + y2;

					double[] pt = new double[2];
					if (bounded(fP, xx1, xx2, yy2, yy1)) {
						pt[0] = x;
						pt[1] = y;
					} else {
						pt[0] = x;
						pt[1] = Double.NaN;
					}

					xy.add(pt);
				}

			}
		}


		if (foldPeriod < 0) {
			return xy;
		} else {
			Collections.sort(xy, pointComparator);
			return xy;
		}

	}

	private final boolean bounded(double[] point, double xx1, double xx2, double yy1, double yy2) {
		if ((point[0] <= xx2) && (point[0] >= xx1) && (point[1] <= yy2) && (point[1] >= yy1)) {
			return true;
		} else {
			return false;
		}
	}
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.Box.Filler filler1;
    private javax.swing.JScrollBar hSb;
    private javax.swing.JButton resetZoom;
    private javax.swing.JPanel toolbar;
    private javax.swing.JScrollBar vSb;
    // End of variables declaration//GEN-END:variables

	public void componentResized(ComponentEvent e) {
		doUpdateCurves = true;
		repaint();
	}

	public void componentMoved(ComponentEvent e) {
		doUpdateCurves = true;
		repaint();
	}

	public void componentShown(ComponentEvent e) {
	}

	public void componentHidden(ComponentEvent e) {
	}

	/**
	 * @return the smallTics
	 */
	public int getSmallTics() {
		return smallTics;
	}

	/**
	 * @param smallTics the smallTics to set
	 */
	public void setSmallTics(int smallTics) {
		this.smallTics = smallTics;
	}
    
    public Double getPlotX() {
        if (! mouseIn)
            return null;
        else 
            return (double)(xmouse - x1) * (double)(x2-x1)/(xBounds[1]-xBounds[0]) + xBounds[0];
    }
    
    public Double getPlotY() {
        if (! mouseIn)
            return null;
        else 
            return (double)(ymouse - y1) * (double)(y2-y1)/(yBounds[1]-yBounds[0]) + yBounds[0];
    }

    public double plotXToScreenX(double x) {
        return (x - xBounds[0])/(xBounds[1]-xBounds[0]) * (x2-x1) + x1;
    }
    
    public double screenXToPlotX(double x) {
        return (x - x1)/(x2-x1) * (xBounds[1]-xBounds[0]) + xBounds[0];
    }
    
    public double plotYToScreenY(double y) {
        return y2 - (y - yBounds[0])/(yBounds[1]-yBounds[0]) * (y2-y1);
    }
    
    public double screenYToPlotY(double y) {
        return (y - y1)/(y2-y1) * (yBounds[1]-yBounds[0]) + yBounds[0];
    }
    
    public double[] getPlotBounds() {
        return new double[] {x1, x2, y1, y2};
    }
    
	public boolean export(String fn, String fmt) {

		String com = "#";
		try {
			PrintWriter pw = new PrintWriter(fn);

			if (curves != null) {
				for (int i = 0; i < curves.size(); i++) {
					double[][] curve = curves.get(i);

					pw.println(com + " Curve " + i + " [" + this.xyCurvesProps.get(i).getName() + "] ");
					pw.println(com + " " + curve.length + " points");

					for (int j = 0; j < curve.length; j++) {
						for (int k = 0; k < curve[j].length; k++) {
							pw.format(fmt, curve[j][k]);
						}
						pw.println();
					}

					pw.println();
					pw.println();

				}
			}

			if (data != null) {
				for (int i = 0; i < data.size(); i++) {

					double[][] curve = data.get(i);


					pw.println(com + " Dataset " + i + " [" + this.props.get(i).name + "] ");
					pw.println(com + " " + curve.length + " points");

					for (int j = 0; j < curve.length; j++) {
						for (int k = 0; k < curve[j].length; k++) {
							pw.format(fmt, curve[j][k]);
						}
						pw.println();
					}

					pw.println();
					pw.println();

				}
			}


			pw.close();
			return true;
		} catch (Exception e) {
			return false;
		}
	}
}
