/*
 * HistogramWindow.java
 *
 * Created on April 18, 2007, 3:33 PM
 */

package plotting;

import java.awt.Container;
import java.util.ArrayList;
import javax.swing.BoxLayout;
import utils.Utils;
/**
 * HistogramWindow holds and display an instance of Histogram. To modify the 
 * embedded Histogram, use the getHistogram() method.
 * @author Ste
 */



public class Histogram extends javax.swing.JFrame {
    ArrayList<double[]> data;
    
    /** Creates new form HistogramWindow */
    public Histogram(double[] data) {
        initComponents();
	this.data = new ArrayList<double[]>();
        hist = new FoldPlot("histogram");
        hist.setCaps(FoldPlot.PRINT);
        container.setLayout(new BoxLayout(container, BoxLayout.X_AXIS));
	
        container.add(hist);
        
        setSize(500, 400);
        setTitle("Test histogram");
        addData(data);
    }
    
	/**
	 * Sets the title of the Histogram.
	 * @param text Text to display as the title of the histogram.
	 */
    public void setTitle(String text) {
        labelTitle.setText(text);
    }
    
    
    
	/**
	 * Returns the displayed Histogram object.
	 * @return The instance of Histogram currently displayed.
	 */
    public FoldPlot getPlotter() {
        return hist;
    }
    
    public void print() {
        //hist.doPrint();
    }

    public void setBins(int bins) {
        txtBins.setText(Integer.toString(bins));
        bin();
    }

    public void setMin(double minv) {
	txtMin.setText(Double.toString(minv));
	bin();
    }

    public void setMax(double maxv) {
	txtMax.setText(Double.toString(maxv));
	bin();
    }

    public CurveProp addData(double[] data) {
        this.data.add(data);
        bin();
        return hist.getCurvesProps().get(hist.getCurvesSize()-1);
    }

    private void bin() {
        try {
            int bins = Integer.parseInt(txtBins.getText());
            Double min = null;
            Double max = null;
            hist.clear();
            
            if (!txtMin.getText().equals(""))
                min = Double.parseDouble(txtMin.getText());

            if (!txtMax.getText().equals(""))
                max = Double.parseDouble(txtMax.getText());

            for (int i = 0; i < data.size(); i++) {
                double[][] curve = Utils.bin(data.get(i), min, max, bins, null, null);
                hist.addHistogram(curve);
            }
            
            hist.setSymmetric(false);
            hist.updateCurves();
            hist.setWarning("");
        } catch (Exception e) {
            hist.setWarning("Error in input parameters.");
            return;
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        container = new javax.swing.JPanel();
        labelTitle = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        txtBins = new javax.swing.JTextField();
        txtMin = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        txtMax = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setLocationByPlatform(true);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        container.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        org.jdesktop.layout.GroupLayout containerLayout = new org.jdesktop.layout.GroupLayout(container);
        container.setLayout(containerLayout);
        containerLayout.setHorizontalGroup(
            containerLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 605, Short.MAX_VALUE)
        );
        containerLayout.setVerticalGroup(
            containerLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 290, Short.MAX_VALUE)
        );

        labelTitle.setFont(new java.awt.Font("Lucida Grande", 1, 13));
        labelTitle.setText("Title");

        jButton1.setText("Print...");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jLabel1.setText("Bins:");

        txtBins.setText("40");
        txtBins.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                txtBinsActionPerformed(evt);
            }
        });
        txtBins.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                txtBinsKeyPressed(evt);
            }
        });

        txtMin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                txtMinActionPerformed(evt);
            }
        });

        jLabel2.setText("Min:");

        txtMax.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                txtMaxActionPerformed(evt);
            }
        });

        jLabel3.setText("Max:");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(container, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(jLabel1)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(txtBins, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 63, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(txtMin, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 63, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel3)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(txtMax, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 63, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 198, Short.MAX_VALUE)
                        .add(jButton1))
                    .add(labelTitle))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(labelTitle)
                .add(7, 7, 7)
                .add(container, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jButton1)
                    .add(jLabel1)
                    .add(txtBins, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel2)
                    .add(txtMin, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel3)
                    .add(txtMax, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing

    }//GEN-LAST:event_formWindowClosing

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        print();
    }//GEN-LAST:event_jButton1ActionPerformed

    private void txtBinsKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_txtBinsKeyPressed

    }//GEN-LAST:event_txtBinsKeyPressed

    private void txtBinsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_txtBinsActionPerformed
        bin();
    }//GEN-LAST:event_txtBinsActionPerformed

    private void txtMinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_txtMinActionPerformed
        bin();
    }//GEN-LAST:event_txtMinActionPerformed

    private void txtMaxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_txtMaxActionPerformed
        bin();
    }//GEN-LAST:event_txtMaxActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
	java.awt.EventQueue.invokeLater(new Runnable() {
	    public void run() {
		new Histogram(null).setVisible(true);
	    }
	});
    }
    
    
    private FoldPlot hist;
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel container;
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel labelTitle;
    private javax.swing.JTextField txtBins;
    private javax.swing.JTextField txtMax;
    private javax.swing.JTextField txtMin;
    // End of variables declaration//GEN-END:variables
    
}
