/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package plotting;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.JComponent;
import repl.*;
import static java.lang.Math.*;
import java.util.HashMap;
import java.util.Map;
import javax.swing.*;
import icons.Icons;
import numal.*;
/**
 *
 * @author stefano
 */
public class Orbits extends JComponent {
    
    private double[][] els;
    private Shape[] shapes;
    private double I = 0.5*PI;
    private double O = 0.;
    private double W = 0.;
    private double zoom = 50;
    private double psize = 4;
    private Color chalk = new Color(122, 131, 146);
    private Color transpChalk = new Color(122, 131, 146, 0).darker();
    private static Map hints = new HashMap();
    private Paint gradient;
    private boolean inside;
    public Orbits() {
        setPreferredSize(new Dimension(100, 150));
        setBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK));
        
        hints.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        hints.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
        
        addMouseListener(new MouseListener() {
            public void mouseClicked(MouseEvent e) {
                Rectangle r = new Rectangle(getWidth() - 60, getHeight() - 30, 28, 28);
                if (r.contains(e.getPoint()))
                    zoomIn();
                r.setLocation(getWidth()-30, getHeight() - 30);
                if (r.contains(e.getPoint()))
                    zoomOut();
                
            }

            public void mousePressed(MouseEvent e) {
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
                inside = true;
                repaint();
                
            }

            public void mouseExited(MouseEvent e) {
                inside = false;
                repaint();
            }            
        });
        
        addMouseMotionListener(new MouseMotionListener() {
            public void mouseDragged(MouseEvent e) {
                I = (double)(e.getY() - 0.5*getHeight())/(0.5*getHeight()) * 0.5 * PI;
                W = (double)(e.getX() - 0.5*getWidth())/(0.5*getWidth()) * PI;
                calcShapes();
                repaint();
            }

            public void mouseMoved(MouseEvent e) {
                
            }
            
        });
    }
    
    public void setElements(double[][] els) {
        this.els = els;
        
        calcShapes();
        repaint();
    }
    
    @Override
    public void paintComponent(Graphics g) {
        
        Graphics2D g2 = (Graphics2D) g;
        AffineTransform at = g2.getTransform();
        g2.addRenderingHints(hints);
        
        g2.setBackground(chalk);
        g2.clearRect(0, 0, getWidth(), getHeight());
        
        gradient = new RadialGradientPaint((float)(0.5*getWidth()), (float)(0.5*getHeight()),
                (float)Math.max(0.5*getWidth(), 0.5*getHeight()),
                new float[] {0f, 1f}, new Color[] { chalk, chalk.darker() });
        g2.setPaint(gradient);
        g2.fillRect(0, 0, getWidth(), getHeight());
        
        
        g2.translate(0.5*getWidth(), 0.5*getHeight());
        
        g2.setColor(Color.GRAY);
        drawAll(g2);
        g2.translate(-0.5, -0.5);
        g2.setColor(Color.BLACK);
        drawAll(g2);
        g2.translate(-1, -1);
        g2.setColor(Color.WHITE);
        drawAll(g2);
        
        if (inside) {
            Icons.orbitsZoomIn.paintIcon(this, g2, (int)(0.5*getWidth() - 60), (int)(0.5*getHeight() - 30));
            Icons.orbitsZoomOut.paintIcon(this, g2, (int)(0.5*getWidth() - 30), (int)(0.5*getHeight() - 30));
        }
        g2.setTransform(at);
        
        
    }

    private void drawAll(Graphics2D g2) {
        drawShapes(g2);
        
        g2.drawLine((int)(-0.5*getWidth() + 10), (int)(0.5*getHeight() - 10), 
                (int)(-0.5 * getWidth() + 10 + zoom), (int)(0.5*getHeight() - 10));
        g2.drawString("1 AU", (int)(-0.5*getWidth() + 10), (int)(0.5*getHeight() - 13));
        
        
    }
    
    private void drawShapes(Graphics2D g2) {
        if (shapes == null)
            return;
        for (int i = 1; i < shapes.length; i++) {
            g2.draw(shapes[i]);
            if (i % 2 == 1)
                g2.fill(shapes[i]);
        }
    }
    
    private void calcShapes() {
        shapes = new Shape[2*els.length];
        
        shapes[1] = new Ellipse2D.Double(-0.5*psize, -0.5*psize, psize, psize);
        
        double[] x = new double[3];
        double[] X = new double[3];
        double[] P = new double[3];
        
        for (int i = 1; i < els.length; i++) {
            Polygon orbit = new Polygon();
            double a = els[i][Defs.SMA];
            double e = els[i][Defs.ECC];
            double p = Math.toRadians(els[i][Defs.LOP]);
            double inc = Math.toRadians(els[i][Defs.INC]);
            double node = Math.toRadians(els[i][Defs.NODE]);
            
            for (double theta = 0; theta <= 2*PI; theta+=0.025) {
                double f = theta - p;
                double r = zoom * a * (1-e*e) / (1 + e*cos(f));
                x[0] = r * cos(f);
                x[1] = r * sin(f);
                NumUtils.rotate(x, X, p, node, inc);
                NumUtils.rotate(X, P, W, O, I);
                
                orbit.addPoint((int) P[0], (int) P[1]);
            }
            
            shapes[2*i] = orbit;
            double f = Math.toRadians(els[i][Defs.TRUEANOMALY]);
            double r = zoom * a * (1-e*e) / (1 + e*cos(f));
            
            x[0] = r*cos(f);
            x[1] = r*sin(f);
            NumUtils.rotate(x, X, p, node, inc);
            NumUtils.rotate(X, P, W, O, I);
            shapes[2*i+1] = new Ellipse2D.Double(P[0] - 0.5*psize, P[1] - 0.5*psize, psize, psize);
        }
    }
    
    public void zoomIn() {
        zoom *= 1.5;
        calcShapes();
        repaint();
    }
    
    public void zoomOut() {
        zoom = Math.max(1., 0.75 * zoom);
        calcShapes();
        repaint();
    }
}
