/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package repl;

import controls.*;
import icons.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Map;
import javax.swing.*;
import palette.Container;
import plotting.FoldPlot;
import plotting.Orbits;
import uicomponents.*;
import utils.*;

/**
 *
 * @author stefano
 */
public class Console extends javax.swing.JPanel implements NotificationListener {
    private Terminal t;
    private FoldPlot rv;
    private FoldPlot res;
    private Periodogram per;
    private Periodogram per_res;
    private Frame frame;
    private Orbits orbits;
    
    private Controller controller;
    private PropertyPanel ppanel;
    private ElementsPanel epanel;
    private DataOffsetsPanel opanel;
    private Browser browser;
    private SourceList kernels;
    private double folding = 0;
    /**
     * 
     * Creates new form Console
     */
    private Console() {
        
    }
    public Console(Controller c, Frame f) {
        initComponents();
        
        controller = c;
        frame = f;
        
        t = new Terminal(c);
        topPanel.add(t);
        
        rv = new FoldPlot("RV Plot");
        res = new FoldPlot("Residuals Plot");
        rv.setNoDataWarning("No radial velocities loaded.");
        
        res.setValueColumn(7);
        res.setNoDataWarning("No residuals.");
        
        
        dataPanel.add(Container.getContainerWithHeader(rv, "RADIAL VELOCITY DATA", topLeft, null));
        dataPanel.add(Container.getContainerWithHeader(res, "RESIDUALS", topLeft, null));
        
        per = new Periodogram();
        per_res = new Periodogram();
        
        Container container = Container.getContainerWithHeader(per, "PERIODOGRAM", topLeft, null);
        perPanel.add(container);
        
        container = Container.getContainerWithHeader(per_res, "PERIODOGRAM OF RESIDUALS", topLeft, null);
        perPanel.add(container);
        
        
        //rightSidebar.add(sp, BorderLayout.CENTER);
        ppanel = PropertyPanel.getInstance(c, this);
        
        
        kernels = new SourceList(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String kernel = kernels.getSelectedItem();
                controller.setKernel(kernel);
            }
        });
        kernels.setSorted(true);
        
        kernels.setPopupMenu(UIUtils.createPopupMenu(
                UIUtils.createMenuItem(this, "Clone", "cloneKernel", null, ""),
                UIUtils.createMenuItem(this, "Delete", "deleteKernel", null, "")
        ));
        
        Container kernelsContainer = Container.getContainerForSidebar(kernels, "KERNELS");
        JButton addKernel = new JButton(Icons.add);
        addKernel.setBorder(BorderFactory.createEmptyBorder(4, 0, 4, 4));
        addKernel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                t.setInput("# = kernel.new()").requestFocusInWindow();
            }
        });
        kernelsContainer.getToolbar().add(addKernel);
        leftSidebar.add(kernelsContainer, new GridBagConstraints(0, 0, 1, 1, 1, 0.0, GridBagConstraints.CENTER,
                GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        leftSidebar.add(Container.getContainerForSidebar(ppanel, "PROPERTIES"),
                new GridBagConstraints(0, 1, 1, 1, 1, 0.9, GridBagConstraints.CENTER,
                GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        leftSidebar.setBackground(UIUtils.sidebarColor);
        
        orbits = new Orbits();
        leftSidebar.add(orbits, new GridBagConstraints(0, 2, 0, 1, 1, 0.2, GridBagConstraints.CENTER,
                GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        
        epanel = new ElementsPanel(controller, this);
        container = Container.getContainerWithHeader(epanel, "ORBITAL ELEMENTS", leftRight, topBottom);
        bottomPanel.add(container, BorderLayout.CENTER);
        epanel.setMinimumSize(new Dimension(100, 300));
        epanel.setPreferredSize(new Dimension(100, 300));
        epanel.setMaximumSize(new Dimension(100, 340));
        epanel.setContainer(container);
        
        opanel = new DataOffsetsPanel(controller, this);
        container = Container.getContainerWithHeader(opanel, "DATA OFFSETS", leftRight, null);
        bottomPanel.add(container, BorderLayout.SOUTH);
        opanel.setPreferredSize(new Dimension(100, 60));
        opanel.setMaximumSize(new Dimension(100, 60));
        opanel.setContainer(container);
        
        bottomPanel.setMinimumSize(new Dimension(100, 400));
        bottomPanel.setPreferredSize(new Dimension(100, 400));
        
        c.addGUIListener(this);
        c.addGUIListener(t);
        t.requestFocusInWindow();
        
        browser = new Browser(c, this);
        browserPanel.setLayout(new BorderLayout());
        browserPanel.add(Container.getContainerWithHeader(browser, "DATA BROWSER", topLeft, null),
                BorderLayout.CENTER);
        
        rightSidebar.add(Container.getContainerWithHeader(Transits.getTransitsPanel(c), "TRANSIT TIMINGS", topLeft, null), "ttv");
        topBottom.resetToPreferredSizes();
        
        switchRightSidebar("rvs");
        if (!System.getProperty("os.name").toLowerCase().contains("mac"))
            topBottom.setDividerLocation(-1);
    }

    public void defaultFocus() {
        t.defaultFocus();
    }
    
    public Terminal getTerminal() {
        return t;
    }
    
    public Frame getFrame() {
        return frame;
    }
    
    public void switchRightSidebar(String what) {
        ((CardLayout) rightSidebar.getLayout()).show(rightSidebar, what);
        
        if (what.equals("rvs")) {
            defaultFocus();
            showRV.setSelected(true);
        }
        else if (what.equals("browser")) {
            browser.show();
            showBrowser.setSelected(true);
        }
        else if (what.equals("per")) {
            defaultFocus();
            showPeriodogram.setSelected(true);
        } else if (what.equals("ttv")) {
            defaultFocus();
            showTTV.setSelected(true);
        }
    }
    
    protected void updateSizes() {
        statusBar.setLayout(null);
        statusBar.setMinimumSize(new Dimension(100, 31));
        statusBar.setPreferredSize(new Dimension(100, 30));
        leftStatus.setSize(new Dimension(leftSidebar.getWidth(), 30));
        centerStatus.setSize(new Dimension(topPanel.getWidth(), 30));
        centerStatus.setLocation(leftStatus.getWidth(), 1);
        rightStatus.setSize(new Dimension(rightSidebar.getWidth(), 30));
        rightStatus.setLocation(leftStatus.getWidth() + centerStatus.getWidth(), 1);
        statusBar.revalidate();
    }
    
    private void setFolding(double num) {
        folding = num;
        
        if ((int) folding == 0) {
            rv.setFoldPeriod(-1);
            res.setFoldPeriod(-1);
        } else if (folding > 0) {
            rv.setFoldPeriod(folding);
            res.setFoldPeriod(folding);
        } else {
            notification(Notification.notification(Controller.TYPE_STATS, null, 
                    controller.getKernel()));
        };
    }
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        rightSidebarButtonGroup = new javax.swing.ButtonGroup();
        topLeft = new javax.swing.JSplitPane();
        leftRight = new javax.swing.JSplitPane();
        topBottom = new javax.swing.JSplitPane();
        topPanel = new javax.swing.JPanel();
        bottomPanel = new javax.swing.JPanel();
        leftSidebar = new javax.swing.JPanel();
        rightSidebar = new javax.swing.JPanel();
        browserPanel = new javax.swing.JPanel();
        dataPanel = new javax.swing.JPanel();
        perPanel = new javax.swing.JPanel();
        statusBar = new javax.swing.JPanel();
        leftStatus = new javax.swing.JPanel();
        filler3 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        centerStatus = new javax.swing.JPanel();
        filler2 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        rightStatus = new javax.swing.JPanel();
        bottomStatusCards = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        fold = new controls.JMacButton();
        filler1 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        showRV = new controls.JToggleMacButton();
        showTTV = new controls.JToggleMacButton();
        showPeriodogram = new controls.JToggleMacButton();
        showBrowser = new controls.JToggleMacButton();

        setBorder(javax.swing.BorderFactory.createMatteBorder(1, 0, 0, 0, new java.awt.Color(102, 102, 102)));
        setPreferredSize(new java.awt.Dimension(1024, 511));
        setSize(new java.awt.Dimension(1024, 0));
        setLayout(new java.awt.BorderLayout());

        topLeft.setBackground(new java.awt.Color(102, 102, 102));
        topLeft.setBorder(null);
        topLeft.setDividerLocation(750);
        topLeft.setDividerSize(1);
        topLeft.setContinuousLayout(true);

        leftRight.setBackground(new java.awt.Color(102, 102, 102));
        leftRight.setBorder(null);
        leftRight.setDividerLocation(200);
        leftRight.setDividerSize(1);
        leftRight.setContinuousLayout(true);
        leftRight.setPreferredSize(new java.awt.Dimension(300, 510));

        topBottom.setBackground(new java.awt.Color(102, 102, 102));
        topBottom.setBorder(null);
        topBottom.setDividerLocation(350);
        topBottom.setDividerSize(1);
        topBottom.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        topBottom.setContinuousLayout(true);

        topPanel.setPreferredSize(new java.awt.Dimension(100, 400));
        topPanel.setSize(new java.awt.Dimension(100, 400));
        topPanel.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                topPanelComponentResized(evt);
            }
        });
        topPanel.setLayout(new java.awt.BorderLayout());
        topBottom.setTopComponent(topPanel);

        bottomPanel.setBackground(new java.awt.Color(239, 238, 238));
        bottomPanel.setLayout(new java.awt.BorderLayout());
        topBottom.setRightComponent(bottomPanel);

        leftRight.setRightComponent(topBottom);

        leftSidebar.setBackground(new java.awt.Color(223, 227, 233));
        leftSidebar.addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                leftSidebarComponentResized(evt);
            }
        });
        leftSidebar.setLayout(new java.awt.GridBagLayout());
        leftRight.setLeftComponent(leftSidebar);

        topLeft.setLeftComponent(leftRight);

        rightSidebar.setLayout(new java.awt.CardLayout());

        browserPanel.setBackground(new java.awt.Color(239, 238, 238));
        rightSidebar.add(browserPanel, "browser");

        dataPanel.setLayout(new java.awt.GridLayout(2, 1));
        rightSidebar.add(dataPanel, "rvs");

        perPanel.setLayout(new java.awt.GridLayout(2, 0));
        rightSidebar.add(perPanel, "per");

        topLeft.setRightComponent(rightSidebar);

        add(topLeft, java.awt.BorderLayout.CENTER);

        statusBar.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createMatteBorder(1, 0, 0, 0, new java.awt.Color(0, 0, 0)), javax.swing.BorderFactory.createEmptyBorder(0, 2, 0, 2)));
        statusBar.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        leftStatus.setLayout(new javax.swing.BoxLayout(leftStatus, javax.swing.BoxLayout.LINE_AXIS));
        leftStatus.add(filler3);

        statusBar.add(leftStatus, new org.netbeans.lib.awtextra.AbsoluteConstraints(2, 17, -1, -1));

        centerStatus.setLayout(new javax.swing.BoxLayout(centerStatus, javax.swing.BoxLayout.LINE_AXIS));
        centerStatus.add(filler2);

        statusBar.add(centerStatus, new org.netbeans.lib.awtextra.AbsoluteConstraints(2, 17, -1, -1));

        rightStatus.setLayout(new javax.swing.BoxLayout(rightStatus, javax.swing.BoxLayout.LINE_AXIS));

        bottomStatusCards.setLayout(new java.awt.CardLayout());

        jPanel1.setLayout(new javax.swing.BoxLayout(jPanel1, javax.swing.BoxLayout.LINE_AXIS));

        fold.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/folding.png"))); // NOI18N
        fold.setText("No folding");
        fold.setButtonType("textured");
        fold.setFocusable(false);
        fold.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                foldActionPerformed(evt);
            }
        });
        jPanel1.add(fold);

        bottomStatusCards.add(jPanel1, "rvs");

        rightStatus.add(bottomStatusCards);
        rightStatus.add(filler1);

        rightSidebarButtonGroup.add(showRV);
        showRV.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/rv.png"))); // NOI18N
        showRV.setSelected(true);
        showRV.setToolTipText("Radial velocities");
        showRV.setButtonType("segmentedTextured");
        showRV.setFocusable(false);
        showRV.setMinimumSize(new java.awt.Dimension(40, 32));
        showRV.setPreferredSize(new java.awt.Dimension(40, 32));
        showRV.setSegmentPosition(0);
        showRV.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showRVActionPerformed(evt);
            }
        });
        rightStatus.add(showRV);

        rightSidebarButtonGroup.add(showTTV);
        showTTV.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/ttv.png"))); // NOI18N
        showTTV.setToolTipText("Transit timings");
        showTTV.setButtonType("segmentedTextured");
        showTTV.setFocusable(false);
        showTTV.setSegmentPosition(1);
        showTTV.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showTTVActionPerformed(evt);
            }
        });
        rightStatus.add(showTTV);

        rightSidebarButtonGroup.add(showPeriodogram);
        showPeriodogram.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/periodogram.png"))); // NOI18N
        showPeriodogram.setToolTipText("Periodogram");
        showPeriodogram.setButtonType("segmentedTextured");
        showPeriodogram.setFocusable(false);
        showPeriodogram.setSegmentPosition(1);
        showPeriodogram.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showPeriodogramActionPerformed(evt);
            }
        });
        rightStatus.add(showPeriodogram);

        rightSidebarButtonGroup.add(showBrowser);
        showBrowser.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/systems.png"))); // NOI18N
        showBrowser.setToolTipText("Data browser");
        showBrowser.setButtonType("segmentedTextured");
        showBrowser.setFocusable(false);
        showBrowser.setSegmentPosition(2);
        showBrowser.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showBrowserActionPerformed(evt);
            }
        });
        rightStatus.add(showBrowser);

        statusBar.add(rightStatus, new org.netbeans.lib.awtextra.AbsoluteConstraints(2, 1, -1, -1));

        add(statusBar, java.awt.BorderLayout.PAGE_END);
    }// </editor-fold>//GEN-END:initComponents

    private void showRVActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showRVActionPerformed
        switchRightSidebar("rvs");
    }//GEN-LAST:event_showRVActionPerformed

    private void showBrowserActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showBrowserActionPerformed
        switchRightSidebar("browser");
    }//GEN-LAST:event_showBrowserActionPerformed

    private void leftSidebarComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_leftSidebarComponentResized
        updateSizes();
    }//GEN-LAST:event_leftSidebarComponentResized

    private void topPanelComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_topPanelComponentResized
        updateSizes();
    }//GEN-LAST:event_topPanelComponentResized

    private static JPopupMenu foldMenu;
    
    private void foldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_foldActionPerformed
        
        if (foldMenu == null) {
            foldMenu = new JPopupMenu();
            
            ActionListener listener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String cmd = e.getActionCommand();
                    if (cmd.equals("0"))
                        setFolding(0.);
                    else if (cmd.equals("custom")) {
                        String ret = JOptionPane.showInputDialog("Enter folding period:", "100.");
                        if (ret != null)
                            setFolding(Double.valueOf(ret));
                    } else
                        setFolding(-Double.valueOf(cmd));
                    
                    fold.setText(((JMenuItem) e.getSource()).getText());
                }
            };
            
            JMenuItem mi = foldMenu.add("No folding");
            mi.setActionCommand(Integer.toString(0));
            mi.addActionListener(listener);
            
            foldMenu.addSeparator();
            
            for (int i = 1; i <= 6; i++) {
                mi = foldMenu.add("Planet " + i);
                mi.addActionListener(listener);
                mi.setActionCommand(Integer.toString(i));
            }
            
            foldMenu.addSeparator();
            mi = foldMenu.add("Custom...");
            mi.setActionCommand("custom");
            mi.addActionListener(listener);
        }
        
        foldMenu.show(rightStatus, 0, -foldMenu.getHeight());
        
    }//GEN-LAST:event_foldActionPerformed

    private void showPeriodogramActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showPeriodogramActionPerformed
        switchRightSidebar("per");
    }//GEN-LAST:event_showPeriodogramActionPerformed

    private void showTTVActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showTTVActionPerformed
        switchRightSidebar("ttv");
    }//GEN-LAST:event_showTTVActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel bottomPanel;
    private javax.swing.JPanel bottomStatusCards;
    private javax.swing.JPanel browserPanel;
    private javax.swing.JPanel centerStatus;
    private javax.swing.JPanel dataPanel;
    private javax.swing.Box.Filler filler1;
    private javax.swing.Box.Filler filler2;
    private javax.swing.Box.Filler filler3;
    private controls.JMacButton fold;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JSplitPane leftRight;
    private javax.swing.JPanel leftSidebar;
    private javax.swing.JPanel leftStatus;
    private javax.swing.JPanel perPanel;
    private javax.swing.JPanel rightSidebar;
    private javax.swing.ButtonGroup rightSidebarButtonGroup;
    private javax.swing.JPanel rightStatus;
    private controls.JToggleMacButton showBrowser;
    private controls.JToggleMacButton showPeriodogram;
    private controls.JToggleMacButton showRV;
    private controls.JToggleMacButton showTTV;
    private javax.swing.JPanel statusBar;
    private javax.swing.JSplitPane topBottom;
    private javax.swing.JSplitPane topLeft;
    private javax.swing.JPanel topPanel;
    // End of variables declaration//GEN-END:variables

    public void notification(Notification e) {
        
        if (e.is(Controller.TYPE_RVCURVE, Controller.TYPE_UPDATE) && e.name.equals(controller.getKernel())) {
            double[][] curve = controller.getRVCurveForKernel(e.name);
            
            rv.beginUpdate();
            rv.clearCurves();
            if (curve != null)
                rv.addCurve(curve);
            rv.endUpdate();
            
            
            curve = controller.getPerResidualsForKernel(e.name);
            per_res.beginUpdate();
            per_res.clear();
            if (curve != null)
                per_res.addCurve(curve);
            
            per_res.setSymmetric(false);
            per_res.endUpdate();
            
        }
        
        if (e.is(Controller.TYPE_DATA, Controller.TYPE_UPDATE) && e.name.equals(controller.getKernel())) {
            double[][] data = controller.getDataForKernel(e.name, Defs.T_RV);
            
            rv.beginUpdate();
            rv.clearData();
            if (data != null)
                rv.addData(data, e.name, 9);
            
            rv.setValueColumn(Defs.T_SVAL);
            rv.setHighlight(1);
            rv.endUpdate();
            res.beginUpdate();
            res.clearData();
            res.setValueColumn(Defs.T_GUI);
            
            if (data != null)
                res.addData(data, e.name, 9);
            res.setHighlight(1);
            res.setXRange(rv.getXRange());
            res.endUpdate();
            
        }
        
        if (e.is(Controller.TYPE_PERIODOGRAM) && e.name.equals(controller.getKernel())) {
            double [][] data = controller.getPeriodogramForKernel(e.name);
            per.setPeriodogramData(data);
        }
        
        if (e.is(Controller.TYPE_PER_RES) && e.name.equals(controller.getKernel())) {
            double [][] data = controller.getPerResidualsForKernel(e.name);
            per_res.setPeriodogramData(data);
        }
        
        if (e.is(Controller.TYPE_STATS, Controller.TYPE_UPDATE) && e.name.equals(controller.getKernel())) {
            Map<String, Double> stats = controller.getStatsForKernel(e.name);
            if (stats != null) {                
                double[][] elements = controller.getElementsForKernel(controller.getKernel());
                if (folding < 0 && elements != null) {
                    int np = -(int) folding;
                    if (np < elements.length) {
                        rv.setFoldPeriod(elements[np][0]);
                        res.setFoldPeriod(elements[np][0]);
                    } else {
                        rv.setFoldPeriod(-1);
                        res.setFoldPeriod(-1);
                    }
                } else {
                    rv.setFoldPeriod(-1);
                    res.setFoldPeriod(-1);
                }
                orbits.setElements(elements);
            } else {
                rv.setFoldPeriod(-1);
                res.setFoldPeriod(-1);
            }
            
            
        }  
        
        if (e.is(Controller.TYPE_BUSY)) {
            
        } else if (e.is(Controller.TYPE_READY)) {
            
        }
        
        if (e.is(Controller.TYPE_NEW_KERNEL, Controller.TYPE_DELETE_KERNEL, Controller.TYPE_UPDATE)) {
            String[] kernelList = controller.getKernels();
            kernels.setItems(kernelList);
            kernels.setSelectedItem(controller.getKernel());
            
            for (int i = 0; i < kernelList.length; i++)
                kernels.setIcon(i, Icons.kernelIcon);
        }
    }
    
    public void cloneKernel() {
        String name = kernels.getSelectedItem();
        t.setInput(String.format("# = %s:clone()", name));
    }
    
    public void deleteKernel() {
        String name = kernels.getSelectedItem();
        t.exec(String.format("%s = nil", name));
    }
}
