/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package repl;

import java.io.*;
import java.io.File;
import java.lang.ref.WeakReference;
import java.util.*;
import java.util.logging.*;
import javax.swing.SwingUtilities;
import org.apache.commons.io.*;
import utils.FileUtils;
import utils.Notification;
import utils.NotificationListener;
import utils.Utils;

/**
 *
 * @author stefano
 */
public class Controller implements NotificationListener {
    private HashMap<String, double[][]> rvCurves = new HashMap<String, double[][]>();
    private HashMap<String, double[][]> rvData = new HashMap<String, double[][]>();
    private HashMap<String, HashMap<String, Double>> kernStats = new HashMap<String, HashMap<String, Double>>();
    private HashMap<String, double[][]> elements = new HashMap<String, double[][]>();
    private HashMap<String, int[][]> elFlags = new HashMap<String, int[][]>();
    private HashMap<String, int[]> parFlags = new HashMap<String, int[]>();
    private HashMap<String, double[][]> periodograms = new HashMap<String, double[][]>();
    private HashMap<String, double[][]> perResiduals = new HashMap<String, double[][]>();
    
    private HashSet<String> kernels = new HashSet<String>();
    private ArrayList<String> outputs = new ArrayList<String>();
    private Instance i;
    private ArrayList<WeakReference<NotificationListener>> clist = new ArrayList<WeakReference<NotificationListener>>();
    private ArrayList<WeakReference<NotificationListener>> guilist = new ArrayList<WeakReference<NotificationListener>>();
    private String currentKernel;
    private Vector execBuffer = new Vector();
    
    private boolean isReady = true;
    private boolean isSilent = false;
            
    // Communication notifications
    public static String TYPE_OUTPUT = "output";
    public static String TYPE_RVCURVE = "rvcurve";
    public static String TYPE_PERIODOGRAM = "periodogram";
    public static String TYPE_PER_RES = "per_res";
    public static String TYPE_DATA = "data";
    public static String TYPE_NEW_KERNEL = "newkernel";
    public static String TYPE_DELETE_KERNEL = "delkernel";
    public static String TYPE_STATS = "stats";
    public static String TYPE_STATE = "state";
    public static String TYPE_READY = "ready";
    public static String TYPE_BUSY = "busy";
    public static String TYPE_PROGRESS = "progress";
    public static String TYPE_SAVE_SESSION = "savesession";
    public static String TYPE_CLEAR_SESSION = "clearsession";
    
    // Internal notifications
    public static String TYPE_UPDATE = "#update";
    
    public static int MAX_LINES_SINCE_READY = 100;
    private int lines_since_ready = 0;
    public static int ELEMENTS_SIZE = 15;
    
    private Controller() {        
    }
    
    public static Controller newController() {
        Controller c = new Controller();
        c.i = Instance.getInstance(c);
        return c;
    }
    
    public void restart() {
        i.stop();
        i = Instance.getInstance(this);
    }

    private void processNotification(final Notification n) {
        
        try {
            if (n.is(TYPE_RVCURVE)) {
                double[][] curve = rvCurves.get(n.name);
                rvCurves.put(n.name, i.readBinaryMatrix(curve));
            } else if (n.is(TYPE_DATA)) {
                double[][] data = i.readBinaryMatrix(rvData.get(n.name));
                if (data != null)
                    for (int i = 0; i < data.length; i++)
                        data[i][Defs.T_GUI] = data[i][Defs.T_SVAL] - data[i][Defs.T_PRED];
                rvData.put(n.name, data);
            } else if (n.is(TYPE_PERIODOGRAM)) {
                double[][] data = periodograms.get(n.name);
                periodograms.put(n.name, i.readBinaryMatrix(data));
            } else if (n.is(TYPE_PER_RES)) {
                double[][] data = perResiduals.get(n.name);
                perResiduals.put(n.name, i.readBinaryMatrix(data));
                
            } else if (n.is(TYPE_NEW_KERNEL)) {
                if (kernels.size() == 0)
                    currentKernel = n.name;
                kernels.add(n.name);
            } else if (n.is(TYPE_DELETE_KERNEL)) {
                kernels.remove(n.name);
                kernStats.remove(n.name);
                rvData.remove(n.name);
                rvCurves.remove(n.name);
                elFlags.remove(n.name);
                parFlags.remove(n.name);
            } else if (n.is(TYPE_OUTPUT)) {
                if (lines_since_ready >= MAX_LINES_SINCE_READY) {
                    if (lines_since_ready == MAX_LINES_SINCE_READY) {
                        n.data = "...further output chopped";
                    } else {
                        n.type = "";
                        n.data = null;
                    }
                } else {
                    outputs.add((String) n.data);
                }
                if (!isReady)
                    lines_since_ready++;
            } else if (n.is(TYPE_STATS)) {
                HashMap<String, Double> vals = new HashMap<String, Double>();
                
                ArrayList<String> lines = FileUtils.readFile(Instance.tmpDir + n.name + "_" + n.type);

                for (int i = 0; i < lines.size(); i++) {
                    String[] pair = lines.get(i).split("=");
                    vals.put(pair[0].trim(), Utils.toDouble(pair[1].trim()));
                }

                kernStats.put(n.name, vals);
            } else if (n.is(TYPE_READY)) {
                isReady = true;
                lines_since_ready = 0;
                if (! execBuffer.isEmpty()) {
                    i.exec((String) execBuffer.firstElement());
                    execBuffer.remove(0);
                    n.type = TYPE_BUSY;
                }
                
            } else if (n.is(TYPE_BUSY)) {
                isReady = false;
            } else if (n.is(TYPE_STATE)) {
                double[][] els = elements.get(n.name);
                elements.put(n.name, i.readBinaryMatrix(els));
                int[][] flags = elFlags.get(n.name);
                elFlags.put(n.name, i.readAsciiIntMatrix(flags));
                int[] pFlags = parFlags.get(n.name);
                parFlags.put(n.name, i.readAsciiIntVector(pFlags));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void notification(final Notification n) {
        processNotification(n);
        
        if (!isSilent) {
            for (int i = 0; i < clist.size(); i++)
                if (clist.get(i).get() != null)
                    clist.get(i).get().notification(n);

            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    for (int i = 0; i < guilist.size(); i++) {
                        if (guilist.get(i).get() != null)
                            guilist.get(i).get().notification(n);
                    }
                }
            });
        }
    }
    
    public void exec(String line) {
        if (isReady) {
            i.exec(line);
        } else {
            execBuffer.add(line);
        }
    }
    
    public void message(String message) {
        notification(Notification.notification(TYPE_OUTPUT, message, null));
    }
    
    public boolean isReady() {
        return isReady;
    }
    
    public void addListener(NotificationListener cl) {
        clist.add(new WeakReference(cl));
    }
    
    public void addGUIListener(NotificationListener cl) {
        guilist.add(new WeakReference(cl));
    }
    public void setKernel(String k) {
        if (! currentKernel.equals(k)) {
            currentKernel = k;
            notification(Notification.notification(Controller.TYPE_UPDATE, 
                null, k));
        }
    }
    
    public void setSilent(boolean silent) {
        isSilent = silent;
    }
    
    public Map<String, Double> getStatsForKernel(String k) {
        return kernStats.get(k);
    }
    
    
    public String getKernel() {
        return currentKernel;
    }
    
    public String[] getKernels() {
        return kernels.toArray(new String[1]);
    }
    
    public double[][] getDataForKernel(String k, int flag) {
        if (flag < 0)
            return rvData.get(k);
        else {
            double[][] data = rvData.get(k);
            if (data == null)
                return null;
            
            int size = 0;
            for (int i = 0 ; i < data.length; i++)
                if (Math.floor(data[i][Defs.T_FLAG]) == flag)
                    size++;
            if (size == 0)
                return null;
            
            double[][] ret = new double[size][data[0].length];
            size = 0;
            for (int i = 0 ; i < data.length; i++) {
                if (Math.floor(data[i][Defs.T_FLAG]) == flag) {
                    ret[size] = data[i];
                    size++;
                }
            }
            return ret;
        }
    }
    
    public double[][] getRVCurveForKernel(String k) {
        return rvCurves.get(k);
    }
    
    public double[][] getPeriodogramForKernel(String k) {
        return periodograms.get(k);
    }
    
    public double[][] getPerResidualsForKernel(String k) {
        return perResiduals.get(k);
    }
    
    public double[][] getElementsForKernel(String k) {
        return elements.get(k);
    }
    
    public int getNPlanetsForKernel(String k) {
        double[][] els = elements.get(k);
        if (els == null)
            return -1;
        else
            return els.length - 1;
    }
    
    public int[][] getElFlagsForKernel(String k) {
        return elFlags.get(k);
    }
    
    public int[] getParFlagsForKernel(String k) {
        return parFlags.get(k);
    }
    
    public Instance getInstance() {
        return i;
    }
}
