/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package repl;

import java.io.*;
import java.nio.charset.*;
import java.util.*;
import java.util.ArrayList;
import java.util.Scanner;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import org.apache.commons.io.*;
import utils.Notification;

/**
 *
 * @author stefano
 */

class ByteArray {
    private static int DEFAULT_CAPACITY = 256;
    private byte[] arr = new byte[DEFAULT_CAPACITY];
    private int cursor = -1;
    
    public void append(byte b) {
        cursor++;
        
        if (cursor >= arr.length) {
            arr = Arrays.copyOf(arr, arr.length*2);
        }
        
        arr[cursor] = b;
    }
    
    public void clear() {
        cursor = 0;
    }
    
    public int length() {
        return cursor + 1;
    }
    
    public byte[] array() {
        return arr;
    }
}

public class Instance {
    public static String replScript = "lua/gui.lua";
    public static String interpreter = "./local/bin/luajit-2.0.0-beta9";
    public static String tmpDir = "./";
    
    private Process repl;
    private DataInputStream disOut;
    private InputStreamReader rdOut;
    private BufferedWriter replIn;
    private BufferedReader replErr;
    public static int RVSAMPLES = 2500;
    public static int PERSAMPLES = 20000;
    public static int PERSIZE = 6;
    public static int MAX_GUI_PLANETS = 6;
    
    private Scanner scanner;
    private Controller controller;
    
    private BufferedWriter logger;
    private String id;
    private boolean destroyed = false;
    
    private Instance(Controller c) throws IOException {
        id = UUID.randomUUID().toString();
        tmpDir = File.createTempFile("systemic", id).toString();
        if (Prefs.getValue("LUA", null) == null) {
            if (new File("./local/bin/luajit-2.0.0-beta9").exists())
                Prefs.setValue("LUA", "./local/bin/luajit-2.0.0-beta9");
            else if (new File("/usr/local/bin/luajit-2.0.0-beta9").exists())
                Prefs.setValue("LUA", "/usr/local/bin/luajit-2.0.0-beta9");
            else if (new File("/opt/local/bin/luajit-2.0.0-beta9").exists())
                Prefs.setValue("LUA", "/opt/local/bin/luajit-2.0.0-beta9");
        }
        
        if (Prefs.getValue("LUA", null) == null || !(new File(Prefs.getValue("LUA", "/usr/local/bin/luajit-2.0.0-beta9")).exists())) {
            String where = JOptionPane.showInputDialog("<html>Please enter the path to LuaJIT<br><small>The default path would be /usr/local/bin/luajit-2.0.0-beta9, but I couldn't find it there :(");
            if (where == null)
                System.exit(0);
            Prefs.setValue("LUA", where);
        }
        
        System.out.println("Using " + tmpDir);
        System.out.println(Prefs.getValue("LUA", null));
        
        ProcessBuilder pb = new ProcessBuilder(Prefs.getValue("LUA", null), replScript);
        pb.environment().put("LUA_PATH", ";;lua/?.lua");
        repl = pb.start();
        
        rdOut = new InputStreamReader(repl.getInputStream());
        scanner = new Scanner(rdOut);
        replErr = new BufferedReader(new InputStreamReader(repl.getErrorStream()));
        replIn = new BufferedWriter(new OutputStreamWriter(repl.getOutputStream()));
        
        disOut = new DataInputStream(repl.getInputStream());
        
        controller = c;
        
        exec(String.format("gui.setdir('%s'); defaults.samples = %d; \n", tmpDir, RVSAMPLES));
        
        
        new Thread(new Runnable() {
            public void run() {
                captureOutput();
            }
        }).start();
        
        new Thread(new Runnable() {
            public void run() {
                try {
                    String line;
                    while ((line = replErr.readLine()) != null) {
                        System.out.println("!!!!!! " + line);
                        controller.notification(Notification.notification(Controller.TYPE_OUTPUT, line, null));
                    }
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        }).start();
        
        
    }
    
    
    public static Instance getInstance(Controller c) {
        try {
            if (! new File(replScript).exists())
                return null;
            
            Instance i = new Instance(c);
            
            return i;
        } catch (IOException ex) {
            Logger.getLogger(Instance.class.getName()).log(Level.SEVERE, null, ex);
            return null;
        }
    }
    
    public void exec(String str) {
        try {
            replIn.write(str + "\n");
            replIn.flush();
        } catch (IOException ex) {
            
        }
    }
    
    public void stop() {
        destroyed = true;
        repl.destroy();
    }
    
    public void complain() {
        complain(null);
    }
    public void complain(String msg) {
        if (! destroyed) {
            String text = "<html><center><img src=\"" + getClass().getResource("/icons/crash.jpg").toString() +"\"><p><br>";
            String humanText = "This Console instance has crashed. You should close the window, or restart the instance.";
            if (msg != null)
                humanText += msg;

            controller.notification(Notification.notification(Controller.TYPE_OUTPUT, humanText, null));

            JOptionPane.showMessageDialog(null, text+humanText, "X_X", JOptionPane.ERROR_MESSAGE);
        }
    }
    
    public String readLine() throws IOException {
        ByteArray ba = new ByteArray();
        int b = disOut.read();
        while ((byte) b != '\n') {
            if (b == -1)
                return null;
            ba.append((byte) b);
            b = disOut.read();
        }
        
        return new String(ba.array(), 0, ba.length(), Charset.defaultCharset());
    }
    
    public double[][] readBinaryMatrix(double[][] buf) throws IOException {
        String line = readLine();
        Scanner sc = new Scanner(line);
        
        int rows = sc.nextInt();
        
        if (rows < 0)
            return null;
        int cols = sc.nextInt();
        
        if (buf == null || buf.length != rows || buf[0].length != cols)
            buf = new double[rows][cols];
        
        for (int i = 0; i < rows; i++)
            for (int j = 0; j < cols; j++) {
                buf[i][j] = EndianUtils.readSwappedDouble(disOut);
            }
        
        
        return buf;
        
    }
    
    public void signalStop() {
        try {
            new File(tmpDir + "_stop").createNewFile();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
    public double[][] readAsciiMatrix(double[][] buf) throws IOException {
        Scanner sc = new Scanner(readLine());
        int rows = sc.nextInt();
        if (rows < 0)
            return null;
        
        int cols = sc.nextInt();
        
        
        if (buf == null || buf.length != rows || buf[0].length != cols)
            buf = new double[rows][cols];
        
        for (int i = 0; i < rows; i++) {
            Scanner l = new Scanner(readLine());
            for (int j = 0; j < cols; j++)
                buf[i][j] = l.nextDouble();
        }
        return buf;
    }
    
    
    public int[][] readAsciiIntMatrix(int[][] buf) throws IOException {
        Scanner sc = new Scanner(readLine());
        int rows = sc.nextInt();
        if (rows < 0)
            return null;
        
        int cols = sc.nextInt();
        
        
        if (buf == null || buf.length != rows || buf[0].length != cols)
            buf = new int[rows][cols];
        
        for (int i = 0; i < rows; i++) {
            Scanner l = new Scanner(readLine());
            for (int j = 0; j < cols; j++)
                buf[i][j] = l.nextInt();
        }
        return buf;
    }
    
    public int[] readAsciiIntVector(int[] buf) throws IOException {
        Scanner sc = new Scanner(readLine());
        int rows = sc.nextInt();
        if (rows < 0)
            return null;
        
        if (buf == null || buf.length != rows)
            buf = new int[rows];
        
        Scanner l = new Scanner(readLine());
        for (int j = 0; j < rows; j++)
            buf[j] = l.nextInt();
    
        return buf;
    }
    
    private void captureOutput() {
        
            try {
                String line;
                while ((line = readLine()) != null) {
                    Notification ev;
                    
                    if (line.startsWith("@@@@")) {
                        String[] data = line.split(",");
                        
                        if (data[1].trim().equals("catz")) {
                            JOptionPane.showMessageDialog(null, "<html><center><img src=\"" + getClass().getResource("/icons/easter.png").toString() +"\">");
                            continue;
                        }
                        
                        ev = Notification.notification(data[1].trim(), null, 
                                (data.length >= 3 ? data[2].trim() : ""));
                    } else {
                        ev = Notification.notification(Controller.TYPE_OUTPUT, line, null);
                    }
                    
                    controller.notification(ev);
                    
                }
                complain();
            } catch (Exception ex) {
                ex.printStackTrace();
                complain(ex.toString());
                return;
            }
        
    }
}
