/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package uicomponents;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import utils.*;
import utils.Utils;
import repl.*;
import icons.Icons;


/**
 *
 * @author stefano
 */


class BrowserCellRenderer extends JLabel implements ListCellRenderer {
    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
        setOpaque(true);
        String lab = value.toString();
        setText(lab);
        
        if (isSelected) {
            setBackground(SystemColor.textHighlight);
            setForeground(SystemColor.textHighlightText);
        } else {
            setBackground(SystemColor.text);
            setForeground(SystemColor.textText);
        }
        
        if (lab.endsWith("sys"))
            setFont(getFont().deriveFont(Font.BOLD));
        else
            setFont(getFont().deriveFont(Font.PLAIN));
        
        if (lab.endsWith("sys"))
            setIcon(Icons.sys);
        else if (lab.endsWith("tds") || lab.endsWith("tt"))
            setIcon(Icons.ttv);
        else if (lab.endsWith("vels"))
            setIcon(Icons.rv);
        return this;
    }
}

public class Browser extends javax.swing.JPanel {
    private Controller controller;
    private repl.Console console;
    private DefaultListModel model;
    private WeakHashMap<String, File[]> paths = new WeakHashMap<String, File[]>();
    private ArrayList<File> fileList = new ArrayList<File>();
    private String current = "";
    private ListCellRenderer renderer = new BrowserCellRenderer();
    
    public Browser(Controller c, repl.Console con) {
        controller = c;
        console = con;
        initComponents();
        
        model = new DefaultListModel();
        list.setModel(model);
        list.setCellRenderer(renderer);
        path.setText(Prefs.getValue("BROWSER_DIR", "./datafiles/").trim());
        tab.setSelectedIndex(0);
        init();
    }
    
    public void show() {
        String[] kernels = controller.getKernels();
        kernel.setText(controller.getKernel());
        path.requestFocusInWindow();
    }
    
    private void init() {
        String pathStr = path.getText();
        
        if (pathStr.equals(current))
            return;
        
        model = new DefaultListModel();
        fileList.clear();
        
        File pathFile = new File(pathStr);
        try {
            
            File dir;
            String needle; 
            if (pathFile.isDirectory()) {
                dir = pathFile;
                needle = "";
            } else {
                dir = new File(pathStr).getParentFile();
                needle = pathFile.getName();
            }
            
            
            File[] files = (paths.get(dir.getCanonicalPath()) == null ? dir.listFiles() : paths.get(dir.getCanonicalPath()));
            if (files != null) {
                for (File f : files) {
                    if (! (f.getName().endsWith(".sys") || f.getName().endsWith(".tds") || f.getName().endsWith(".vels"))) 
                        continue;
                    if (! f.getName().toLowerCase().startsWith(needle.toLowerCase()))
                        continue;

                    model.addElement(f.getName());
                    fileList.add(f);
                }
            }
            
            Prefs.setValue("BROWSER_DIR", pathStr);
            
            paths.put(dir.getCanonicalPath(), files);
            
        } catch (Exception ex) {
            model.clear();
            model.addElement("<no files>");
        }
        list.setModel(model);
        list.setSelectedIndex(0);
        current = pathStr;
        
        path.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0), "tab");
        path.getActionMap().put("tab", new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                String text = path.getText();
                if (text.startsWith("~"))
                    text = System.getenv("HOME") + text.substring(1);
                
                text = AutoComplete.complete(text);
                path.setText(text.trim());
                path.setSelectionStart(text.length()-1);
                path.setSelectionEnd(text.length()-1);
            }
        });
        
    }
    
    private void load() {
        File path = fileList.get(list.getSelectedIndex());
        try {
            if (list.getSelectedIndex() == -1)
                return;
            
            
            String[] dataFiles = null;
            
            StringBuilder cmd = new StringBuilder();
            String[] kernels = controller.getKernels();
            String kname = kernel.getText();
            
            if (! Utils.find(kernels, kname))
                cmd.append(String.format("%s = kernel.new(); ", kname));
            
            if (path.getName().endsWith("sys")) {
                SystemicData data = SystemicData.readFromFile(path.getPath(),
                    path.getParent());
                dataFiles = data.data;
                cmd.append(String.format("%s:remove_data(); ", kname));    
                if (data.Mstar != null)
                    cmd.append(String.format("%s:mstar(%f); ", kname, data.Mstar));
                if (data.epoch != null)
                    cmd.append(String.format("%s:epoch(%f); ", kname, data.epoch));
            } else {
                dataFiles = new String[] {path.getPath()};
            }
            
            cmd.append(String.format("%s:add_data{", kname));
            
            for (String fn : dataFiles) {
                cmd.append(String.format("\"%s\", ", fn));
            }
            cmd.append("}; ");
            
            console.getTerminal().exec(cmd.toString());
            console.switchRightSidebar("rvs");
            controller.setKernel(kname);
        } catch (FileNotFoundException ex) {
            controller.message("!! Could not open " + path.getPath() + "\n" + ex.toString());
        } catch (IOException ex) {
            controller.message("!! Could not open " + path.getPath() + "\n" + ex.toString());
        }
    }
    
    private void selectionChanged() {
        int idx = list.getSelectedIndex();
        File f = fileList.get(idx);
        
        plot.beginUpdate();
        plot.clear();
        try {
            
            String[] paths = null;
            if (f.getName().endsWith("sys")) {
                SystemicData data = SystemicData.readFromFile(f.getPath(), f.getParent());
                paths = data.data;
            } else {
                paths = new String[] {f.getPath()};
            }
            dataText.setText(FileUtils.get_contents(f.getPath()));
            dataText.setSelectionStart(0);
            dataText.setSelectionEnd(0);
            
            int i = 1;
            for (String fn : paths) {
                double[][] table = Utils.readTable(fn);
                if (fn.endsWith("vels")) {
                    plot.addData(table, fn);
                    plot.addLabel(String.format("%s [%d]", new File(fn).getName(), table.length),
                        100, 12 * i + 10, true).setFont(new Font("sans serif", Font.PLAIN, 10));
                    i++;
                }
            }
            plot.setHighlight(1);
            for (String fn : paths) {
                double[][] table = Utils.readTable(fn);
                if (!fn.endsWith("vels")) {
                    plot.addLabel(String.format("%s [%d]", new File(fn).getName(), table.length),
                        100, 12 * i + 10, true).setFont(new Font("sans serif", Font.PLAIN, 10));
                    i++;
                }
            }
        } catch (FileNotFoundException ex) {
            plot.setWarning("Could not open " + f.getPath());
        } catch (IOException ex) {
            plot.setWarning("Could not open " + f.getPath());
        }
        
        plot.endUpdate();
    }
    
    

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        list = new javax.swing.JList();
        tab = new javax.swing.JTabbedPane();
        jPanel5 = new javax.swing.JPanel();
        plot = new plotting.FoldPlot();
        jScrollPane2 = new javax.swing.JScrollPane();
        dataText = new javax.swing.JTextArea();
        jPanel2 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        path = new javax.swing.JTextField();
        jMacButton1 = new controls.JMacButton();
        jPanel3 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        kernel = new javax.swing.JTextField();
        filler1 = new javax.swing.Box.Filler(new java.awt.Dimension(0, 0), new java.awt.Dimension(0, 0), new java.awt.Dimension(32767, 0));
        loadButton = new javax.swing.JButton();

        setBackground(new java.awt.Color(238, 238, 239));
        setLayout(new java.awt.BorderLayout());

        jPanel1.setLayout(new java.awt.BorderLayout());

        jScrollPane1.setBorder(javax.swing.BorderFactory.createMatteBorder(1, 0, 1, 0, new java.awt.Color(0, 0, 0)));

        list.setModel(new javax.swing.AbstractListModel() {
            String[] strings = { "Item 1", "Item 2", "Item 3", "Item 4", "Item 5" };
            public int getSize() { return strings.length; }
            public Object getElementAt(int i) { return strings[i]; }
        });
        list.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                listValueChanged(evt);
            }
        });
        list.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                listFocusGained(evt);
            }
        });
        jScrollPane1.setViewportView(list);

        jPanel1.add(jScrollPane1, java.awt.BorderLayout.NORTH);

        tab.setBackground(new java.awt.Color(239, 238, 238));
        tab.setOpaque(true);

        jPanel5.setLayout(new java.awt.BorderLayout());

        plot.setMinimumSize(new java.awt.Dimension(189, 190));
        jPanel5.add(plot, java.awt.BorderLayout.CENTER);

        tab.addTab("Plot", jPanel5);

        dataText.setColumns(20);
        dataText.setEditable(false);
        dataText.setFont(new java.awt.Font("Monospaced", 0, 14)); // NOI18N
        dataText.setRows(5);
        jScrollPane2.setViewportView(dataText);

        tab.addTab("Data", jScrollPane2);

        jPanel1.add(tab, java.awt.BorderLayout.CENTER);

        add(jPanel1, java.awt.BorderLayout.CENTER);

        jPanel2.setBackground(new java.awt.Color(239, 238, 238));
        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.LINE_AXIS));

        jLabel1.setText(" System: ");
        jPanel2.add(jLabel1);

        path.setText("./datafiles/");
        path.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                pathActionPerformed(evt);
            }
        });
        path.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                pathKeyPressed(evt);
            }
        });
        jPanel2.add(path);

        jMacButton1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/icons/folder.png"))); // NOI18N
        jMacButton1.setButtonType("gradient");
        jMacButton1.setEnabled(false);
        jPanel2.add(jMacButton1);

        add(jPanel2, java.awt.BorderLayout.PAGE_START);

        jPanel3.setBackground(new java.awt.Color(238, 238, 239));
        jPanel3.setBorder(javax.swing.BorderFactory.createMatteBorder(1, 0, 0, 0, new java.awt.Color(0, 0, 0)));
        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.LINE_AXIS));

        jLabel2.setText(" Load in:");
        jPanel3.add(jLabel2);

        kernel.setColumns(5);
        kernel.setText("k");
        kernel.setMaximumSize(new java.awt.Dimension(84, 20));
        kernel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                kernelActionPerformed(evt);
            }
        });
        jPanel3.add(kernel);
        jPanel3.add(filler1);

        loadButton.setText("Load");
        loadButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadButtonActionPerformed(evt);
            }
        });
        jPanel3.add(loadButton);

        add(jPanel3, java.awt.BorderLayout.PAGE_END);
    }// </editor-fold>//GEN-END:initComponents

    private void pathActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_pathActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_pathActionPerformed

    private void pathKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_pathKeyPressed
        
        if (evt.getKeyCode() == KeyEvent.VK_DOWN) { 
            int idx = list.getSelectedIndex();
            if (idx != -1 && idx < model.size() - 1)
                list.setSelectedIndex(idx + 1);
            list.ensureIndexIsVisible(list.getSelectedIndex());
            evt.consume();
        } else if (evt.getKeyCode() == KeyEvent.VK_UP) {
            int idx = list.getSelectedIndex();
            if (idx != -1 && idx > 0)
                list.setSelectedIndex(idx - 1);
            list.ensureIndexIsVisible(list.getSelectedIndex());
            evt.consume();
        } else if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
            load();
        } else {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    init();
                }
            });
            
        }
    }//GEN-LAST:event_pathKeyPressed

    private void listFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_listFocusGained
        path.requestFocusInWindow();
    }//GEN-LAST:event_listFocusGained

    private void listValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_listValueChanged
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                selectionChanged();
            }
        });
    }//GEN-LAST:event_listValueChanged

    private void loadButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadButtonActionPerformed
        load();
    }//GEN-LAST:event_loadButtonActionPerformed

    private void kernelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_kernelActionPerformed
        load();
    }//GEN-LAST:event_kernelActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea dataText;
    private javax.swing.Box.Filler filler1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private controls.JMacButton jMacButton1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTextField kernel;
    private javax.swing.JList list;
    private javax.swing.JButton loadButton;
    private javax.swing.JTextField path;
    private plotting.FoldPlot plot;
    private javax.swing.JTabbedPane tab;
    // End of variables declaration//GEN-END:variables
}
