/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package uicomponents;

import controls.*;
import icons.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.Border;
import repl.Console;
import repl.*;
import utils.Notification;
import utils.NotificationListener;
import utils.UIUtils;

/**
 *
 * @author stefano
 */


public class DataOffsetsPanel extends JLayeredPane implements NotificationListener, ComponentListener {

    static int ndatasets = 16;
    private boolean[] active = new boolean[ndatasets];
    private JPanel leftHeader;
    private JPanel[] panels;
    private Controller controller;
    private JTextField[] shifts;
    private SmallSlider[] sliders;
    private JMacButton[] btnDataNames = new JMacButton[ndatasets];
    private JButton[] dots = new JButton[ndatasets];
    
    private Console console;
    private Color bg = UIUtils.defColor;
    private Border borderNormal = BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.gray),
            BorderFactory.createEmptyBorder(4, 4, 4, 4));
    private Border borderHighlight = BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.gray, 1),
            BorderFactory.createEmptyBorder(4, 4, 4, 4));
    private JScrollPane scroller;
    private Map<String, Set<String>> dataFiles = new HashMap<String, Set<String>>();
    
    static NumberFormat numberFormat1 = NumberFormat.getNumberInstance();
    
    private DataOffsetsPanel() {
    }

    public DataOffsetsPanel(Controller c, Console cons) {
        controller = c;
        console = cons;

        
        numberFormat1.setMaximumFractionDigits(4);
        numberFormat1.setMinimumFractionDigits(0);
        
        scroller = new JScrollPane();
        JPanel cont = new JPanel();
        cont.setLayout(new BoxLayout(cont, BoxLayout.LINE_AXIS));
        cont.setOpaque(true);
        cont.setBackground(bg);

        panels = new JPanel[ndatasets];
        shifts = new JTextField[ndatasets];
        sliders = new SmallSlider[ndatasets];
        
        for (int j = 0; j < ndatasets; j++) {
            final int np = j;

            panels[j] = new JPanel();
            panels[j].setPreferredSize(new Dimension(150, 60));
            panels[j].setMaximumSize(new Dimension(150, 60));
            
            panels[j].setLayout(new GridBagLayout());
            
            GridBagConstraints co = new GridBagConstraints(0, 0, 3, 1, 0, 0, GridBagConstraints.CENTER,
                    GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0);
            
            //JPanel top = new JPanel();
            btnDataNames[j] = new JMacButton();
            btnDataNames[j].setButtonType("roundRect");
            btnDataNames[j].setSmall(true);

            btnDataNames[j].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    console.getTerminal().exec(String.format("%s:remove_data(%d)", controller.getKernel(), np + 1));
                }
            });

            panels[j].add(btnDataNames[j], co);

            
            co.gridx = 0;
            co.gridy = 1;
            co.gridwidth = 1;
            co.weightx = 0.1;
            dots[j] = new JButton(Icons.grayDot);
            dots[j].setSelectedIcon(Icons.greenDot);
            dots[j].setBorder(null);

            dots[j].addMouseListener(new MouseListener() {
                public void mouseClicked(MouseEvent e) {
                    if (e.isPopupTrigger()) {

                    } else {
                        toggleMinimize(np);
                    }
                };

                public void mousePressed(MouseEvent e) {}
                public void mouseReleased(MouseEvent e) {}
                public void mouseEntered(MouseEvent e) {}
                public void mouseExited(MouseEvent e) {}
            });
            panels[j].add(dots[j], co);
            
            co.gridy = 1;
            co.gridx = 1;
            co.weightx = 0.8;
            shifts[j] = new PersistentTextField();
            panels[j].add(shifts[j], co);
            
            shifts[j].addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    Double val = Double.valueOf(shifts[np].getText());
                    console.getTerminal().exec(String.format("%s.offs[%d] = %e",
                            controller.getKernel(),
                            np + 1,
                            val));
                }
            });

            shifts[j].setBackground(bg);
            shifts[j].setBorder(null);
            shifts[j].setHorizontalAlignment(JTextField.CENTER);

            shifts[j].addFocusListener(new FocusListener() {

                public void focusGained(FocusEvent e) {
                    setSelected(np, true);
                }

                public void focusLost(FocusEvent e) {
                    setSelected(np, false);
                }
            });
            
            
            panels[j].setBackground(bg);
            
            co.gridy = 2;
            co.gridx = 0;
            co.gridwidth = 3;
            sliders[j] = new SmallSlider(-2000, 2000, 0, false, new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    Double val = sliders[np].getVal();
                    shifts[np].setText(numberFormat1.format(val));
                    if (e.getActionCommand().equals("released"))
                        console.getTerminal().exec(String.format("%s.offs[%d] = %e",
                            controller.getKernel(),
                            np + 1,
                            val), ! e.getActionCommand().equals("released"));
                }
            });
            sliders[j].setPreferredSize(new Dimension(150, 10));
            sliders[j].setMaximumSize(new Dimension(150, 10));
            sliders[j].setBackground(bg);
            panels[j].add(sliders[j], co);
            
            cont.add(panels[j]);
        }


        scroller.setViewportView(cont);
        scroller.setRowHeaderView(leftHeader);
        scroller.setBorder(null);
        scroller.setBackground(bg);
        
        //setLayout(new BorderLayout());
        add(scroller, JLayeredPane.DEFAULT_LAYER);
        
        setBackground(bg);
        cont.add(Box.createHorizontalGlue());
        controller.addGUIListener(this);
        
        
        
        for (int i = 0; i < ndatasets; i++) {
               disableColumn(i);
        }
        
        addComponentListener(this);
        
    }
    

    public void setSelected(int column, boolean sel) {
        if (sel) {
            shifts[column].setBorder(borderHighlight);
        } else {
            shifts[column].setBorder(borderNormal);
        }
    }

    public void notification(Notification n) {
        
        if (n.is(Controller.TYPE_STATS, Controller.TYPE_UPDATE) && n.name.equals(controller.getKernel())) {
            
            Map<String, Double> pairs = controller.getStatsForKernel(n.name);
            if (pairs == null) {
                for (int i = 0; i < ndatasets; i++) {
                    disableColumn(i);
                }
                return;
            }

            int nsets = pairs.get("nsets").intValue();
            
            for (int i = 0; i < nsets; i++) {
                enableColumn(i);
            }
            for (int i = nsets; i < ndatasets; i++) {
                disableColumn(i);
            }
            
            for (Map.Entry<String, Double> entry : pairs.entrySet()) {
                if (entry.getKey().contains("!")) {
                    String[] sp = entry.getKey().split("!");
                    
                    if (dataFiles.get(controller.getKernel()) == null)
                        dataFiles.put(controller.getKernel(), new HashSet<String>());
                    
                    dataFiles.get(controller.getKernel()).add(sp[1]);
                    set(Integer.valueOf(sp[0]), sp[1], entry.getValue());
                }
            }

            int[] flags = controller.getParFlagsForKernel(controller.getKernel());
            for (int i = 0; i < flags.length; i++)
                dots[i].setSelected((flags[i] & Defs.MINIMIZE) != 0);
            
            double[][] data = controller.getDataForKernel(controller.getKernel(), -1);
            if (data != null)
            for (int j = 0; j < nsets; j++) {
                for (int i = 0; i < data.length; i++)
                    if (Math.floor(data[i][Defs.T_SET]) == j) {
                        if (Math.floor(data[i][Defs.T_FLAG]) == Defs.T_RV)
                            btnDataNames[j].setIcon(Icons.rv);
                        else if (Math.floor(data[i][Defs.T_FLAG]) == Defs.T_TIMING)
                            btnDataNames[j].setIcon(Icons.ttv);
                        else
                            btnDataNames[j].setIcon(null);
                    }
            }
            
        }

    }

    private void enableColumn(int which) {
        active[which] = true;
        for (Component c : panels[which].getComponents())
            c.setVisible(true);
        panels[which].setVisible(true);
    }

    private void disableColumn(int which) {
        active[which] = false;
        for (Component c : panels[which].getComponents())
            c.setVisible(false);
        panels[which].setVisible(false);
    }

    private void set(int column, String dataName, double value) {
        btnDataNames[column-1].setText(new File(dataName).getName());
        shifts[column-1].setText(numberFormat1.format(value));
        sliders[column-1].setVal(value);
    }
    
    
    private void popupChoices(JComponent next) {
        JPopupMenu pop = new JPopupMenu();
        
        Set<String> files = dataFiles.get(controller.getKernel());
        final String k = controller.getKernel();
        if (files != null) {
            for (final String file : files) {
                JMenuItem mi = pop.add(file);
                
                mi.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        console.getTerminal().exec(String.format("%s:add_data('%s')",
                                k, file));
                    }
                });
            }
            pop.addSeparator();
        }
        
        JMenuItem mi = pop.add("Browse...");
        mi.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                console.switchRightSidebar("browser");
            }
        });
        
        pop.show(next, 0, next.getHeight());
    }
    
    
    public void componentResized(ComponentEvent e) {
        scroller.setBounds(new Rectangle(0, 0, getWidth(), getHeight()));

    }

    public void componentMoved(ComponentEvent e) {
        
    }

    public void componentShown(ComponentEvent e) {
        
    }

    public void componentHidden(ComponentEvent e) {
        
    }
    
    
    private void toggleMinimize(int data) {
        console.getTerminal().exec(String.format("%s:flag('offs', %d, bxor(%s:flag('offs', %d), MINIMIZE))", controller.getKernel(),
            data + 1, controller.getKernel(), data+1));
    }
    
    public void setContainer(palette.Container c) {
        final JButton btn = new JButton(Icons.add);
        btn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                popupChoices(btn);
            }
        });
        btn.setBorder(null);
        c.getToolbar().add(btn);
        c.getToolbar().add(Box.createHorizontalStrut(10));
    }
    
}
