/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package uicomponents;

import controls.*;
import icons.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import javax.swing.*;
import javax.swing.border.Border;
import numal.*;
import repl.*;
import utils.*;
import utils.Notification;
import utils.NotificationListener;
import utils.UIUtils;

/**
 *
 * @author stefano
 */


public class ElementsPanel extends JScrollPane implements NotificationListener {
    static int nelem = 8;
    static String[] names = new String[] {"period", "mass", "ma", "ecc",
        "lop", "inc", "node", "radius"};
    static String[] displayNames = new String[] {"Period", "Mass",
        "Mean anomaly", "Eccentricity", "Long. Peri",
        "Inclination",
        "Node", "Radius"
    };
    private boolean dragging = false;
    private static double[] minmax = new double[] {0.1, 10000,
        1e-5, 30, 0, 360, 0, 0.999, 0, 360,
        0, 360, 0, 360, 1e-5, 30
    };
    private static boolean[] log = new boolean[] { true, true,
        false, false, false, false, false, true
    };
    
    static NumberFormat numberFormat1 = NumberFormat.getNumberInstance();
    static NumberFormat numberFormat2 = NumberFormat.getNumberInstance();
    
    static int nplanets = 6;
    private boolean[] active = new boolean[nplanets];
    private JPanel leftHeader;
    
    private JPanel[] panels;
    private Controller controller;
    private JTextField[][] fields;
    private SmallSlider[][] sliders;
    private JButton[][] dots;
    private JMacButton[] btnPlanets = new JMacButton[nplanets];
    
    private Console console;
    private Color bg = UIUtils.defColor;
    private Color bgHighlight = bg.brighter();
    private Border borderHighlight = BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.gray, 1), 
            BorderFactory.createEmptyBorder(0, 4, 0, 4));
    private Border borderNormal = BorderFactory.createCompoundBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.gray),
            BorderFactory.createEmptyBorder(0, 4, 0, 4));
    
    private ElementsPanel() {
        
    }
    
    public ElementsPanel(final Controller c, final Console cons) {
        controller = c;
        console = cons;
        
        JPanel cont = new JPanel();
        cont.setLayout(new BoxLayout(cont, BoxLayout.LINE_AXIS));
        cont.setOpaque(true);
        cont.setBackground(bg);
        
        numberFormat1.setMaximumFractionDigits(4);
        numberFormat1.setMinimumFractionDigits(0);
        numberFormat2.setMaximumFractionDigits(4);
        numberFormat2.setMinimumFractionDigits(0);
        numberFormat2.setGroupingUsed(false);
        addMouseListener(new MouseAdapter() {
            public void mouseEntered(MouseEvent e) {
                //ElementsPanel.this.getHorizontalScrollBar().setUI();
            }

            public void mouseExited(MouseEvent e) {
                
                //ElementsPanel.this.getHorizontalScrollBar().setUI(null);
            }
        });
        
        
        leftHeader = new JPanel(new GridLayout(nelem, 1));
        leftHeader.setBackground(bg);
        leftHeader.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 10));
        
        
        
        JPanel topHeader = new JPanel();
        topHeader.setBackground(bg);
        topHeader.setLayout(new BoxLayout(topHeader, BoxLayout.LINE_AXIS));
        
        for (int i = 0; i < nelem; i++) {
            JLabel lab = new JLabel(displayNames[i]);
            lab.setFont(lab.getFont().deriveFont(Font.BOLD, 10f));
            lab.setForeground(UIUtils.sidebarHeaderColor);
            lab.setHorizontalAlignment(JLabel.RIGHT);
            lab.setOpaque(true);
            lab.setBackground(bg);
            leftHeader.add(lab);
        }
        
        panels = new JPanel[nplanets];
        fields = new JTextField[nplanets][];
        sliders = new SmallSlider[nplanets][];
        dots = new JButton[nplanets][];
        
        for (int j = 0; j < nplanets; j++) {
            final int np = j;
            
            panels[j] = new JPanel(new GridLayout(nelem, 1));
            panels[j].setMaximumSize(new Dimension(100, 32768));
            panels[j].setMinimumSize(new Dimension(100, 1));
            
            //JPanel top = new JPanel();
            btnPlanets[j] = new JMacButton("Planet " + (j+1));
            btnPlanets[j].setButtonType("roundRect");
            btnPlanets[j].setSmall(true);
            btnPlanets[j].setMaximumSize(new Dimension(100, 18));
            btnPlanets[j].setMinimumSize(new Dimension(100, 18));
            btnPlanets[j].setPreferredSize(new Dimension(100, 18));
            
            btnPlanets[j].addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (!active[np]) {
                        addPlanet((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
                    } else {
                        console.getTerminal().exec(String.format("%s:remove(%d)", controller.getKernel(), np+1));
                    }
                    console.defaultFocus();
                }
            });
            
            
            //top.add(btnPlanets[j]);
            //top.setBackground(bg);
            //panels[j].add(btnPlanets[j]);
            
            topHeader.add(btnPlanets[j]);
            
            fields[j] = new JTextField[nelem];
            sliders[j] = new SmallSlider[nelem];
            dots[j] = new JButton[nelem];
            
            for (int i = 0; i < nelem; i++) {
               fields[j][i] = new PersistentTextField();
               fields[j][i].setBorder(borderNormal);
               fields[j][i].setBackground(bg);
               //fields[j][i].setMinimumSize(new Dimension(150-30, ));
               final int ne = i;
               
               fields[j][i].addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        Double val = Double.valueOf(fields[np][ne].getText());
                        if (!dragging)
                        sliders[np][ne].setVal(val);
                        console.getTerminal().exec(String.format("%s.%s[%d] = %e",
                                controller.getKernel(),
                                names[ne],
                                np + 1,
                                val));
                    }
               });
               
               
               sliders[j][i] = new SmallSlider(minmax[ne*2], minmax[ne*2+1], 5, log[ne], new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        Double val = sliders[np][ne].getVal();
                        fields[np][ne].setText(numberFormat1.format(val));
                        if (e.getActionCommand().equals("released")) {
                            dragging = false;
                            console.getTerminal().exec(String.format("%s.%s[%d] = %e",
                                controller.getKernel(),
                                names[ne],
                                np + 1,
                                val), ! e.getActionCommand().equals("released"));
                        }/*
                        if (e.getActionCommand().equals("moved")) {
                            dragging = true;
                            console.getTerminal().exec(String.format("%s.%s[%d] = %e",
                                controller.getKernel(),
                                names[ne],
                                np + 1,
                                val), ! e.getActionCommand().equals("released"));
                        }*/
                        }
               });
               
               sliders[j][i].setFocusable(false);
               
               
               
               sliders[j][i].setBackground(bg);
               sliders[j][i].setForeground(Color.GRAY);
               
               dots[j][i] = new JButton(Icons.grayDot);
               
               dots[j][i].setSelectedIcon(Icons.greenDot);
               dots[j][i].setBorder(null);
               dots[j][i].setFocusable(false);
               dots[j][i].addMouseListener(new MouseListener() {
                    public void mouseClicked(MouseEvent e) {
                        if (e.isPopupTrigger()) {
                            
                        } else {
                            toggleMinimize(np, ne);
                        }
                    };

                    public void mousePressed(MouseEvent e) {}
                    public void mouseReleased(MouseEvent e) {}
                    public void mouseEntered(MouseEvent e) {}
                    public void mouseExited(MouseEvent e) {}
               });
               
               final JButton act = new JButton(Icons.rightArrow);
               act.setBorder(null);
               act.setMaximumSize(act.getSize());
               act.setMinimumSize(act.getSize());
               act.setFocusable(false);
               act.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        PopOver p = PopOver.getPopOver(act, console.getFrame().getPopupLayer());
                        setupPopOver(p, np, ne);
                    }
               });
               
               final JPanel couple = new JPanel(new BorderLayout(0, 0));
               couple.setBackground(bg);
               
               couple.add(fields[j][i], BorderLayout.CENTER);
               couple.add(dots[j][i], BorderLayout.WEST);
               couple.add(sliders[j][i], BorderLayout.SOUTH);
               couple.add(act, BorderLayout.EAST);
               couple.setVisible(false);
               couple.setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
               couple.setPreferredSize(new Dimension(100, 1));
               act.addMouseListener(fields[j][i].getMouseListeners()[0]);
               
               panels[j].add(couple);
            }
            
            panels[j].setBackground(bg);
            
            
            cont.add(panels[j]);
        }
        topHeader.add(Box.createHorizontalGlue());
        
        cont.add(Box.createHorizontalGlue());
        
        setViewportView(cont);
        setRowHeaderView(leftHeader);
        setColumnHeaderView(topHeader);
        setBackground(bg);
        setBorder(null);
        
        controller.addGUIListener(this);
    }
    
    private int currentp;
    private int currente;
    private JPanel popContents;
    private JMacButton minimize1d;

    private void setupPopOver(PopOver p, int np, int ne) {
        
        if (popContents == null) {
            popContents = new JPanel();
            popContents.setOpaque(false);
            popContents.setLayout(new GridBagLayout());
            
            minimize1d = new JMacButton("Minimize");
            minimize1d.setButtonType("segmentedTextured");
            minimize1d.setSegmentPosition(-1);
            
            minimize1d.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    console.getTerminal().exec(String.format("%s:minimize1d(%d, '%s')",
                            controller.getKernel(), currentp + 1, names[currente]));
                }
            });
            
            popContents.add(minimize1d, new GridBagConstraints(0, 0, 2, 1, 1, 0., GridBagConstraints.NORTH,
                    GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
        }
        
        currentp = np;
        currente = ne;
        
        p.removeAll();
        p.add(popContents, BorderLayout.CENTER);
    }

    
    public void setSelected(int row, int column, boolean sel) {
        if (sel) {
            fields[column][row].setBorder(borderHighlight);
        } else {
            fields[column][row].setBorder(borderNormal);
        }
    }
    
    public void notification(Notification n) {
        if (n.is(Controller.TYPE_STATE, Controller.TYPE_UPDATE) && n.name.equals(controller.getKernel())) {
            double[][] elements = controller.getElementsForKernel(n.name);
            int[][] flags = controller.getElFlagsForKernel(n.name);
            
            if (elements == null) {
                for (int i = 0; i < nplanets; i++)
                    disableColumn(i);
                return;
            }
            
            int np = elements.length;
            for (int i = 1; i < np; i++)
                enableColumn(i-1);
            for (int i = np; i <= nplanets; i++)
                disableColumn(i-1);
            
            btnPlanets[np-1].setVisible(true);
            panels[np-1].setVisible(true);
            
            for (int i = 1; i < elements.length; i++) 
                for (int j = 0; j < nelem; j++) {
                    set(i-1, j, elements[i][j]);
                    dots[i-1][j].setSelected((flags[i][j] & Defs.MINIMIZE) != 0);
                }
        } 
        
        
    }
    
    public void enableColumn(int which) {
        active[which] = true;
        btnPlanets[which].setSelected(true);
        for (Component c : panels[which].getComponents())
            c.setVisible(true);
        panels[which].setVisible(true);
    }
    
    public void disableColumn(int which) {
        active[which] = false;
        btnPlanets[which].setSelected(false);
        for (Component c : panels[which].getComponents())
            if (c != btnPlanets[which])
                c.setVisible(false);
        btnPlanets[which].setVisible(false);
        panels[which].setVisible(false);
    }
    
    public void set(int row, int column, double value) {
        fields[row][column].setText(numberFormat1.format(value));
        sliders[row][column].setVal(value);
    }
    
    public void toggleMinimize(int pl, int el) {
        console.getTerminal().exec(String.format("%s:flag(%d, '%s', bxor(%s:flag(%d, '%s'), MINIMIZE))", controller.getKernel(),
            pl + 1, names[el], controller.getKernel(), pl+1, names[el]));
    }
    
    public void addPlanet(boolean minimizeAll) {
        String cmd;
        double[][] data = controller.getDataForKernel(controller.getKernel(), Defs.T_RV);
        
        if (data == null || controller.getPeriodogramForKernel(controller.getKernel()) == null || true) {
            console.getTerminal().exec(String.format("%s:add{period=300, mass=1, ma = 0, ecc = 0, lop = 0}",
                                controller.getKernel()));
        } else if (controller.getPerResidualsForKernel(controller.getKernel()) == null) {
            double[][] per = controller.getPeriodogramForKernel(controller.getKernel());
            
            double mstar = controller.getStatsForKernel(controller.getKernel()).get("mstar");
            int idx = Utils.amaxi(per, Defs.PS_Z);
            double P = per[idx][Defs.PS_TIME];
            double MA = per[idx][Defs.PS_TAU] * 180./Math.PI;
            double K = NumUtils.stdDev(Utils.column(data, Defs.T_SVAL));
            double M = Math.pow(mstar * Defs.MSUN, 2./3.) * Math.pow(P * Defs.DAY, 1./3.) *
                    Math.pow(2*Math.PI * Defs.GGRAV, -1./3.) * K * 100. / Defs.MJUP;
            
            console.getTerminal().exec(String.format("%s:add{period=%s, mass = %s, ma = 0, ecc = 0, lop = 0};\n%s:minimize1d(1, 'ma')",
                    controller.getKernel(), numberFormat2.format(P), numberFormat2.format(M), controller.getKernel(),
                    (minimizeAll ? "()": "1d(1, 'ma')")));
        } else if (controller.getPerResidualsForKernel(controller.getKernel()) != null) {
            double[][] per = controller.getPerResidualsForKernel(controller.getKernel());
            double mstar = controller.getStatsForKernel(controller.getKernel()).get("mstar");
            int idx = Utils.amaxi(per, Defs.PS_Z);
            
            double P = per[idx][Defs.PS_TIME];
            double MA = per[idx][Defs.PS_TAU] * 180./Math.PI;
            double K = NumUtils.stdDev(Utils.column(data, Defs.T_SVAL));
            double M = Math.pow(mstar * Defs.MSUN, 2./3.) * Math.pow(P * Defs.DAY, 1./3.) *
                    Math.pow(2*Math.PI * Defs.GGRAV, -1./3.) * K * 100. / Defs.MJUP;
            
            console.getTerminal().exec(String.format("%s:add{period=%s, mass = %s, ma = 0, ecc = 0, lop = 0}",
                    controller.getKernel(), numberFormat2.format(P), numberFormat2.format(M)));
            if (minimizeAll)
                console.getTerminal().exec(String.format("%s:minimize()", controller.getKernel()));
            else
                console.getTerminal().exec(String.format("%s:minimize1d(%s:nplanets(), 'ma')", controller.getKernel(), 
                        controller.getKernel()));
        }
        
        fields[controller.getNPlanetsForKernel(controller.getKernel())][0].requestFocusInWindow();
    }

    public void setContainer(palette.Container c) {
        JPanel toolbar = c.getToolbar();
        
    }
    
}
