/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package uicomponents;

import java.awt.*;
import plotting.*;
import repl.*;
import utils.*;
/**
 *
 * @author stefano
 */
public class Periodogram extends FoldPlot implements FoldPlotPainter {
    private double[][] data;
    private double[][] peaks;
    private Integer[] sort;
    
    public Periodogram() {
        super();
        setSymmetric(false);
        setXLog(true); 
        setDraggable(false);
        setPainter(this);
    }
    
    public void setPeriodogramData(double[][] d) {
        data = d;
        try {
            beginUpdate();
            clear();
            if (data != null)
                addCurve(data);
            
            findPeaks();
            findBounds();
            
        } catch (Exception e) {
        }
        finally {
            endUpdate();
            
        }
    }
    
    private void findPeaks() {
        Table peaksTable = new Table();
        
        for (int i = 1; i < data.length - 1; i++)
            if (data[i][Defs.PS_Z] >= data[i-1][Defs.PS_Z] && data[i][Defs.PS_Z] >= data[i+1][Defs.PS_Z])
                peaksTable.add(data[i]);
        
        peaks = peaksTable.toArray();
        sort = Utils.sort(peaks, Defs.PS_Z, false);
        adjusted = false;
    }
    
    private double[][] floaters = new double[20][6];
    private static Font annFont;
    private boolean adjusted = false;
    
    public void draw(FoldPlot plot, Graphics2D g2, double[] xBounds, double[] yBounds, double[] plotXBounds, double[] plotYBounds) {
        int lastX = 0;
        double[] pBounds = getPlotBounds();
        int maxY = 0;
        
        for (int i = 0; i < 20; i++) {
            double P = peaks[sort[i]][Defs.PS_TIME];
            double Z = peaks[sort[i]][Defs.PS_Z];
            double FAP = peaks[sort[i]][Defs.PS_FAP];
            
            double x = plotXToScreenX(Math.log10(P));
            double y = plotYToScreenY(Z);
            floaters[i][0] = x;
            floaters[i][1] = y;
            floaters[i][2] = P;
            floaters[i][3] = Z;
            floaters[i][4] = FAP;
        }
        
        if (annFont == null)
            annFont = getFont().deriveFont(Font.BOLD, 9f);
        
        g2.setFont(annFont);
        for (int i = 0; i < 6; i++) {
            String Ps = String.format("P = %.2e", floaters[i][2]);
            String FAPs = String.format("FAP = %.2e", floaters[i][4]);
            double w1 = g2.getFontMetrics().stringWidth(Ps);
            double w2 = g2.getFontMetrics().stringWidth(FAPs);
            
            double w = Math.max(g2.getFontMetrics().stringWidth(Ps),
                    g2.getFontMetrics().stringWidth(FAPs)) + 20;
            double h = 2 * g2.getFontMetrics().getHeight() + 10;
            int x = (int) floaters[i][0];
            int y = (int) floaters[i][1];
            maxY = Math.max(maxY, (int)floaters[i][3] + (int)((h+15) / (pBounds[3]-pBounds[2]) * (yBounds[1] - yBounds[0])));
            
            g2.setColor(Color.BLACK);
            g2.fillRoundRect((int)(x-0.5*w), (int)(y-h-10), (int)w, (int)h, 5, 5);
            g2.setColor(Color.WHITE);
            g2.drawString(Ps, (int)(x-0.5 * w1), (int)(y-h));
            g2.drawString(FAPs, (int)(x-0.5 * w2), (int)(y-h) + g2.getFontMetrics().getHeight());
        }
        
        if (!adjusted) {
            setYRange(0, maxY + 10);
            adjusted = true;
        }
    }
    
}
