/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package uicomponents;

import controls.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import javax.swing.*;
import utils.*;
import icons.Icons;
import java.awt.event.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import repl.*;
import repl.Controller;

/**
 *
 * @author stefano
 */

class Row extends JPanel {
    private NumberFormat nf = null;
    private JLabel lab;
    private JTextField field;
    private ActionListener act;
    private JPopupMenu po;
    private int type;
    private String[] options;
    public static int EDITABLE = 1;
    public static int LOCKABLE = 2;
    public static int DROPDOWN = 4;
    public static int SEPARATOR = 8;
    public String tag;
    
    public Row(String tag, String title, int type, String[] options, ActionListener a) {
        this.type = type;
        this.options = options;
        this.tag = tag;
        
        setBackground(UIUtils.sidebarColor);
        if ((type & SEPARATOR) == SEPARATOR) {
            setMaximumSize(new Dimension(10000, 10));
            setMinimumSize(new Dimension(1, 10));
            return;
        }
        
        setPreferredSize(new Dimension(100, 15));
        setMaximumSize(new Dimension(1000, 20));
        act = a;
        
        nf = NumberFormat.getInstance();
        nf.setMaximumFractionDigits(4);
        nf.setGroupingUsed(false);
        
        setBackground(UIUtils.sidebarColor);
        lab = new JLabel(title);
        lab.setForeground(UIUtils.sidebarHeaderColor);
        lab.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 3));
        lab.setFont(lab.getFont().deriveFont(11f));
        
        JPanel fieldCont = new JPanel(new BorderLayout());
        fieldCont.setBackground(UIUtils.sidebarColor);
        
        field = new PersistentTextField();
        field.setEditable(false);
        field.setBackground(UIUtils.sidebarColor);
        field.setBorder(null);
        
        if ((type & EDITABLE) == EDITABLE) {
            field.setEditable(true);
            fieldCont.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.gray));
            field.addActionListener(a);
            field.setActionCommand(tag);
        } else if ((type & LOCKABLE) == LOCKABLE) {
            field.setEditable(true);
            fieldCont.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.gray));
            
            JToggleButton btn = new JToggleButton();
            
            btn.setIcon(Icons.unlocked);
            btn.setSelectedIcon(Icons.locked);
            btn.setActionCommand(tag + "_lock");
            
            btn.addActionListener(a);
            
            fieldCont.add(btn, BorderLayout.EAST);
            btn.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
            
            field.addActionListener(a);
            field.setActionCommand(tag);
        } else if ((type & DROPDOWN) == DROPDOWN) {
            final JButton btn = new JButton(Icons.dropArrow);
            btn.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    po.show(btn, 0, btn.getHeight());
                }
            });
            
            field.addMouseListener(new MouseAdapter() {
                @Override
                public void mouseClicked(MouseEvent e) {
                    btn.doClick();
                }
            });
            
            fieldCont.add(btn, BorderLayout.EAST);
            fieldCont.setBorder(BorderFactory.createMatteBorder(0, 0, 1, 0, Color.gray));
            
            po = new JPopupMenu();
            for (int i = 0; i < options.length; i++) {
                JMenuItem mi = po.add(options[i]);
                mi.setActionCommand(tag + "_" + options[i]);
                mi.addActionListener(a);
            }
            
            btn.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
        }
        
        fieldCont.add(field, BorderLayout.CENTER);
        
        setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
        lab.setPreferredSize(new Dimension(90, 15));
        lab.setMinimumSize(new Dimension(90, 15));
        lab.setMaximumSize(new Dimension(90, 15));
        add(lab);
        add(fieldCont);
    }

    public void setValue(double val) {
        if ((type & DROPDOWN) == DROPDOWN) {
            field.removeActionListener(act);
            field.setText(options[(int) val]);
            field.addActionListener(act);
        } else {
            field.removeActionListener(act);
            if (val != val) {
                field.setText("--");
            } else
                field.setText(nf.format(val));
            field.addActionListener(act);
        }
        field.setSelectionStart(0);
    }
    
    public String getValue() {
        return field.getText();
    }
}

public class PropertyPanel extends JPanel implements ActionListener, NotificationListener {
    private Map<String, Row> corr = new HashMap<String, Row>();
    private Controller controller;
    private Console console;
    private PropertyPanel() {};
    
    public static PropertyPanel getInstance(Controller c, Console con) {
        PropertyPanel p = new PropertyPanel();
        p.setBorder(BorderFactory.createMatteBorder(0, 15, 0, 0, UIUtils.sidebarColor));
        p.setLayout(new BoxLayout(p, BoxLayout.PAGE_AXIS));
        p.setBackground(UIUtils.sidebarColor);
        p.addRow("nplanets", "# Planets", 0, null);
        p.addRow("mstar", "<html>Star mass [M<sub>o</sub>]", Row.EDITABLE, null);
        p.addRow("epoch", "Epoch [JD]", Row.LOCKABLE, null);
        p.addRow("", "", Row.SEPARATOR, null);
        
        p.addRow("chi2", "<html>Chi<sup>2</sup><sub>r</sub>", 0, null);
        p.addRow("ndata", "# Data", 0, null);
        p.addRow("nrpars", "Norm. par.", 0, null);
        
        p.addRow("", "", Row.SEPARATOR, null);
        p.addRow("chi2_rvs", "<html>Chi<sup>2</sup><sub>RV</sub>", 0, null);
        p.addRow("rms", "<html>RMS<sub>RV</sub> [m/s]", 0, null);
        p.addRow("jitter", "Jitter [m/s]", 0, null);
        p.addRow("nrvs", "# RVs", 0, null);
        
        p.addRow("", "", Row.SEPARATOR, null);
        p.addRow("chi2_tts", "<html>Chi<sup>2</sup><sub>TT</sub>", 0, null);
        p.addRow("rms_tts", "<html>RMS<sub>TT</sub> [d]", 0, null);
        p.addRow("ntts", "# TTs", 0, null);
        
        
        p.addRow("", "", Row.SEPARATOR, null);
        p.addRow("intmethod", "Int. Method", Row.DROPDOWN, new String[] { "Keplerian", "Runge-Kutta 5", "Runge-Kutta 8", "Bulirsch-Stoer", "Wisdom-Holman", "RMVS" });
        
        c.addGUIListener(p);
        p.controller = c;
        p.console = con;
        p.setPreferredSize(new Dimension(80, p.getComponentCount() * 15));
        p.setMaximumSize(new Dimension(80, p.getComponentCount() * 15));
        return p;
    }
    
    private void addRow(String statstr, String label, int type, String[] options) {
        Row row = new Row(statstr, label, type, options, this);
        row.tag = statstr;
        corr.put(statstr, row);
        add(row);
    }

    public void actionPerformed(ActionEvent e) {
        String tag = e.getActionCommand();
        Row row = corr.get(tag);
        
        if (tag.equals("mstar")) {
            console.getTerminal().exec(String.format("%s:mstar(%s)", controller.getKernel(),
                    row.getValue()));
        } else if (tag.startsWith("intmethod")) {
            String[] sp = tag.split("_");
            console.getTerminal().exec(String.format("%s:int_method('%s')", controller.getKernel(),
                    sp[1]));
        } else if (tag.equals("epoch")) {
            console.getTerminal().exec(String.format("%s:epoch(%s)", controller.getKernel(),
                    row.getValue()));
        } else if (tag.equals("epoch_lock")) {
            controller.exec(String.format("gui.lock('epoch', %s)", corr.get("epoch").getValue()));
        }
        
    }

    public void notification(Notification n) {
        if (n.is(Controller.TYPE_STATS, Controller.TYPE_UPDATE) && n.name.equals(controller.getKernel())) {
            Map<String, Double> map = controller.getStatsForKernel(controller.getKernel());
            
            for (Entry<String, Double> e : map.entrySet()) {
                if (corr.containsKey(e.getKey())) {
                    Row r = corr.get(e.getKey());
                    r.setValue(e.getValue());
                }
            }
        }
    }
    
}
