/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package uicomponents;

import editing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.*;
import java.util.regex.*;
import javax.swing.AbstractAction;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.text.BadLocationException;
import repl.*;
import repl.Controller;
import utils.*;
import utils.Notification;
import utils.NotificationListener;

/**
 *
 * @author stefano
 */
public class Terminal extends javax.swing.JPanel implements NotificationListener {
    private List<String> cmdStack = new ArrayList<String>();
    private int pos = -1;
    private Controller controller;
    
    public static String ECHO_ELEMENTS = "Echo elements change";
    public static String ECHO_DATA = "Echo dataset loading";
    public static String ECHO_KERNEL = "Echo kernel creation";
    
    /**
     * Creates new form Terminal
     */
    private Terminal() {
        
    };
    
    public Terminal(Controller c) {
        controller = c;
        initComponents();
        initUI();
        
    }
    
    private void initUI() {

        setSize(new Dimension(400, 400));

        UIUtils.addTextPopupMenu(input);
        UIUtils.addTextPopupMenu(output);
        
        input.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "exec");
        input.getActionMap().put("exec", new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                exec();
            }
        });
        input.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0), "moveUp");
        input.getActionMap().put("moveUp", new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                move(-1);
            }
        });

        input.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0), "moveDown");
        input.getActionMap().put("moveDown", new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                move(1);
            }
        });

        input.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, KeyEvent.SHIFT_DOWN_MASK), "enter");
        input.getActionMap().put("enter", new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                try {
                    input.getDocument().insertString(input.getSelectionStart(), "\n", null);
                } catch (BadLocationException ex) {
                    
                }
            }
        });

        
        input.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0), "tab");
        input.getActionMap().put("tab", new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                new Thread(new Runnable() {
                    public void run() {
                        complete();
                    }
                }).start();
                
            }
        });

        input.getDocument().putProperty("IGNORE_NORMAL", Boolean.TRUE);
        output.addMouseListener(new MouseListener() {

            public void mouseClicked(MouseEvent e) {
                
            }

            public void mousePressed(MouseEvent e) {
                
            }

            public void mouseReleased(MouseEvent e) {
                try {
                    if (Math.abs(output.getSelectionEnd() - output.getSelectionStart()) > 0)
                        return;
                    int s = output.getSelectionStart();
                    String text = output.getText();
                    int loc = text.substring(0, s-1).lastIndexOf("\u219Dhttp://");
                    if (loc < 0)
                        return;
                    
                    String url = text.substring(loc).trim();
                    Matcher mat = Pattern.compile("(http\\:\\/\\/[\\d\\w\\S/]*)").matcher(url);
                    
                    if (mat.find()) {
                        if (s < mat.start() + loc || s > mat.end() + loc)
                            return;
                        Desktop.getDesktop().browse(new URI(mat.group()));
                    }
                } catch (Exception ex) {
                    Logger.getLogger(Terminal.class.getName()).log(Level.SEVERE, null, ex);
                }
            }

            public void mouseEntered(MouseEvent e) {
                
            }

            public void mouseExited(MouseEvent e) {
                
            }
            
        });
    }

    private static Pattern quotePattern = Pattern.compile("[\"\\']([^\"\\']*?)$");
    
    //private static Pattern kernelPattern = Pattern.compile("(.([^\\(\\)\\{\\}\\s]*?)$");
    
    public SolTextPane setInput(String inp) {
        int pos = inp.indexOf("#");
        input.setText(inp.replaceFirst("#", ""));
        if (pos >= 0) {
            input.setSelectionStart(pos);
            input.setSelectionEnd(pos);
        }
        return input;
    }
    
    public void complete() {
        String text = input.getText().substring(0, input.getSelectionEnd());
        String insert = null;
        
        
        Matcher quoteMatcher = quotePattern.matcher(text);
        if (quoteMatcher.find()) {
            String capt = quoteMatcher.group(1).trim();
            int pos = quoteMatcher.start(1);
            if (capt.startsWith("~")) {
                try {
                    input.getDocument().remove(pos, 1);
                    input.getDocument().insertString(pos, System.getenv("HOME"), null);
                    input.setSelectionStart(input.getSelectionStart() + System.getenv("HOME").length());
                } catch (BadLocationException ex) {
                    Logger.getLogger(Terminal.class.getName()).log(Level.SEVERE, null, ex);
                }
                capt = System.getenv("HOME") + "/" + capt.substring(1);
            }
            
            insert = AutoComplete.complete(capt);
        }
        
        
        
        
        if (insert != null) {
            final String doInsert = insert;
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    try {
                        input.getDocument().insertString(input.getSelectionStart(), doInsert, null);
                    } catch (BadLocationException ex) {
                        Logger.getLogger(Terminal.class.getName()).log(Level.SEVERE, null, ex);
                    }
                }
            });
        }
    }
    
    public void defaultFocus() {
        input.requestFocusInWindow();
    }
    
    public void notification(Notification e) {
        if (e.is(Controller.TYPE_OUTPUT)) {
            if (e.data.toString().contains(">>http://"))
                write(e.data.toString().replaceAll(">>http", "\u219Dhttp") + "\n");
            else
                write(e.data + "\n");
        } else if (e.is(Controller.TYPE_SAVE_SESSION)) {
            try {
                BufferedWriter bw = new BufferedWriter(new FileWriter(e.name, true));
                bw.write("@Output\n");
                bw.write(output.getText());
                bw.write("\n@End\n");
                bw.write("--]]\n");
                bw.close();
            } catch (IOException ex) {
                write("!!!! Could not save to " + e.name);
            }
            
        } else if (e.is(Controller.TYPE_CLEAR_SESSION)) {
            output.setText("");
        }
    }
    public void write(final String str) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                output.setText(output.getText() + str);
            }
        });
        
    }

    public void exec(String cmd, boolean silent) {
        pos = -1;
        
        if (cmd.trim().startsWith("--")) {
            if (!silent)
                write(cmd + "\n");
            return;
        }
        
        if (! silent)
            write("=> " + cmd + "\n");
        controller.exec(cmd);
    }
    
    public void exec(String cmd, String echo, boolean def) {
        if (Prefs.getValue(echo, def))
            exec(cmd, false);
    }
    
    public void exec(String cmd) {
        exec(cmd, false);
    }
    
    private void exec() {
        pos = -1;
        
        String execLine = input.getText();
        cmdStack.add(input.getText());
        if (execLine.trim().startsWith("--")) {
            write(execLine + "\n");
        } else if (execLine.trim().startsWith("!")) {
            String k = execLine.substring(1);
            if (Utils.find(controller.getKernels(), k))
                controller.setKernel(k);
        } else {
            write("=> " + execLine.replaceAll("\n", "\n=> ") + "\n");
            controller.exec(execLine);
        }
        input.selectAll();
    }

    private void move(int by) {
        if (pos == -1)
            pos = cmdStack.size();

        pos = pos + by;

        if (pos < 0) {
            pos = 0;
            return;
        } else if (pos > cmdStack.size() - 1) {
            pos = cmdStack.size() - 1;
            return;
        }
        
        input.setText(cmdStack.get(pos));
    }

    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        output = new editing.SolTextPane();
        jPanel1 = new javax.swing.JPanel();
        input = new editing.SolTextPane();

        setLayout(new java.awt.BorderLayout());

        jScrollPane1.setBorder(null);
        jScrollPane1.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        output.setBorder(javax.swing.BorderFactory.createEmptyBorder(4, 4, 4, 4));
        output.setEditable(false);
        output.setFont(new java.awt.Font("Monospaced", 0, 14)); // NOI18N
        output.setMinimumSize(new java.awt.Dimension(50, 23));
        output.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                outputFocusGained(evt);
            }
        });
        jScrollPane1.setViewportView(output);

        add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jPanel1.setPreferredSize(new java.awt.Dimension(108, 50));
        jPanel1.setLayout(new java.awt.BorderLayout());

        input.setBorder(javax.swing.BorderFactory.createCompoundBorder(javax.swing.BorderFactory.createMatteBorder(1, 0, 0, 0, new java.awt.Color(204, 204, 204)), javax.swing.BorderFactory.createEmptyBorder(6, 6, 6, 6)));
        input.setFont(new java.awt.Font("Monospaced", 0, 14)); // NOI18N
        input.setMinimumSize(new java.awt.Dimension(8, 100));
        input.setPreferredSize(new java.awt.Dimension(108, 100));
        input.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                inputFocusGained(evt);
            }
        });
        jPanel1.add(input, java.awt.BorderLayout.CENTER);

        add(jPanel1, java.awt.BorderLayout.SOUTH);
    }// </editor-fold>//GEN-END:initComponents

    private void inputFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_inputFocusGained
        
    }//GEN-LAST:event_inputFocusGained

    private void outputFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_outputFocusGained
        
    }//GEN-LAST:event_outputFocusGained

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private editing.SolTextPane input;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private editing.SolTextPane output;
    // End of variables declaration//GEN-END:variables
}
