/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package uicomponents;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import plotting.*;
import repl.*;
import utils.*;

/**
 *
 * @author stefano
 */
public class Transits extends JPanel implements NotificationListener {
    private Controller controller;
    private ArrayList<FoldPlot> plotters;
    private int np = 0;
    private double[][] data = null;
    
    private Transits() {
        
    }
    
    private Transits(Controller c) {
        controller = c;
        plotters = new ArrayList<FoldPlot>();
        controller.addGUIListener(this);
        
        for (int i = 0; i < Instance.MAX_GUI_PLANETS; i++) {
            plotters.add(new FoldPlot());
        }
        updatePlots();
    }
    
    public static Transits getTransitsPanel(Controller c) {
        return new Transits(c);
    }
    
    
    
    private void updatePlots() {
        if (np == 0 || data == null) {
            removeAll();
            setLayout(new GridLayout(1, 1));
            JLabel warning = new JLabel();
            warning.setFont(warning.getFont().deriveFont(Font.ITALIC, 20f));
            warning.setBackground(UIUtils.defColor);
            warning.setOpaque(true);
            warning.setForeground(Color.gray);
            
            String text = "";
            if (data == null) {
                warning.setText("<html><center>Load a transit timing dataset to see transit timing residuals.");
            } else if (np == 0) {
                warning.setText("<html><center>Add a planet to see transit timing residuals.");
            }
            add(warning);
        } else if (np != getComponentCount() || getComponent(0) instanceof JLabel) {
            removeAll();
            setLayout(new GridLayout(np, 1));
            for (int i = 0; i < np; i++) {
                add(plotters.get(i));
            }
        }
        
        
    }

    public void notification(Notification n) {
        
        if (n.is(Controller.TYPE_DATA, Controller.TYPE_UPDATE) && n.name.equals(controller.getKernel())) {
            
            if (controller.getNPlanetsForKernel(controller.getKernel()) < 1 || controller.getDataForKernel(controller.getKernel(), Defs.T_TIMING) == null) {
                np = 0;
                data = controller.getDataForKernel(controller.getKernel(), Defs.T_TIMING);
            } else {

                np = controller.getNPlanetsForKernel(controller.getKernel());
                data = controller.getDataForKernel(controller.getKernel(), Defs.T_TIMING);

                for (int i = 1; i < np + 1; i++) {
                    ArrayList<double[]> d = new ArrayList<double[]>();

                    for (int j = 0; j < data.length; j++) {
                        if (((int) data[j][Defs.T_TDS_PLANET]) == i) {
                            d.add(data[j]);
                        }
                    }

                    System.out.println("-> " + d.size());
                    FoldPlot plotter = plotters.get(i-1);
                    plotter.beginUpdate();
                    plotter.clear();
                    plotter.setValueColumn(Defs.T_GUI);
                    plotter.addData(d.toArray(new double[1][]), "TTV_" + i);
                    plotter.setNoDataWarning("No timing data for planet " + i);
                    plotter.setXLabel("Transit times");
                    plotter.setYLabel("O-C (Planet " + i + ")");
                    plotter.endUpdate();
                }
            }
            
            updatePlots();
        }
        
        
    }
    
}
