/*
 * Routines for simple I/O.
 */

package utils;

import java.io.*;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.net.*;

/**
 *
 * @author Ste
 */
public class FileUtils {

    public FileUtils() {
    }
    public static String authentication;

    public static ArrayList<String> readFile(String fileName) {
        java.io.BufferedReader rd = null;
        
        ArrayList<String> ret = null;
        try {
            rd = new java.io.BufferedReader(new java.io.FileReader(fileName));

            String line = rd.readLine();
            ret = new ArrayList<String>();
            while (line != null) {
                ret.add(line);
                line = rd.readLine();
            }

        } catch (IOException ex) {
            ex.printStackTrace();
            return null;
        }

        try {
                rd.close();
        } catch (IOException ex) {
                Logger.getLogger("global").log(Level.SEVERE, null, ex);
        }
        return ret;
    }

    public static StringBuffer readFileToBuffer(String fileName) {
        java.io.BufferedReader rd = null;
        StringBuffer buf = new StringBuffer();
        
        try {
            rd = new java.io.BufferedReader(new java.io.FileReader(fileName));

            String line = rd.readLine();

            while (line != null) {
                buf.append(line + "\n");
                line = rd.readLine();
            }

        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            try {
                rd.close();
            } catch (IOException ex) {
                Logger.getLogger("global").log(Level.SEVERE, null, ex);
            }
        }
        return buf;

    }

    public static String stripHTML(String html) {
        return html.toString().replaceAll("\\<.*?>","");
    }
    
    public static int url_get_binary(String urlStr, OutputStream os) {
        URLConnection conn = null;
        InputStream in = null;
        int retBytes = 0;
        try {
            System.out.println("Reading " + urlStr);
            URL url = new URL(urlStr);

            conn = url.openConnection();
			System.out.println("Connected");
            conn.setRequestProperty("Authorization", "Basic " + authentication);
            in = conn.getInputStream();
            int readBytes;

            byte[] buffer = new byte[1024];

            while ((readBytes = in.read(buffer)) != -1) {
                os.write(buffer, 0, readBytes);
                os.flush();
                retBytes += readBytes;
            }
            System.out.println(retBytes + " read.");
        } catch (Exception e) {
            e.printStackTrace();
            return -1;
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(RetrieveSys.class.getName()).log(Level.SEVERE, null, ex);
            }
            return retBytes;
        }

    }

    public static String url_download(String urlStr) {
	URLConnection conn = null;
        BufferedReader in = null;
        StringBuffer text = new StringBuffer();
        try {
            System.out.println("Reading " + urlStr);
            URL url = new URL(urlStr);
            conn = url.openConnection();

            conn.setRequestProperty("Authorization", "Basic " + authentication);
            in = new BufferedReader(new InputStreamReader(conn.getInputStream()));


            String buffer = "";

            while ((buffer = in.readLine()) != null) {
                text.append(buffer + "\n");
            }

        } catch (Exception e) {
            e.printStackTrace();
            return null;
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(RetrieveSys.class.getName()).log(Level.SEVERE, null, ex);
            }
            return text.toString();
        }

    }

    public static String url_get_contents(String urlStr) {
        URLConnection conn = null;
        BufferedReader in = null;
        StringBuffer text = new StringBuffer();
        try {
            System.out.println("Reading " + urlStr);
            URL url = new URL(urlStr);
            conn = url.openConnection();

            conn.setRequestProperty("Authorization", "Basic " + authentication);
            in = new BufferedReader(new InputStreamReader(conn.getInputStream()));

            String buffer = "";

            while ((buffer = in.readLine()) != null) {
                text.append(buffer + "\n");
            }

        } catch (Exception e) {
            e.printStackTrace();
            return null;
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(RetrieveSys.class.getName()).log(Level.SEVERE, null, ex);
            }
            return text.toString();
        }

    }

    public static void url_get_contents(String urlStr, ArrayList<String> buf) {
        URLConnection conn = null;
        BufferedReader in = null;
        try {
            System.out.println("Reading " + urlStr);
            URL url = new URL(urlStr);
            conn = url.openConnection();

            conn.setRequestProperty("Authorization", "Basic " + authentication);
            in = new BufferedReader(new InputStreamReader(conn.getInputStream()));


            String buffer = "";

            while ((buffer = in.readLine()) != null) {
                buf.add(buffer + "\n");
            }

        } catch (Exception e) {
            e.printStackTrace();
            return;
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (IOException ex) {
                Logger.getLogger(RetrieveSys.class.getName()).log(Level.SEVERE, null, ex);
            }
            
            return;
        }

    }

    public static int put_contents(String fileName, ArrayList<String> content) {
        PrintWriter pw = null;
        try {
            pw = new PrintWriter(new FileWriter(fileName));

            for (int i = 0; i < content.size(); i++)
                pw.write(content.get(i) + "\n");
        } catch (Exception e) {
            return -1;
        } finally {
            pw.close();
            return 0;
        }
    }



    public static boolean writeTable(String fileName, double[][] table, String format) {
        try {
            PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));

            for (int i = 0; i < table.length; i++) {
                for (int j = 0; j < table[i].length; j++) {
                    pw.format(format, table[i][j]);
                }
                pw.println();
            }
            pw.close();
            return true;
        } catch (IOException ex) {
            return false;
        }

    }

	


    public static boolean writeArray(String fileName, double[] table, String format) {
        try {
            PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(fileName)));

            for (int i = 0; i < table.length; i++) {
                pw.format(format, table[i]);
                pw.println();
            }
            pw.close();
            return true;
        } catch (IOException ex) {
            return false;
        }

    }

    public static boolean writeArray(StringBuffer pw, double[] table, String format) {
        for (int i = 0; i < table.length; i++) {
            pw.append(String.format(format + "\n", table[i]));
        }

        return true;
    }


    public static double[] readArray(String fileName) {
        BufferedReader rd = null;
        try {
            ArrayList<Double> ret = new ArrayList<Double>();
            rd = new java.io.BufferedReader(new java.io.FileReader(fileName));
            String line = rd.readLine();
            while (line != null) {
                if (! line.trim().startsWith("#")) {
                    ret.add(Double.valueOf(line.trim()));
                }

                line = rd.readLine();
            }

            return Utils.toArray(ret);
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        } finally {
            try {
                rd.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }

    public static double[][] readTable(String fileName) {
        BufferedReader rd = null;
        try {
            ArrayList<double[]> ret = new ArrayList<double[]>();
            rd = new java.io.BufferedReader(new java.io.FileReader(fileName));
            String line = rd.readLine();
            while (line != null) {
                if (! line.trim().startsWith("#")) {
                    ret.add(Utils.numbers(line));
                }

                line = rd.readLine();
            }

            return Utils.toMatrix(ret);
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        } finally {
            try {
                rd.close();
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
    }


    public static double[][] readTable(BufferedReader rd, int lines) {
        try {
            ArrayList<double[]> ret = new ArrayList<double[]>();
            String line = rd.readLine();
            for (int i = 0; i < lines; i++) {
                if (! line.trim().startsWith("#")) {
                    ret.add(Utils.numbers(line));
                }

                line = rd.readLine();
            }

            return Utils.toMatrix(ret);
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        } 
    }



    public static int put_contents(String fileName, String content) {
        PrintWriter pw = null;
        try {
            pw = new PrintWriter(new FileWriter(fileName));
            pw.write(content);
        } catch (Exception e) {
            return -1;
        } finally {
            pw.close();
            return 0;
        }
    }

    public static String get_contents(String fileName) {
        java.io.BufferedReader rd = null;

        StringBuffer ret = new StringBuffer();
        try {
            rd = new java.io.BufferedReader(new java.io.FileReader(fileName));

            String line = rd.readLine().concat("\n");


            while (line != null) {
                ret.append(line);
                line = rd.readLine();
                if (line != null) {
                    line = line.concat("\n");
                }
            }

        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        } finally {
            try {
                if (rd != null) {
                    rd.close();
                }
            } catch (Exception ex) {
                Logger.getLogger("global").log(Level.SEVERE, null, ex);
            }
            return ret.toString();
        }

    }

    public static int findContainingString(ArrayList<String> list, String needle) {

        for (int i = 0; i < list.size(); i++) {

            if (list.get(i).toUpperCase().contains(needle.toUpperCase())) {
                return i;
            }
        }
        return -1;
    }

    

    public static FilenameFilter createFileNameFilter(final String filter, final boolean noDotFiles) {
	return new FilenameFilter() {
	    public boolean accept(File dir, String name) {
		if (noDotFiles)
		    if (name.matches("^\\..*"))
			return false;
		return name.matches(filter);
	    }
	};
    }
}
