/*
 * Routines to simplify some UI tasks. 
 *
 */
package utils;

import com.sun.awt.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.reflect.Method;
import java.util.*;
import java.util.concurrent.*;
import java.util.logging.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 *
 * @author Ste
 */
public class UIUtils {

    private static Color[] colors;
    public static Color alternateColor = new Color(234, 241, 252);
    ;
    public static Color alternateColorDarker = new Color(204, 211, 222);
    ;

    public static Color sidebarColor = new Color(223, 227, 233);
    public static Color sidebarHeaderColor = new Color(100, 100, 100);
    public static Color defColor = new Color(238, 238, 239);
    private static final String errMsg = "Error attempting to launch web browser";
    private static FutureTask<FileDialog> fd;
        
    static {
        fd = new FutureTask<FileDialog>(new Callable<FileDialog>() {
            public FileDialog call() throws Exception {
                FileDialog fd = new FileDialog((Frame) null);
                fd.setDirectory(".");
                return fd;
            }
        });
        Utils.submitTask(fd);
    }
    
    public static String openFile(String title) {
        try {
            FileDialog ff = fd.get();
            ff.setMode(FileDialog.LOAD);
            ff.setTitle(title);
            ff.setVisible(true);
            if (ff.getFile() == null)
                return null;
            else
                return ff.getDirectory() + File.separator + ff.getFile();
        } catch (Exception ex) {
            return null;
        }
    }
    
    public static String saveFile(String title) {
        try {
            FileDialog ff = fd.get();
            ff.setMode(FileDialog.SAVE);
            ff.setTitle(title);
            ff.setVisible(true);
            if (ff.getFile() == null)
                return null;
            else
                return ff.getDirectory() + File.separator + ff.getFile();
        } catch (Exception ex) {
            return null;
        }
    }
    public static void openURL(String url) {
        String osName = System.getProperty("os.name");
        try {
            if (osName.startsWith("Mac OS")) {
                Class fileMgr = Class.forName("com.apple.eio.FileManager");
                Method openURL = fileMgr.getDeclaredMethod("openURL", new Class[]{String.class});
                openURL.invoke(null, new Object[]{url});
            } else if (osName.startsWith("Windows")) {
                Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + url);
            } else { //assume Unix or Linux
                String[] browsers = {"firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape"};
                String browser = null;
                for (int count = 0; count < browsers.length && browser == null; count++) {
                    if (Runtime.getRuntime().exec(new String[]{"which", browsers[count]}).waitFor() == 0) {
                        browser = browsers[count];
                    }
                }
                if (browser == null) {
                    throw new Exception("Could not find web browser");
                } else {
                    Runtime.getRuntime().exec(new String[]{browser, url});
                }
            }
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, errMsg + ":\n" + e.getLocalizedMessage());
        }
    }

    public static Color getColor(int c) {
        if (colors == null) {
            colors = new Color[1];
            ArrayList<Color> cols = new ArrayList<Color>();
            // blue
            cols.add(new Color(20, 79, 250));
            // red
            cols.add(new Color(220, 0, 0));
            // green
            cols.add(new Color(80, 215, 80));
            // magenta
            cols.add(new Color(255, 0, 255));
            // mocha
            cols.add(new Color(128, 64, 0));
            // purple
            cols.add(new Color(64, 64, 128));
            // asparagus
            cols.add(new Color(128, 128, 0));
            // orange
            cols.add(new Color(255, 128, 0));
            // black
            cols.add(new Color(0, 0, 0));
            // medium gray
            cols.add(new Color(128, 128, 128));

            colors = cols.toArray(colors);
        }

        if (c >= colors.length) {
            return new Color((float) Math.random(), (float) Math.random(), (float) Math.random());
        }

        return colors[c];
    }

    public static final void runInEDT(Runnable r, boolean wait) {
        if (SwingUtilities.isEventDispatchThread()) {
            r.run();
        } else if (wait) {
            try {
                SwingUtilities.invokeAndWait(r);
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            SwingUtilities.invokeLater(r);
        }
    }

    static public String colorToHTML(Color color) {
        String rgb = Integer.toHexString(color.getRGB());
        rgb = rgb.substring(2, rgb.length());
        return rgb;
    }

    public static void addPopupMenu(final JComponent b, final JPopupMenu menu) {
        addPopupMenu(b, menu, true, false);
    }

    public static void addPopupMenu(final JComponent b, final JPopupMenu menu, final boolean under, final boolean popupClick) {
        b.addMouseListener(new MouseListener() {

            public void mouseClicked(MouseEvent e) {
                
            }

            public void mousePressed(MouseEvent e) {
                if (popupClick && !e.isPopupTrigger()) {
                    return;
                }
                menu.show(e.getComponent(), (under ? 0 : e.getX()), (under ? b.getHeight() : e.getY()));
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
            }

            public void mouseExited(MouseEvent e) {
            }
        });
    }

    public static String getKeystrokeBase() {
        if (System.getProperty("os.name").toLowerCase().equals("mac os x")) {
            return "meta";
        } else {
            return "ctrl";
        }
    }

    public static String getAltKeystrokeBase() {
        if (System.getProperty("os.name").toLowerCase().equals("mac os x")) {
            return "alt";
        } else {
            return "alt";
        }
    }

    public static String colorToHtml(Color c) {
        if (c == null) {
            c = Color.BLACK;
        }
        return "#" + Integer.toHexString((c.getRGB() & 0xffffff) | 0x1000000).substring(1);
    }

    public static JMenuItem createMenuItem(String text, String accelerator, final String action, final Object responder) {
        JMenuItem mi = new JMenuItem(text);
        mi.setAccelerator(KeyStroke.getKeyStroke(getKeystrokeBase() + " " + accelerator));
        mi.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                String field = "do" + action;
                try {
                    responder.getClass().getMethod(field).invoke(responder);
                } catch (Exception ex) {
                    System.err.println(action + " not implemented for " + responder.getClass().getName());
                }
            }
        });
        return mi;
    }

    public static void addTextPopupMenu(final JTextComponent c) {
        final JPopupMenu pm = new JPopupMenu();
        JMenuItem mi = new JMenuItem("Cut");
        mi.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                c.cut();
            }
        });
        if (!c.isEditable()) {
            mi.setEnabled(false);
        }
        pm.add(mi);
        mi = new JMenuItem("Copy");
        mi.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                c.copy();
            }
        });
        pm.add(mi);
        mi = new JMenuItem("Paste");
        mi.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                c.paste();
            }
        });
        if (!c.isEditable()) {
            mi.setEnabled(false);
        }
        pm.add(mi);
        pm.addSeparator();
        mi = new JMenuItem("Select All");
        mi.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                c.selectAll();
            }
        });
        pm.add(mi);
        c.addMouseListener(new MouseListener() {

            public void mouseClicked(MouseEvent e) {
            }

            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    pm.show(c, e.getX(), e.getY());
                }
            }

            public void mouseReleased(MouseEvent e) {
            }

            public void mouseEntered(MouseEvent e) {
            }

            public void mouseExited(MouseEvent e) {
            }
        });

    }

    public static void fadeAndHide(final Window w) {
        final float[] tr = new float[]{1.f};

        final javax.swing.Timer timer = new javax.swing.Timer(100, null);
        timer.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                tr[0] -= 0.05;
                setWindowOpacity(w, tr[0]);
                if (tr[0] <= 0.) {
                    w.setVisible(false);
                    setWindowOpacity(w, 1.f);
                    timer.stop();
                };
                
            }
        });
        timer.start();
    }

    public static void setWindowOpacity(Window w, float op) {
        try {
            AWTUtilities.setWindowOpacity(w, op);
        } catch (Exception e) {};
    }
    
    public static void fadeAndShow(final Window w) {
        
        final float[] tr = new float[]{0.f};
        setWindowOpacity(w, 0.f);
        w.setVisible(true);
        setWindowOpacity(w, 0.f);
        final javax.swing.Timer timer = new javax.swing.Timer(50, null);
        timer.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                tr[0] += 0.3;

                if (tr[0] >= 1.) {
                    setWindowOpacity(w, 1.f);
                    timer.stop();
                } else {
                    setWindowOpacity(w, tr[0]);
                }
            }
        });
        timer.start();
    }

    public static JMenuItem createMenuItem(final Object receiver, String caption, final String command, final Object parameter, String accelerator) {
        JMenuItem mi = new JMenuItem(caption);
        mi.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    if (parameter != null) {
                        receiver.getClass().getMethod(command, Object.class).invoke(receiver, parameter);
                    } else {
                        receiver.getClass().getMethod(command).invoke(receiver);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        });
        mi.setActionCommand(command);
        mi.setAccelerator(KeyStroke.getKeyStroke(accelerator));
        return mi;
    }

    public static JButton createButton(final Object receiver, String caption, Icon icon, final String command, final Object parameter) {
        JButton btn = new JButton(caption);
        btn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    if (parameter != null) {
                        receiver.getClass().getMethod(command, Object.class).invoke(receiver, parameter);
                    } else {
                        receiver.getClass().getMethod(command).invoke(receiver);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        });
        btn.setActionCommand(command);
        btn.setIcon(icon);
        return btn;
    }
    
    
    public static JButton createEmptyBorderButton(final Object receiver, String caption, Icon icon, final String command, final Object parameter) {
        JButton btn = new JButton(caption);
        btn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    if (parameter != null) {
                        receiver.getClass().getMethod(command, Object.class).invoke(receiver, parameter);
                    } else {
                        receiver.getClass().getMethod(command).invoke(receiver);
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        });
        btn.setActionCommand(command);
        btn.setIcon(icon);
        btn.setBorder(null);
        return btn;
    }
    
    public static JPopupMenu createPopupMenu(JMenuItem... mi) {
        JPopupMenu pm = new JPopupMenu();
        for (JMenuItem m : mi) {
            pm.add(m);
        }
        return pm;
    }
}
