/*
 * Arrays[]/Arrays[][] operations.
 *
 */
package utils;

import java.io.*;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.table.*;
import numal.NumUtils;
import static java.lang.Math.*;
import java.util.concurrent.*;
import org.apache.commons.io.*;
import repl.*;

/**
 *
 * @author stefano
 */
public class Utils {
    private static Executor executor = Executors.newCachedThreadPool();
    
    public static void submitTask(Runnable r) {
        executor.execute(r);
    }
	/**
	 *
	 */
	static public boolean DEBUG = false;

	/**
	 * Searches the array for the specified item and returns its index within the array, or -1.
	 * @param array Array to search
	 * @param item Item to search
	 * @return Index of the item, or -1
	 */
	static public <T> int indexOf(T[] array, T item) {
        for (int i = 0; i < array.length; i++)
            if (array[i].equals(item))
                return i;
        return -1;
        }

      

	/**
	 *
	 * @param str
	 */
	static public void dprintln(String str) {
        if (DEBUG) {
            System.out.println(str);
        }
    }

	/**
	 *
	 * @param str
	 */
	static public void dprintln(double str) {
        if (DEBUG) {
            System.out.println(str);
        }
    }

    static public boolean find(Object[] array, Object obj) {
        for (int i = 0; i < array.length; i++)
            if (array[i].equals(obj))
                return true;
        return false;
    }
    
	/**
	 *
	 * @param str
	 */
	static public void dprintln(int str) {
        if (DEBUG) {
            System.out.println(str);
        }
    }

	/**
	 *
	 */
	static public void dprintln() {
        if (DEBUG) {
            System.out.println();
        }
    }

	/**
	 *
	 * @param str
	 */
	static public void dprint(String str) {
        if (DEBUG) {
            System.out.print(str);
        }
    }

	/**
	 *
	 * @param str
	 */
	static public void dprint(double str) {
        if (DEBUG) {
            System.out.print(str);
        }
    }

	/**
	 *
	 * @param str
	 */
	static public void dprint(int str) {
        if (DEBUG) {
            System.out.print(str);
        }
    }

	/**
	 * Resizes an array to a new size.
	 * @param source Array
	 * @param size New length of the array
	 * @return Resized array
	 */
	static public double[] resize(double[] source, int size) {
        double[] dest = new double[size];
        for (int i = 0; i < Math.min(size, source.length); i++)
            dest[i] = source[i];
        return source;
    }

	/**
	 * Copies a source array into the destination array.
	 * @param source
	 * @param dest
	 */
	static final public void copy(final int[] source, int[] dest) {
        if ((source == null) || (source.length == 0)) {
            return;
        }
        System.arraycopy(source, 0, dest, 0, source.length);
    }

	/**
	 * Returns a copy of the specified array.
	 * @param source
	 * @return
	 */
	static final public int[] copy(final int[] source) {
        if ((source == null) || (source.length == 0)) {
            return null;
        }
        int[] dest = new int[source.length];
        copy(source, dest);
        return dest;
    }


	/**
	 * Returns a copy of the specified array.
	 * @param vo
	 * @return
	 */
	static final public boolean[] copy(boolean[] vo) {
        boolean[] ret = new boolean[vo.length];
        System.arraycopy(vo, 0, ret, 0, vo.length);
        return ret;
    }

	/**
	 * Copies a source array into the destination array.
	 * @param source
	 * @param dest
	 */
	static final public void copy(final double[] source, double[] dest) {
        System.arraycopy(source, 0, dest, 0, source.length);
    }



	/**
	 * Copies a source array into the destination array.
	 * @param source
	 * @param dest
	 */
	static final public void copy(String[] source, String[] dest) {
        System.arraycopy(source, 0, dest, 0, source.length);
    }

	/**
	 * Copies a source array into the destination array.
	 * @param source
	 * @param dest
	 */
	static final public void copy(final double[][] source, double[][] dest) {
        for (int i = 0; i < source.length; i++) {
            for (int j = 0; j < source[i].length; j++)
                dest[i][j] = source[i][j];
        }
    }

	/**
	 * Returns the index of the smallest item within the array.
	 * @param src
	 * @return
	 */
	static public int amini(double[] src) {
        double ret = src[0];
        int idx = 0;
        for (int i = 1; i < src.length; i++) {
            if (src[i] < ret) {
                ret = src[i];
                idx = i;
            }
        }

        return idx;
    }

        /**
         * Returns the index of the largest item within the array.
         * @param src
         * @return
         */
        static public int amaxi(double[] src) {
        double ret = src[0];
        int idx = 0;
        for (int i = 1; i < src.length; i++) {
            if (src[i] > ret) {
                ret = src[i];
                idx = i;
            }
        }

        return idx;
    }

        /**
         * Returns the index of the smallest item within the table for column col.
         * @param src
         * @param col
         * @return
         */
        static public int amini(double[][] src, int col) {
        double ret = src[0][col];
        int idx = 0;
        for (int i = 1; i < src.length; i++) {
            if (src[i][col] < ret) {
                ret = src[i][col];
                idx = i;
            }
        }

        return idx;
    }

        /**
         * Returns the index of the largest item within the table for column col.
         * @param src
         * @param col
         * @return
         */
        static public int amaxi(double[][] src, int col) {
        double ret = src[0][col];
        int idx = 0;
        for (int i = 1; i < src.length; i++) {
            if (src[i][col] > ret) {
                ret = src[i][col];
                idx = i;
            }
        }

        return idx;
    }
	/**
	 * Returns an array of length len with elements equally spaced between a and b.
	 * @param a
	 * @param b
	 * @param len
	 * @return
	 */
	static public double[] linspace(double a, double b, int len) {
		double[] ret = new double[len];
		ret[len - 1] = b;
		ret[0] = a;

		double dx = (b - a) / (len - 1);

		for (int i = 1; i < len - 1; i++)
			ret[i] = ret[i-1] + dx;

		return ret;
	}

        
        /**
         *
         * @param list
         * @param tagsAndClasses
         * @return
         */
        static public boolean checkOptions(Object[] list, Object... tagsAndClasses) {
            // check for unrecognized options
            if (list == null)
                return true;
            
            String[] tags = new String[(int) (tagsAndClasses.length / 2.)];
            Class[] classes = new Class[(int) (tagsAndClasses.length / 2.)];

            int idx = 0;
            for (int i = 0; i < tagsAndClasses.length; i+=2) {
                tags[idx] = (String) tagsAndClasses[i];
                classes[idx] = (Class) tagsAndClasses[i + 1];
                idx++;
            }

            for (int i = 0; i < list.length; i+=2) {
                if (!(list[i] instanceof String)) {
                    System.err.println("Error: taglist format error at index " + i);
                    return false;
                }
                String tag = (String) list[i];
                if (indexOf(tags, tag) == -1) {
                    System.err.println("Error: unrecognized option " + tag);
                    return false;
                }
            }

            /*
            for (int i = 1; i < list.length; i+=2) {
                String tag = (String) list[i - 1];
                Class expectedClass = classes[indexOf(tags, tag)];

                if (! expectedClass.equals(list[i].getClass())) {
                    System.err.println("Error: unexpected type for " + tag +
                            "(found " + list[i].getClass().getName() +
                            ", expected " + expectedClass.getName() + ")");
                    return false;
                }
            }
            */
            return true;

        }

	/**
	 *
	 * @param list
	 * @param tag
	 * @param defaultValue
	 * @return
	 */
	static public Object parseOptions(Object[] list, String tag, Object defaultValue) {
		if (list == null)
			return defaultValue;

		for (int i = 0; i < list.length; i += 2)
			if (((String) list[i]).equals(tag))
				return list[i+1];
		return defaultValue;
	}

	/**
	 *
	 * @param list
	 * @param tag
	 * @param defaultValue
	 * @return
	 */
	static public double parseOptions(Object[] list, String tag, double defaultValue) {
		if (list == null)
			return defaultValue;

		for (int i = 0; i < list.length; i += 2)
			if (((String) list[i]).equals(tag))
				return (Double) list[i+1];
		return defaultValue;
	}



	/**
	 *
	 * @param list
	 * @param tag
	 * @param defaultValue
	 * @return
	 */
	static public int parseOptions(Object[] list, String tag, int defaultValue) {
		if (list == null)
			return defaultValue;

		for (int i = 0; i < list.length; i += 2)
			if (((String) list[i]).equals(tag))
				return (Integer) list[i+1];
		return defaultValue;
	}


	/**
	 *
	 * @param list
	 * @param tag
	 * @param defaultValue
	 * @return
	 */
	static public Boolean parseOptions(Object[] list, String tag, boolean defaultValue) {
		if (list == null)
			return defaultValue;

		for (int i = 0; i < list.length; i += 2)
			if (((String) list[i]).equals(tag))
				return (Boolean) list[i+1];
		return defaultValue;
	}


	/**
	 * Returns an array containing <i>count</i> elements from <i>from</i> (inclusive).
	 * @param array Array to slice
	 * @param from First index
	 * @param count Slice size
	 * @return Slice
	 */
	static public double[] slice(double[] array, int from, int count) {
		double[] ret = new double[count];
		for (int i = from; i < from + count; i++)
			ret[i-from] = array[i];
		return ret;
	}

        /**
         * Returns a slice of the specified table (all rows between from and from + count)
         * @param array
         * @param from
         * @param count
         * @return
         */
        static public double[][] slice(double[][] array, int from, int count) {
		double[][] ret = new double[count][];
		for (int i = from; i < from + count; i++)
			ret[i-from] = array[i];
		return ret;
	}



        /**
         * Returns the span of the array (max(array) - min(array))
         * @param array
         * @return
         */
        static public double span(double[] array) {
            return amax(array) - amin(array);
        }

        /**
         * Returns the span of the table (max(array) - min(array)) for the given column
         * @param table
         * @param column
         * @return
         */
        static public double span(double[][] table, int column) {
            return amax(table, column) - amin(table, column);
        }

	/**
	 * Returns the smallest item of the array
	 * @param src
	 * @return
	 */
	static public double amin(double[] src) {
        double ret = src[0];

        for (int i = 1; i < src.length; i++)
			if (! Double.isNaN(src[i]))
				ret = Math.min(ret, src[i]);

        return ret;
    }

        /**
         * Returns the smallest item of the table for the specified column
         * @param src
         * @param column
         * @return
         */
        static public double amin(double[][] src, int column) {
        double ret = src[0][column];

        for (int i = 1; i < src.length; i++)
			if (! Double.isNaN(src[i][column]))
				ret = Math.min(ret, src[i][column]);

        return ret;
    }


        /**
         * Given an array a, returns an array containing the differences between
         * successive array elements (b[0] = a[1]-a[0], b[i] = a[i+1]-a[i]).
         * @param src The input array
         * @return Array of differences
         */
        static public double[] adiff(double[] src) {
            double[] ret = new double[src.length - 1];

            for (int i = 1; i < src.length; i++)
                ret[i-1] = src[i] - src[i-1];

            return ret;
        }

	/**
	 * Returns the largest element in the array
	 * @param src
	 * @return
	 */
	static public double amax(double[] src) {
        double ret = src[0];

        for (int i = 1; i < src.length; i++)
			if (! Double.isNaN(src[i]))
				ret = Math.max(ret, src[i]);

        return ret;
    }

        /**
         * Returns the largest element in table for the specified column
         * @param src
         * @param column
         * @return
         */
        static public double amax(double[][] src, int column) {
        double ret = src[0][column];

        for (int i = 1; i < src.length; i++)
			if (! Double.isNaN(src[i][column]))
				ret = Math.max(ret, src[i][column]);

        return ret;
    }


	/**
	 * Returns an array of absolute values
	 * @param src
	 * @return
	 */
	static public double[] aabs(double[] src) {
        double[] ret = copy(src);

        for (int i = 0; i < src.length; i++)
            ret[i] = Math.abs(src[i]);
        return ret;
    }


        /**
         * Computes the log10 of the given array
         * @param src
         * @return
         */
        static public double[] alog10(double[] src) {
            double[] dest = new double[src.length];
            for (int i = 0; i < src.length; i++)
                dest[i] = Math.log10(src[i]);
            return dest;
        }




	/**
	 *
	 * @param source
	 * @return
	 */
	static public DefaultTableModel copy(DefaultTableModel source) {

        DefaultTableModel dest = new DefaultTableModel(source.getRowCount(), source.getColumnCount());
        String[] columns = new String[source.getColumnCount()];

        for (int i = 0; i < source.getRowCount(); i++) {
            for (int j = 0; j < source.getColumnCount(); j++) {
                dest.setValueAt(source.getValueAt(i, j), i, j);
                columns[j] = source.getColumnName(j);
            }
        }
        dest.setColumnIdentifiers(columns);
        return dest;
    }

	/**
	 *
	 * @param list
	 * @return
	 */
/*	static public GenericList copyGLAsDoubleArray(GenericList list) {
        GenericList dest = new GenericList();
        list.moveToHead();

        while (list.hasNext()) {
            dest.insert((Object) copy((double[][]) list.next()));
        }

        return dest;
    }

	/**
	 *
	 * @param list
	 * @return
	 */
    /*
	static public GenericList copyGLAsString(GenericList list) {
        GenericList dest = new GenericList();
        list.moveToHead();

        while (list.hasNext()) {
            dest.insert((Object) ((String) list.next()));
        }

        return dest;
    }
*/
	/**
	 * Converts an int table to a boolean table.
	 * @param source
	 * @return
	 */
	static public boolean[][] intToBooleanArray(int[][] source) {
        if (source == null) {
            return null;
        }

        boolean[][] dest = new boolean[source.length][source[0].length];
        for (int i = 0; i < source.length; i++) {
            for (int j = 0; j < source[i].length; j++) {
                dest[i][j] = (source[i][j] == 1);
            }
        }
        return dest;
    }

	/**
	 * Converts a string to a double number. If the string cannot be converted
         * to a double, then the method returns the default value def.
	 * @param v
	 * @param def
	 * @return
	 */
	static public Double stringToDouble(String v, Double def) {
	Double ret = def;
	try {
	    ret = Double.parseDouble(v);
	} catch (Exception e) {};
	return ret;
    }

	/**
	 * Converts a boolean table to an int table (true -> 1, false -> 0)
	 * @param source
	 * @return
	 */
	static public int[][] booleanToIntArray(boolean[][] source) {
        if (source == null) {
            return null;
        }

        int[][] dest = new int[source.length][source[0].length];
        for (int i = 0; i < source.length; i++) {
            for (int j = 0; j < source[i].length; j++) {
                if (source[i][j])
                    dest[i][j] = 1;
                else
                    dest[i][j] = 0;
            }
        }
        return dest;
    }

	/**
	 * Converts an int array to a boolean array.
	 * @param source
	 * @return
	 */
	static public boolean[] intToBooleanArray(int[] source) {
        if (source == null) {
            return null;
        }

        boolean[] dest = new boolean[source.length];
        for (int i = 0; i < source.length; i++) {
                dest[i] = (source[i] == 1);
            }

        return dest;
    }

	/**
	 * Converts a boolean array to an int array.
	 * @param source
	 * @return
	 */
	static public int[] booleanToIntArray(boolean[] source) {
        if (source == null) {
            return null;
        }

        int[] dest = new int[source.length];
        for (int i = 0; i < source.length; i++) {
                if (source[i]) {
                    dest[i] = 1;
                }

        }
        return dest;
    }

	/**
	 * Saves a table to a file, using the default format (%15.8f).
	 * @param src Numeric table
	 * @param fn File name
	 * @throws FileNotFoundException
	 */
	static public void saveData(double[][] src, String fn) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(fn);
        for (int i = 0; i < src.length; i++) {
            for (int j = 0; j < src[i].length; j++)
                pw.format("%15.8f ", src[i][j]);
            pw.println();
        }
        pw.close();
    }

    	/**
	 * Saves a list of double arrays to a file, using the default format (%15.8f).
	 * @param src
	 * @param fn
	 * @throws FileNotFoundException
	 */
	static public void saveData(List<double[]> src, String fn) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(fn);
        for (int i = 0; i < src.size(); i++) {
            for (int j = 0; j < src.get(i).length; j++)
                pw.format("%15.8f ", src.get(i)[j]);
            pw.println();
        }
        pw.close();
    }



	/**
	 * Saves an array to a file,  using the default format (%15.8f).
	 * @param src
	 * @param fn
	 * @throws FileNotFoundException
	 */
	static public void saveData(double[] src, String fn) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(fn);
        for (int i = 0; i < src.length; i++) {
            pw.format("%15.8f\n", src[i]);
        }
        pw.close();
    }

    static public void saveData(Object data, String fn) throws FileNotFoundException {
        PrintWriter pw = new PrintWriter(fn);
        pw.print(data);
        pw.close();
    }
    
    static private boolean containedInEllipse(double x, double y, double[] ell) {

        double x0 = ell[0];
        double y0 = ell[1];
        double a = ell[2];
        double b = ell[3];

        x = x * cos(-ell[4]) - y * sin(-ell[4]);
        y = x * sin(-ell[4]) + y * cos(-ell[4]);


        double v = Math.pow(x - x0, 2) / (a*a) +
                Math.pow(y - y0, 2) / (b*b) - 1;

        return v < 0;
    }

	/**
	 * Creates a new array of the specified length and fills it with the specified
         * double value.
	 * @param v Default value for array elements
	 * @param length Length of the array
	 * @return
	 */
	static public final double[] array(double v, int length) {
        double[] a = new double[length];
        Arrays.fill(a, v);
        return a;
    }

    
    
	/**
	 *
	 * @param data1
	 * @param data2
	 * @param perc
	 * @param isAngle1
	 * @param isAngle2
	 * @param averageType
	 * @return
	 */
    /*
	static public final double[] findContainingEllipse(final double[] data1, final double[] data2,
            final double perc, boolean isAngle1, boolean isAngle2, int averageType) {
        try {
            double[] ret = new double[1];
            final double dir1 = isAngle1 ? NumUtils.percentile_angle(data1, perc, false, averageType) : NumUtils.percentile(data1, perc, averageType);
            final double dir2 = isAngle2 ? NumUtils.percentile_angle(data2, perc, false, averageType) : NumUtils.percentile(data2, perc, averageType);
            final double x0 = isAngle1 ? NumUtils.average_angle(data1, false, averageType) : NumUtils.average(data1, averageType);
            final double y0 = isAngle2 ? NumUtils.average_angle(data2, false, averageType) : NumUtils.average(data2, averageType);
            double angle = 0;

            //ret[0] = dir1;
            //ret[1] = dir2;
            ret[0] = angle;

            MultivariateRealFunction objective = new MultivariateRealFunction() {

                public double value(double[] v) throws FunctionEvaluationException, IllegalArgumentException {
                    int count = 0;
                    for (int i = 0; i < data1.length; i++) {
                        if (containedInEllipse(data1[i], data2[i], new double[] { x0, y0, dir1, dir2, v[0] } )) {
                            count++;
                        }
                    }
                    double diff = Math.abs((double) (count) / (double) data1.length - perc);
                    return diff;
                }
            };

            DirectSearchOptimizer opt = new NelderMead();
            opt.setMaxIterations(500);
            RealPointValuePair p = opt.optimize(objective, GoalType.MINIMIZE, ret);
            final double finalAngle = p.getPoint()[0];

            ret = new double[] { 1. };

            objective = new MultivariateRealFunction() {
                public double value(double[] v) throws FunctionEvaluationException, IllegalArgumentException {
                    int count = 0;
                    for (int i = 0; i < data1.length; i++) {
                        if (containedInEllipse(data1[i], data2[i], new double[] { x0, y0, v[0] * dir1, v[0] * dir2, finalAngle } )) {
                            count++;
                        }
                    }
                    double diff = Math.abs((double) (count) / (double) data1.length - perc);

                    return diff;
                }
            };

            opt = new NelderMead();
            opt.setMaxIterations(500);
            p = opt.optimize(objective, GoalType.MINIMIZE, ret);
            final double finalDir1 = dir1 * p.getPoint()[0];
            final double finalDir2 = dir2 * p.getPoint()[0];

            ret = new double[] { x0, y0, finalDir1, finalDir2, finalAngle };

            return ret;
        } catch (Exception ex) {
            ex.printStackTrace();
            return null;
        }
    }
*/
	/**
	 * Copies the table source into the table dest.
	 * @param source
	 * @param dest
	 */
	static final public void copy(int[][] source, int[][] dest) {
        if (source == null) {
            dest = null;
            return;
        }

        for (int i = 0; i < source.length; i++) {
            System.arraycopy(source[i], 0, dest[i], 0, source[i].length);
        }
    }

	/**
	 * Returns a copy of the source table.
	 * @param source
	 * @return
	 */
	static final public int[][] copy(int[][] source) {
        if (source == null) {
            return null;
        }
        if (source.length == 0) {
            return new int[0][0];
        }

        int[][] ret = new int[source.length][source[0].length];
        copy(source, ret);
        return ret;
    }

	/**
	 * Copies the table source into the table dest.
	 * @param source
	 * @param dest
	 */
	static final public void copy(boolean[][] source, boolean[][] dest) {
        if (source == null) {
            dest = null;
            return;
        }

        for (int i = 0; i < source.length; i++) {
            System.arraycopy(source[i], 0, dest[i], 0, source[i].length);
        }
    }

	/**
	 * Copies the specified column of the table source into the dest array.
	 * @param source
	 * @param dest
	 * @param column
	 */
	static final public void copy(double[][] source, double[] dest, int column) {
        if (source == null) {
            dest = null;
            return;
        }

        for (int i = 0; i < source.length; i++) {
            dest[i] = source[i][column];
        }
    }

	/**
	 * Returns a copy of the table source.
	 * @param source
	 * @return
	 */
	static final public double[][] copy(double[][] source) {
        if (source == null) {
            return null;
        }
        if (source.length == 0) {
            return new double[0][0];
        }

        double[][] ret = new double[source.length][source[0].length];
        copy(source, ret);
        return ret;
    }


	/**
	 * Returns a copy of the double array source.
	 * @param source
	 * @return
	 */
	static final public double[] copy(double[] source) {
        if (source == null) {
            return null;
        }
        if (source.length == 0) {
            return new double[0];
        }

        double[] ret = new double[source.length];
        copy(source, ret);
        return ret;
    }

	/**
	 * Returns a copy of the String array source.
	 * @param source
	 * @return
	 */
	static final public String[] copy(String[] source) {
        if (source == null) {
            return null;
        }
        if (source.length == 0) {
            return new String[0];
        }

        String[] ret = new String[source.length];
        copy(source, ret);
        return ret;
    }


	/**
	 * Returns a copy of the boolean table source.
	 * @param source
	 * @return
	 */
	static final public boolean[][] copy(boolean[][] source) {
        if (source == null) {
            return null;
        }
        if (source.length == 0) {
            return new boolean[0][0];
        }

        boolean[][] ret = new boolean[source.length][source[0].length];
        copy(source, ret);
        return ret;
    }

	/**
	 * Attempts parsing a string into a numeric table. Each row is separated
         * by a newline character. Lines that start with a # character are considered
         * comments.
	 * @param source
	 * @return
	 */
	static public double[][] table(String source) {
        ArrayList<double[]> ret = new ArrayList<double[]>();
        String[] lines = source.split("\n");

        for (int i = 0; i < lines.length; i++) {
            if (lines[i].trim().startsWith("#")) {
                continue;
            }

            ret.add(numbers(lines[i]));
        }

        return Utils.toMatrix(ret);
    }

	/**
	 * Returns an array containing all the numbers in the input string.
	 * @param line
	 * @return
	 */
	static public double[] numbers(String line) {
        ArrayList<Double> nums = new ArrayList<Double>();

        Scanner sc = new Scanner(line);
        while (sc.hasNextDouble())
            nums.add(sc.nextDouble());

        return Utils.toArray(nums);
    }

	

	/**
	 * Returns the specified column from the given table.
	 * @param source
	 * @param column
	 * @return
	 */
	static public double[] column(double[][] source, int column) {
        double[] dest = new double[source.length];
        copy(source, dest, column);
        return dest;
    }

        /**
	 * Returns the specified column from the given table.
	 * @param source
	 * @param column
	 * @return
	 */
	static public double[] _column(double[][] source, int column) {
        double[] dest = new double[source.length];
        copy(source, dest, column);
        return dest;
    }
	/**
	 * Substitutes the specified column in the dest table with the subst array.
	 * @param dest
	 * @param subst
	 * @param column
	 */
	static public void column(double[][] dest, double[] subst, int column) {
        for (int i = 0; i < dest.length; i++)
            dest[i][column] = subst[i];
        return;
    }

	/**
	 * Fills the specified column in the dest table with the value subst.
	 * @param dest
	 * @param subst
	 * @param column
	 */
	static public void column(double[][] dest, double subst, int column) {
        for (int i = 0; i < dest.length; i++)
            dest[i][column] = subst;
        return;
    }


	/**
	 * Returns the given column of the table.
	 * @param source
	 * @param column
	 * @return
	 */
	static public boolean[] column(boolean[][] source, int column) {
        boolean[] dest = new boolean[source.length];
        for (int i = 0; i < dest.length; i++)
            dest[i] = source[column][i];
        return dest;
    }

	/**
	 * Substitutes the specified column in the dest table with the subst array.
	 * @param dest
	 * @param subst
	 * @param column
	 */
	static public void column(boolean[][] dest, boolean[] subst, int column) {
        for (int i = 0; i < dest.length; i++)
            dest[i][column] = subst[i];
        return;
    }

	/**
	 * Fills the specified column in the dest table with the subst value.
	 * @param dest
	 * @param subst
	 * @param column
	 */
	static public void column(boolean[][] dest, boolean subst, int column) {
        for (int i = 0; i < dest.length; i++)
            dest[i][column] = subst;
        return;
    }


	/**
	 * Reshapes a table. Given the source table, this method returns a new table
         * with the the specified number of rows and columns.
	 * @param src
	 * @param rows
	 * @param columns
	 * @return
	 */
	static public double[][] reshape(double[] src, int rows, int columns) {
		double[][] dest = new double[rows][columns];
		int k = 0;
		for (int i = 0; i < rows; i++)
			for (int j = 0; j < columns; j++) {
				dest[i][j] = src[k];
				k++;
			}

		return dest;
	}

	/**
	 * Multiplies the two arrays.
	 * @param result
	 * @param a
	 * @param b
	 */
	static public void emul(double[] result, double[] a, double[] b) {
        for (int i = 0; i < a.length; i++) {
            result[i] = a[i] * b[i];
        }
    }

	/**
	 * Divides the two arrays.
	 * @param result
	 * @param a
	 * @param b
	 */
	static public void ediv(double[] result, double[] a, double[] b) {
        for (int i = 0; i < a.length; i++) {
            result[i] = a[i] / b[i];
        }
    }

	/**
	 *
	 * @param result
	 * @param a
	 * @param b
	 */
	static public void emul(double[] result, double[] a, double b) {
        for (int i = 0; i < a.length; i++) {
            result[i] = a[i] * b;
        }
    }

    /**
	 *
	 * @param result
	 * @param a
     * @param column
     * @param b
	 */
	static public void emul(double[][] result, double[][] a, int column, double b) {
        result = Utils.copy(a);
        for (int i = 0; i < a.length; i++) {
            result[i][column] = a[i][column] * b;
        }
    }

	/**
	 *
	 * @param result
	 * @param a
	 * @param b
	 */
	static public void ediv(double[] result, double[] a, double b) {
        for (int i = 0; i < a.length; i++) {
            result[i] = a[i] / b;
        }
    }

	/**
	 *
	 * @param result
	 * @param a
	 * @param b
	 */
	static public void eadd(double[] result, double[] a, double[] b) {
        for (int i = 0; i < a.length; i++) {
            result[i] = a[i] + b[i];
        }
    }

	/**
	 *
	 * @param result
	 * @param a
	 * @param b
	 */
	static final public void esub(final double[] result, final double[] a, final double[] b) {
        for (int i = 0; i < a.length; i++) {
            result[i] = a[i] - b[i];
        }
    }

	/**
	 *
	 * @param result
	 * @param a
	 * @param b
	 */
	static final public void esub(final double[] result, final double[] a, final double b) {
        for (int i = 0; i < a.length; i++) {
            result[i] = a[i] - b;
        }
    }

	/**
	 *
	 * @param a
	 * @param b
	 * @return
	 */
	static final public double[] esub(final double[] a, final double[] b) {
        double[] ret = new double[a.length];
        esub(ret, a, b);
        return ret;
    }


	/**
	 *
	 * @param a
	 * @param b
	 * @return
	 */
	static final public double[] esub(final double[] a, final double b) {
        double[] ret = new double[a.length];
        esub(ret, a, b);
        return ret;
    }

	/**
	 * "Flattens" a table into an array (each row is appended to make an array).
	 * @param source
	 * @return
	 */
	static final public double[] flatten(final double[][] source) {
        double[] dest = new double[source.length * source[0].length];
        int k = 0;

        for (int i = 0; i < source.length; i++) {
            for (int j = 0; j < source[i].length; j++) {
                dest[k] = source[i][j];
                k++;
            }
        }

        return dest;
    }

	/**
	 * "Flattens" a table into an array (each row is appended to make an array).
	 * @param source
	 * @return
	 */
	static public boolean[] flatten(boolean[][] source) {
        boolean[] dest = new boolean[source.length * source[0].length];
        int k = 0;

        for (int i = 0; i < source.length; i++) {
            for (int j = 0; j < source[i].length; j++) {
                dest[k] = source[i][j];
                k++;
            }
        }

        return dest;
    }

	/**
	 * Fills a table with the value v.
	 * @param m
	 * @param v
	 */
	static public void fill(double[][] m, double v) {
        for (int i = 0; i < m.length; i++) {
            Arrays.fill(m[i], v);
        }
    }
   
    /**
     * Fills an array with the value v (equivalent to Array.fill).
     * @param m
     * @param v 
     */
    static public void fill(double[] m, double v) {
        Arrays.fill(m, v);
    }

    
   
    /**
     * Fills an array with the value v (equivalent to Array.fill).
     * @param m
     * @param v 
     */
    static public void fill(boolean[] m, boolean v) {
        Arrays.fill(m, v);
    }

    
   
    /**
     * Fills an array with the value v (equivalent to Array.fill).
     * @param m
     * @param v 
     */
    static public void fill(int[] m, int v) {
        Arrays.fill(m, v);
    }

    
    
   
    /**
     * Fills an array with the value v (equivalent to Array.fill).
     * @param m
     * @param v 
     */
    static public void fill(Object[] m, Object v) {
        Arrays.fill(m, v);
    }

	/**
	 * Fills a table with the value v.
	 * @param m
	 * @param v
	 */
	static public void fill(boolean[][] m, boolean v) {
        for (int i = 0; i < m.length; i++) {
            Arrays.fill(m[i], v);
        }
    }

	/**
	 * Fills the specified column of the table with the value v.
	 * @param m
	 * @param v
	 * @param column
	 */
	static public void fill(double[][] m, double v, int column) {
        for (int i = 0; i < m.length; i++) {
            m[i][column] = v;
        }
    }
    
    /**
	 * Fills the specified column of the table with the value v.
	 * @param m
	 * @param v
	 * @param column
	 */
	static public void fill(boolean[][] m, boolean v, int column) {
        for (int i = 0; i < m.length; i++) {
            m[i][column] = v;
        }
    }

	/**
	 * Fills the specified column of the table with the given array.
	 * @param m
	 * @param v
	 * @param column
	 */
	static public void fill(double[][] m, double[] v, int column) {
        for (int i = 0; i < m.length; i++) {
            m[i][column] = v[i];
        }
    }

    /**
     * Returns a number between r1 and r2, uniformly distributed.
     * @param r1
     * @param r2
     * @return 
     */
    static public double rndRange(double r1, double r2) {
        return r1 + (r2-r1) * randomGen.nextDouble();
    }
    
    static public int rndIntRange(int r1, int r2) {
        return r1 + randomGen.nextInt(r2-r1);
    }
    
    
    /**
     * Returns a number between r1 and r2, uniformly distributed in log.
     * @param r1
     * @param r2
     * @return 
     */
    static public double rndLogRange(double r1, double r2) {
        return Math.pow(10, Math.log10(r1) + Math.log10(r2)-(Math.log10(r1)) * randomGen.nextDouble());
    }
    
    
    
	/**
	 *
	 * @param data
	 * @param operation
	 * @return
	 */
	static public double[] apply(double[] data, String operation) {
	double[] data2 = new double[data.length];

	if (operation.equals("log10"))
	    for (int i = 0; i < data.length; i++)
		data2[i] = Math.log10(data[i]);

	return data2;
    }


	/**
	 * Fills the table with the value v.
	 * @param m
	 * @param v
	 */
	static public void fill(int[][] m, int v) {
        for (int i = 0; i < m.length; i++) {
            Arrays.fill(m[i], v);
        }
    }

	/**
	 * Fills the table with the value v.
	 * @param m
	 * @param v
	 * @param column
	 */
	static public void fill(int[][] m, int v, int column) {
        for (int i = 0; i < m.length; i++) {
            m[i][column] = v;
        }
    }

	/**
	 * Bins an array between the min and max values, for the given number of bins.
         * This routine returns a new table containing {{bin1, v1}, {bin2, v2}...} <i>bins</i> bins.
	 * @param data Input data.
	 * @param min Minimum value of the bin (can be null to use the minimum value of the data)
	 * @param max Maximum value of the bin (can be null to use the maximum value of the data)
	 * @param bins Number of bins
	 * @param binv2
	 * @param info
	 * @return
	 */
	static public double[][] bin(double[] data, Double min, Double max, int bins, double[][] binv2, double[] info) {
        if ((data == null) || (data.length == 0)) {
            return null;
        }

        double minv = 0;
        double maxv = 0;

        if ((min == null) || (max == null)) {
            double fmin = data[0];
            double fmax = data[0];

            for (int i = 1; i < data.length; i++) {
                fmin = min(data[i], fmin);
                fmax = max(data[i], fmax);
            }

            if (min == null) {
                minv = fmin;
            }
            if (max == null) {
                maxv = fmax;
            }
        }

        if (min != null) {
            minv = min;
        }
        if (max != null) {
            maxv = max;
        }

        double[][] binv = new double[bins][3];
        double bstep = (maxv - minv) / (double) (bins - 1);
        double norm = 0;

        for (int i = 0; i < data.length; i++) {
            if ((data[i] < minv) || (data[i] > maxv)) {
                continue;
            }

            int bin = (int) ((data[i] - minv) / bstep);
            binv[bin][1] += 1. / (double) data.length;

            if ((binv2 != null) && (i < (int) (data.length / 2))) {
                binv2[bin][1] += 2. / (double) data.length;
            }
        }

        for (int bin = 0; bin < bins; bin++) {
            binv[bin][0] = bin * bstep + minv;
            if (binv2 != null) {
                binv2[bin][0] = binv[bin][0];
            }
        }



        return binv;
    }

	/**
	 * @param data
	 * @param col
	 * @return
	 */
	static public double[][] cumsum(double[][] data, int col) {

        double[][] ret = Utils.copy(data);
        for (int i = 1; i < ret.length; i++)
            ret[i][col] += ret[i-1][col];

        return ret;
    }

        /**
         *
         * @param data
         * @return
         */
        static public double sum(double[] data) {
            double ret = data[0];
            for (int i = 0; i < data.length; i++)
                ret += data[i];
            return ret;
        }

	/**
	 *
	 * @param num
	 * @return
	 */
	static public boolean isNumeric(String num) {
        try {
            Double.valueOf(num);
        } catch (Exception e) {
            return false;
        }
        return true;
    }

	/**
	 *
	 * @param data
	 * @param min
	 * @param max
	 * @param bins
	 * @param binv2
	 * @param info
	 * @return
	 */
	static public double[][] bin_nn(double[] data, Double min, Double max, int bins, double[][] binv2, double[] info) {
        if ((data == null) || (data.length == 0)) {
            return null;
        }

        double minv = 0;
        double maxv = 0;

        if ((min == null) || (max == null)) {
            double fmin = data[0];
            double fmax = data[0];

            for (int i = 1; i < data.length; i++) {
                fmin = min(data[i], fmin);
                fmax = max(data[i], fmax);
            }

            if (min == null) {
                minv = fmin;
            }
            if (max == null) {
                maxv = fmax;
            }
        }

        if (min != null) {
            minv = min;
        }
        if (max != null) {
            maxv = max;
        }

        double[][] binv = new double[bins][3];
        maxv = maxv + (maxv - minv) / (double) bins;

        double bstep = (maxv - minv) / (double) (bins - 1);


        for (int i = 0; i < data.length; i++) {
            if ((data[i] < minv) || (data[i] > maxv)) {
                continue;
            }

            int bin = (int) ((data[i] - minv) / bstep);
            binv[bin][1] += 1.;

            if ((binv2 != null) && (i < (int) (data.length / 2))) {
                binv2[bin][1] += 2.;
            }
        }

        for (int bin = 0; bin < bins; bin++) {
            binv[bin][0] = bin * bstep + minv;
            if (binv2 != null) {
                binv2[bin][0] = binv[bin][0];
            }
        }

        System.out.println("Done");
        for (int i = 0; i < binv.length; i++)
            System.out.println(binv[i][0] +  " " + binv[i][1]);
        System.out.println("Done");
        return binv;
    }

	/**
	 *
	 * @param arr
	 * @param el
	 * @return
	 */
	static public String[] append(String[] arr, String el) {
        if (arr == null)
            return new String[] { el };
        String[] ret = new String[arr.length + 1];

        for (int i = 0; i < arr.length; i++)
            ret[i] = arr[i];
        ret[ret.length-1] = el;

        return ret;
    }

    static public <T> T[] deleteIndex(T[] arr, int index) {
        T[] ret = Arrays.copyOf(arr, arr.length-1);
        System.arraycopy(arr, 0, ret, 0, index);
        for (int i = index + 1; i < arr.length; i++)
            ret[i-1] = arr[i];
        return ret;
    }

    static public double[] deleteIndex(double[] arr, int index) {
        double[] ret = Arrays.copyOf(arr, arr.length-1);
        System.arraycopy(arr, 0, ret, 0, index);
        for (int i = index + 1; i < arr.length; i++)
            ret[i-1] = arr[i];
        return ret;
    }

	/**
	 *
	 * @param s
	 * @param defaultn
	 * @return
	 */
	static public double valueOf(String s, double defaultn) {
        try {
            double t = Double.valueOf(s);
            return t;
        } catch (Exception e) {
            return defaultn;
        }
    }

	/**
	 *
	 * @param arr1
	 * @param arr2
	 * @return
	 */
	static public double[][] append(double[][] arr1, double[][] arr2) {
        double[][] arrret = null;
        if (arr1 == null)
            return copy(arr2);

        arrret = new double[arr1.length + arr2.length][arr1[0].length];

        for (int i = 0; i < arr1.length; i++) {
            arrret[i] = arr1[i].clone();
        }
        for (int i = 0; i < arr2.length; i++) {
            arrret[i + arr1.length] = arr2[i].clone();
        }

        return arrret;
    }

	/**
	 *
	 * @param arr1
	 * @param arr2
	 * @return
	 */
	static public double[][] append(double[][] arr1, double[] arr2) {
        double[][] arrret = null;
        if (arr1 == null) {
	    arrret = new double[1][arr2.length];
	    arrret[0] = arr2;
            return arrret;
	}

        arrret = new double[arr1.length + 1][arr1[0].length];

        for (int i = 0; i < arr1.length; i++) {
            arrret[i] = arr1[i].clone();
        }

        arrret[arrret.length-1] = arr2.clone();

        return arrret;
    }

	/**
	 *
	 * @param arr1
	 * @param arr2
	 * @return
	 */
	static public double[][] append_right(double[][] arr1, double[] arr2) {
        double[][] arr_ret = null;
        if (arr1 == null) {
	    arr_ret = new double[arr2.length][1];
            for (int i = 0; i < arr2.length; i++)
                arr_ret[i][0] = arr2[i];
            return arr_ret;
	}

        arr_ret = new double[arr1.length][arr1[0].length + 1];

        for (int i = 0; i < arr1.length; i++) {
            for (int j = 0; j < arr1[i].length; j++)
                arr_ret[i][j] = arr1[i][j];
            arr_ret[i][arr_ret[i].length-1] = arr2[i];
        }

        return arr_ret;
    }



	/**
	 *
	 * @param arr1
	 * @param arr2
	 * @return
	 */
	static public double[] append(double[] arr1, double[] arr2) {
        if (arr1 == null)
            return Utils.copy(arr2);

        double[] arrret = new double[arr1.length + arr2.length];
        for (int i = 0; i < arr1.length; i++)
            arrret[i] = arr1[i];
        for (int i = 0; i < arr2.length; i++)
            arrret[i+arr1.length] = arr2[i];
        return arrret;
    }

	/**
	 *
	 * @param arr1
	 * @param arr2
	 * @param idx
	 * @return
	 */
	static public final double[][] insert(double[][] arr1, double[] arr2, int idx) {
        double[][] arrret = new double[arr1.length + 1][];

        for (int i = 0; i < idx; i++) {
            arrret[i] = Utils.copy(arr1[i]);
        }
        arrret[idx] = Utils.copy(arr2);
        for (int i = idx; i < arr1.length; i++) {
            arrret[i+1] = Utils.copy(arr1[i]);
        }
        return arrret;
    }

	/**
	 *
	 * @param arr
	 * @return
	 */
	public static double[][] toMatrix(java.util.List<double[]> arr) {
        if ((arr == null) || (arr.size() == 0))
            return null;

        double[][] ret = new double[arr.size()][];
        for (int i = 0; i < arr.size(); i++)
            ret[i] = arr.get(i);
        return ret;
    }

	/**
	 *
	 * @param arr
	 * @return
	 */
	public static double[] toArray(java.util.List<Double> arr) {
        if ((arr == null) || (arr.size() == 0))
            return null;

        double[] ret = new double[arr.size()];
        for (int i = 0; i < arr.size(); i++)
            ret[i] = arr.get(i);
        return ret;
    }
        
        public static double toDouble(String val) {
            if (val.equalsIgnoreCase("nan"))
                return Double.NaN;
            else if (val.equalsIgnoreCase("inf"))
                return Double.POSITIVE_INFINITY;
            else if (val.equalsIgnoreCase("-inf"))
                return Double.NEGATIVE_INFINITY;
            else
                return Double.valueOf(val);
        }


    // only use for small ArrayLists
	/**
	 *
	 * @param what
	 * @param sep
	 * @return
	 */
	public static String concat(java.util.List<String> what, String sep) {
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < what.size(); i++)
            sb.append(what.get(i) + sep);
        return sb.toString();
    }

	/**
	 *
	 * @param arr
	 * @param arrcp
	 * @param col
	 * @param startRow
	 * @param endRow
	 */
	public static final void shellSort(double[][] arr, double[][] arrcp, int col,
            int startRow, int endRow) {
        copy(arr, arrcp);
        shellSort(arrcp, col, startRow, endRow);
    }

	/**
	 *
	 * @param arr
	 * @param col
	 * @param startRow
	 * @param endRow
	 */
	public static void shellSort(final double[][] arr, final int col, final int startRow,
            int endRow) {
        shellSort(arr, col, startRow, endRow, false);
    }

        /**
	 *
	 * @param arr
	 * @param col
	 * @param startRow
	 * @param endRow
	 */
	public static void shellSort(final double[][] arr, final int col) {
        shellSort(arr, col, 0, arr.length-1, false);
    }
    
    public static void shellSort(final double[][] arr, final int col, final boolean descending) {
        shellSort(arr, col, 0, arr.length-1, descending);
    }

	/**
	 *
	 * @param arr
	 * @param col
	 * @param startRow
	 * @param endRow
	 * @param descending
	 */
	public static void shellSort2(final double[][] arr, final int col, final int startRow,
            int endRow, boolean descending) {
        int i, j, inc;
        int n = endRow - startRow + 1;
        double[] v = new double[arr[0].length];
        inc = 1;
        do {
            inc *= 3;
            inc++;
        } while (inc <= n);
        do {
            inc /= 3;
            for (i = inc + 1; i <= n; i++) {
                copy(arr[startRow + i - 1], v);
                j = i;

                if (descending) {
                    while (arr[startRow + j - inc - 1][col] < v[col]) {
                        copy(arr[startRow + j - inc - 1], arr[startRow + j - 1]);
                        j -= inc;
                        if (j <= inc) {
                            break;
                        }
                    }
                } else {
                    while (arr[startRow + j - inc - 1][col] > v[col]) {
                        copy(arr[startRow + j - inc - 1], arr[startRow + j - 1]);
                        j -= inc;
                        if (j <= inc) {
                            break;
                        }
                    }
                }
                copy(v, arr[startRow + j - 1]);
            }
        } while (inc > 1);
    }

	/**
	 *
	 * @param arr
	 * @param col
	 * @param startRow
	 * @param endRow
	 * @param descending
	 */
	public final static void shellSort(final double[][] arr, final int col, final int startRow,
            final int endRow, final boolean descending) {
        int i, j, inc;
        int n = endRow - startRow + 1;
        double[] v;
        inc = 1;
        do {
            inc *= 3;
            inc++;
        } while (inc <= n);
        do {
            inc /= 3;
            for (i = inc + 1; i <= n; i++) {
                v = arr[startRow + i - 1];
                j = i;

                if (descending) {
                    while (arr[startRow + j - inc - 1][col] < v[col]) {
                        arr[startRow + j - 1] = arr[startRow + j - inc - 1];
                        j -= inc;
                        if (j <= inc) {
                            break;
                        }
                    }
                } else {
                    while (arr[startRow + j - inc - 1][col] > v[col]) {
                        arr[startRow + j - 1] = arr[startRow + j - inc - 1];
                        j -= inc;
                        if (j <= inc) {
                            break;
                        }
                    }
                }
                arr[startRow + j - 1] = v;
            }
        } while (inc > 1);
    }


	/**
	 *
	 * @param arr
	 * @param col
	 * @param ascending
	 * @return
	 */
	public static Integer[] sort(final double[][] arr, final int col, final boolean ascending) {

	Integer[] indices = new Integer[arr.length];
	for (int i = 0; i < indices.length; i++) indices[i] = i;

	Arrays.sort(indices, new Comparator<Integer>() {
	    public int compare(Integer o1, Integer o2) {
		int ret =  signum(arr[o1][col] - arr[o2][col]);
		if (!ascending) ret = - ret;
		return ret;
	    }

	    public final int signum(double diff) {
		if (diff > 0) {
		    return 1;
		}
		if (diff < 0) {
		    return -1;
		} else {
		    return 0;
		}
	    }
	});

	return indices;
    }

    /**
	 *
	 * @param arr
	 * @param col
	 * @param ascending
	 * @return
	 */
	public static Integer[] sort(final double[] arr, final boolean ascending) {

	Integer[] indices = new Integer[arr.length];
	for (int i = 0; i < indices.length; i++) indices[i] = i;

	Arrays.sort(indices, new Comparator<Integer>() {
	    public int compare(Integer o1, Integer o2) {
		int ret =  signum(arr[o1] - arr[o2]);
		if (!ascending) ret = - ret;
		return ret;
	    }

	    public final int signum(double diff) {
		if (diff > 0) {
		    return 1;
		}
		if (diff < 0) {
		    return -1;
		} else {
		    return 0;
		}
	    }
	});

	return indices;
    }

	/**
	 *
	 * @param arr
	 * @param indices
	 * @return
	 */
	public static double[][] applySort(final double[][] arr, final Integer[] indices) {
	double[][] newArr = new double[arr.length][];

	for (int i = 0; i < indices.length; i++)
	    newArr[i] = arr[indices[i]];

	return newArr;
    }
    
    

    /**
	 *
	 * @param arr
	 * @param indices
	 * @return
	 */
	public static double[] applySort(final double[] arr, final Integer[] indices) {
	double[] newArr = new double[arr.length];

	for (int i = 0; i < indices.length; i++)
	    newArr[i] = arr[indices[i]];

	return newArr;
    }

	/**
	 *
	 * @param arr
	 * @param indices
	 * @return
	 */
	public static int[][] applySort(final int[][] arr, final Integer[] indices) {
	int[][] newArr = new int[arr.length][];

	for (int i = 0; i < indices.length; i++)
	    newArr[i] = arr[indices[i]];

	return newArr;
    }

	/**
	 *
	 * @param arr
	 * @param indices
	 * @return
	 */
	public static String[] applySort(String[] arr, Integer[] indices) {
	String[] newArr = new String[arr.length];

	for (int i = 0; i < indices.length; i++)
	    newArr[i] = arr[indices[i]];

	return newArr;
    }

	/**
	 *
	 * @param arr
	 * @param indices
	 * @return
	 */
	public static Object[] applySort(Object[] arr, Integer[] indices) {
	Object[] newArr = new Object[arr.length];

	for (int i = 0; i < indices.length; i++)
	    newArr[i] = arr[indices[i]];

	return newArr;
    }
    
    
    

	/**
	 *
	 * @param out
	 * @param table
	 * @param fmt
	 */
	public static void printTable(PrintStream out, int[][] table, String fmt) {
	for (int i = 0; i < table.length; i++) {
	    for (int j = 0; j < table[i].length; j++)
		out.printf(fmt, table[i][j]);
	    out.println();
	}
    }

	/**
	 *
	 * @param out
	 * @param table
	 * @param fmt
	 */
	public static void printTable(PrintStream out, double[][] table, String fmt) {
	for (int i = 0; i < table.length; i++) {
	    for (int j = 0; j < table[i].length; j++)
		out.printf(fmt, table[i][j]);
	    out.println();
	}
    }

    public static void printTable(String fn, double[][] table, String fmt) {
        PrintStream ps = openFile(fn, false);
        printTable(ps, table, fmt);
    }

    public static void printTable(String fn, double[] table, String fmt) {
        PrintStream ps = openFile(fn, false);
        printTable(ps, table, fmt);
    }
    

    
	/**
	 *
	 * @param out
	 * @param table
	 * @param fmt
	 */
	public static void printTable(PrintStream out, double[] table, String fmt) {
	for (int i = 0; i < table.length; i++) {
	    out.printf(fmt, table[i]);
	}
        out.println();
    }

    public static PrintStream openFile(String fn, boolean append) {
        PrintStream ps = null;
        try {
            ps = new PrintStream(new FileOutputStream(fn, append));
            return ps;
        } catch (FileNotFoundException ex) {
            return null;
        }
    }
    
	/**
	 *
	 * @param d1
	 * @param d2
	 * @param tolerance
	 * @return
	 */
	public static boolean equals(double d1, double d2, double tolerance) {
	return Math.abs(d1 - d2) < tolerance;
    }


        /**
         *
         * @param data
         * @return
         */
        public static double[][] bootstrapDataSet(double[][] data) {
            double[][] ret = new double[data.length][];
            Random r = new Random();
            for (int i = 0; i < ret.length; i++) {
                int idx = r.nextInt(ret.length);
                ret[i] = Utils.copy(data[idx]);
                //ret[i][1] = data[i][1] + r.nextGaussian() * data[i][2];
            }

            shellSort(ret, 0, 0, ret.length-1);
            return ret;
        }

        /**
         *
         * @param data
         * @return
         */
        public static double[][] bootstrapDataSetAroundMean(double[][] data) {
            double[][] ret = new double[data.length][];
            Random r = new Random();
            double[] v = Utils.column(data, 1);
            double mean = NumUtils.average(v);
            double std = NumUtils.stdDev(v);
            for (int i = 0; i < ret.length; i++) {
                int idx = r.nextInt(ret.length);
                ret[i] = Utils.copy(data[idx]);
                ret[i][1] = r.nextGaussian() * std + mean;
            }

            shellSort(ret, 0, 0, ret.length-1);
            return ret;
        }

        /**
         *
         * @param data
         * @return
         */
        public static double[][] bootstrapDataSetByScrambling(double[][] data) {
            double[][] ret = new double[data.length][3];
            ArrayList<double[]> arr = new ArrayList<double[]>();
            Collections.addAll(arr, data);

            Random r = new Random();

            for (int i = 0; i < ret.length; i++) {
                ret[i][0] = data[i][0];

                int idx = r.nextInt(arr.size());
                ret[i][1] = arr.get(idx)[1];
                ret[i][2] = arr.get(idx)[2];
                arr.remove(idx);
            }


            return ret;
        }

        /**
         *
         * @param data
         * @param dev
         * @return
         */
        public static double[][] bootstrapCreateGaussianDeviates(double[][] data, double dev) {
            Random r = new Random();

            double[][] ret = new double[data.length][3];

            for (int i = 0; i < ret.length; i++) {
                ret[i][0] = data[i][0];
                ret[i][2] = 1.;
                ret[i][1] = r.nextGaussian() * dev;
            }

            return ret;
        }

        private static Random randomGen = new Random();

        public static double[] shuffle(double[] array) {
            array = Utils.copy(array);
            for (int i = 0; i < array.length; i++) {
                int j = randomGen.nextInt(array.length);
                double old = array[i];
                array[i] = array[j];
                array[j] = old;
            }
            return array;
        }

        public static double[][] shuffle(double[][] array) {
            array = Utils.copy(array);
            for (int i = 0; i < array.length; i++) {
                int j = randomGen.nextInt(array.length);
                double[] old = array[i];
                array[i] = array[j];
                array[j] = old;
            }
            return array;
        }
        
        public static double[][] sampleDataSet(double[][] dataset, int length) {
            double[][] ret = new double[length][3];
           
            double t0 = dataset[0][0];
            int ridx = 0;
            Random ran = new Random();
            
            for (int i = 0; i < length; i++) {
                if (i != 0) {
                    ret[i][0] = ret[i-1][0] + (dataset[ridx][0] - dataset[ridx-1][0]);
                } else
                    ret[i][0] = dataset[0][0];
                ret[i][2] = dataset[ran.nextInt(dataset.length)][2];
                ridx ++;
                if (ridx == dataset.length)
                    ridx = 1;
            }
            return ret;
        }
        
        public static double[][] readTable(String fn) {
            return FileUtils.readTable(fn);
        }
        
        public static double[][] readBinaryTable(String fn, double[][] curve) {
    
            DataInputStream dis;
            try {
                dis = new DataInputStream(new FileInputStream(fn));

                for (int i = 0; i < curve.length; i++) 
                    for (int j = 0; j < curve[i].length; j++)
                        curve[i][j] = EndianUtils.readSwappedDouble(dis);
                dis.close();
                return curve;
            } catch (Exception ex) {
                return null;
            } 
          
        }        
}
