-- Mistery meat glue between C and Lua and Java

require 'systemic'
require 'utils'
local bit = require 'bit'

local band = bit.band
local bor = bit.bor
local inp = {}
local _G2 = {}

function _nassert(cnd, str, ...)
	if not cnd then
		error(string.format(str, ...))
	end
end

	
local function wrap_and_broadcast(method, flag) 
	return function(...)
		method(...)
		local k = select(1, ...)
		gui.changed(select(1, ...), flag, ...)
	end
end

local function wrap_getset(method, flag)
	return function(k, v)
		local ret = method(k, v)
		if v then
			gui.changed(k, flag)
		end
		return ret
	end
end

local function parse(str)
	str = ex.trim(str)
	if (str:sub(1, 1) == "=") then
		str = string.format("return %s", str:sub(2))
	elseif str:match("^!%d+") then
		local n = tonumber(str:match("!(%d+)"))
		if inp[n] then return inp[n] end
	end
	return str
end

local function write_table(tbl, fn)
	local fid = io.open(fn, 'w')
	if not fid then error("Could not open " .. fn) end
	
	for k, v in pairs(tbl) do
		if type(v) == "number" then
			fid:write(string.format("%s = %.15e\n", k, v))
		elseif type(v) == "string" then
			fid:write(string.format("%s = [==[%s]==]", k, v))
		end
	end
	
	fid:close()
end

matrix.save_gui = function(m)
	if not m then
		print(-1)
		return
	end
	if type(m) == "table" then
		m = m._h
	end
	print(tonumber(m.size1), tonumber(m.size2))
	_sys.ok_save_matrix_bin(m, nil)
end

matrix.save_gui_int = function(m)
	if not m then
		print(-1)
		return
	end
	if type(m) == "table" then
		m = m._h
	end
	print(tonumber(m.size1), tonumber(m.size2))
	_sys.ok_fprintf_matrix_int(m, nil, "%d ")
end

vector.save_gui_int = function(v)
	if not v then
		print(-1)
		return
	end
	print(tonumber(v.size))
	_sys.ok_fprintf_vector_int(v, nil, "%d ")
end

defaults = {samples = 2500, per_samples = 10000, message = "Hello master %s!\nThis is Console %.4f speaking\nStefano Meschiari, >>http://www.stefanom.org\nType acknowledgments() to view a list of acknowledgments.\n"}

local __gui = {
	ELEMENTS_CHANGED = 1,
	DATA_CHANGED = 2,
	STATS_CHANGED = 4,
	silent = false,
	stopped = false,
	path = ".",
	locks = {},
	__nosave = true,
	lock = function(fnstr, val)
		if gui.locks[fnstr] then
			gui.locks[fnstr] = nil
		else
			gui.locks[fnstr] = val
			
			for k, v in pairs(_G2) do
				if type(v) == "table" and v.__class == "kernel" then
					kernel[fnstr](v, val)
				end
			end
		end
	end,
	changed = function(who, what)
		if gui.silent then
			return
		end

		local ndata = who:ndata()	
		if band(what, gui.DATA_CHANGED) ~= 0 then
			who:calculate()

			if ndata > 0 then
				local c = _sys.K_getCompiled(who._h)

				print("@@@@,data,", who._name)
				print(ndata, _sysc.DATA_SIZE)
				io.stdout:flush()
				if not __gui_debug then
					_sys.ok_save_buf_bin(c, nil, ndata, _sysc.DATA_SIZE)
				end
				io.stdout:flush()
						
				print("@@@@,periodogram,", who._name)
				if who:nrvs() > 0 then
					who:periodogram("all", defaults.per_samples):save_gui()
				else
					print(-1)
				end
			else
				print("@@@@,data,", who._name, "\n-1")
				print("@@@@,periodogram,", who._name, "\n-1")
			end
		end
		if who:epoch() == who:epoch() and who:nplanets() > 0 then
			local from, to = who:trange()
			if from and to then
				local sl = _sys.K_integrateRange(who._h, from, to, defaults.samples, nil)
				local m = matrix.new(_sys.ok_get_rvs(sl, defaults.samples))
				print("@@@@,rvcurve,", who._name)
				m:save_gui()
				
				for i = 0, defaults.samples-1 do
					_sys.ok_free_system(sl[i])
				end
				ffi.C.free(sl)
				print("@@@@,per_res,", who._name)
				if who:nrvs() > 0 then
					who:periodogram("res", defaults.per_samples):save_gui()
				else
					print(-1)
				end
			end
		else
			print("@@@@,rvcurve,", who._name, "\n-1")
			print("@@@@,per_res,", who._name, "\n-1")
		end
		if band(what, gui.STATS_CHANGED) ~= 0 then
			gui.stats(who)
		end
	end,
	setdir = function(dir)
		rawset(gui, 'path', dir)
	end, 
	stats = function(k)
		local nsets = k._h.nsets
		local t = {chi2 = k._h.chi2, rms = k._h.rms, jitter = k._h.jitter, ndata = k._h.ndata, nsets = k._h.nsets, mstar = k._h.Mstar, intmethod = k._h.intMethod, epoch = _sys.K_getEpoch(k._h), nrvs = k._h.nrvs, ntts = k._h.ntts, chi2_rvs = k._h.chi2_rvs, chi2_tts = k._h.chi2_tts, rms_tts = k._h.rms_tts, nrpars = k._h.nrpars, nplanets = k:nplanets()}
		
		for n = 1, nsets do
			t[string.format("%d!%s", n, k.names[n] or ("@Data" .. n))] = _sys.K_getPar(k._h, n-1)
		end
		write_table(t, gui.path .. k._name .. "_stats")
		print("@@@@,stats,", k._name)
		print("@@@@,state,", k._name)
		matrix.save_gui(matrix.new(_sys.K_getAllElements(k._h)))
		matrix.save_gui_int(_sys.K_getElementFlags(k._h))
		vector.save_gui_int(_sys.K_getParFlags(k._h))
	end,
	save = function(fn)
		local cnt = 1
		local fid = io.open(fn, "w")
		local ks = {}
		_nassert(fid, "Could not open %s", fn)
		local buf = ex.save_table(_G2, function(t, k, depth)
			if type(t) == "table" and t.__class then
				if t.__class == "kernel" and _G2[k] == t then
					local b = buf.buffer()
					ks[cnt] = {k, t}
					cnt = cnt+1
					b:appendf("__knew([[%s]], %d, {\n",
						k, cnt-1)
					for i = 1, #t.names do
						b:append("[[", t.names[i], "]], ")
					end
					b:append("})\n")
					return b:tostring()
				else
					return ""
				end
			end
			if type(t) == "table" and t.__nosave then
				return ""
			end
		end)
		fid:write(buf)
		fid:close()
		local fidc = ffi.C.fopen(fn, "a")
		ffi.C.fputs("--[[\n", fidc)
		for i = 1, #ks do
			_sys.K_save(ks[i][2]._h, fidc)
		end

		ffi.C.fclose(fidc)
		print("@@@@,savesession,", fn)
		print("Saved to ", fn)
	end,
	load = function(fn)
		_nassert(ex.exists(fn), "Could not open %s", fn)
		local buf = ex.loadfile(fn)
		local f, err = loadstring(buf)
		_nassert(f, "Could not load %s: %s", fn, err)
		local ks = {}
		local __env = {["__knew"] = function(...)
			ks[#ks+1] = {...}
		end}
		setfenv(f, __env)
		
		local ret = f()()
		
		for k, v in pairs(ret) do
			_G[k] = v
		end
		
		for i = 1, #ks do
			local name = ks[i][1]
			local skip = ks[i][2]
			local names = ks[i][3]
			_G[name] = kernel.new(fn, skip)
			_G[name].names = names
			gui.changed(_G[name], gui.DATA_CHANGED)
		end
		print("@@@@,clearsession,")
		print(buf:match("@Output(.-)@End"))
		print(fn, "loaded")

		return ret
	end,
	run = function() 
	-- input loop
		print(string.format(defaults.message, os.getenv("USER"), _sysc.VERSION))
		while (true) do
			rawset(gui, 'stopped', false)
			os.remove(gui.path .. "_stop")
			local s = io.read("*l")
			print("@@@@,busy,")	
			io.stdin:flush()
			io.stdout:flush()	
			local f, msg = loadstring(parse(s))
			if f then 
				local ret, msg = pcall(f)
				if not ret then
					_noticef(msg or "")
				elseif msg then
					print(msg)
				end
			else
				_noticef(msg)
			end
			print("@@@@,ready,")	
			io.stdin:flush()
			io.stdout:flush()	
			io.stderr:flush()
		end
	end
}


local progress = function(cur, max, h, str)
	h = ffi.cast("ok_kernel*", h)
	print(string.format("@@@@,progress,%d|%d|%e|%s", cur, max, h.chi2, job))
	io.stdout:flush()
	if ex.exists(gui.path .. "_stop") then
		os.remove(gui.path .. "_stop")
		io.stderr:write("Stop requested, please wait...\n")
		rawset(gui, 'stopped', true)
		return _sysc.PROGRESS_STOP
	end
	return _sysc.PROGRESS_CONTINUE
end


setmetatable(_G, {
	__newindex = function(t, k, v)
		if type(v) == "table" and v.__class == "kernel" and v._name then
			_noticef("You cannot alias kernel objects (you are trying to set %s = %s.). If you want to make a copy, use %s = %s:clone()",
			k, v._name, k, v._name)
		elseif type(v) == "table" and v.__class == "kernel" and v._new then
			v._name = k
			v._new = false
			print("@@@@,newkernel,", k)
			v._h.progress = ffi.cast("ok_progress", progress)
			gui.changed(v, bor(gui.STATS_CHANGED, gui.ELEMENTS_CHANGED, gui.DATA_CHANGED))
			
			for fn, val in pairs(gui.locks) do
				kernel[fn](v, val)
			end

		elseif type(_G2[k]) == "table" and _G2[k].__class == "kernel" and not v then
			print("@@@@,delkernel,", k)
		end
		_G2[k] = v
	end,
	__index = function(t, k)
		if not _G2[k] and type(k) == "string" and ex.exists(k .. ".lua") then
			dofile(k .. ".lua")
		end
		return _G2[k]
	end
})

_G2.gui = setmetatable({}, {
	__index = function(t, k)
		return __gui[k]
	end,
	__newindex = function(t, k, v) 
		print("Eeek! Please don't mess with my internals. *cowers in a corner*")
		print(string.format("Why would you try to change my %s to %s?", tostring(k), tostring(v)))
	end
})

kernel.__set = wrap_and_broadcast(kernel.__set, bor(gui.ELEMENTS_CHANGED, gui.DATA_CHANGED))
kernel.__set_data = wrap_and_broadcast(kernel.__set_data, gui.DATA_CHANGED)
kernel.__add = kernel.add
kernel.add = wrap_and_broadcast(kernel.add, bor(gui.ELEMENTS_CHANGED, gui.DATA_CHANGED))
kernel.remove = wrap_and_broadcast(kernel.remove, bor(gui.ELEMENTS_CHANGED, gui.DATA_CHANGED))

local _minimize = kernel.minimize
kernel.minimize = function(k,...)
	local t = os.time()
	_minimize(k, ...)

	gui.changed(k, bor(gui.ELEMENTS_CHANGED, gui.DATA_CHANGED))
	if os.time() - t > 2 then
		print(os.time() - t, "seconds")
	end
end
local _minimize1d = kernel.minimize1d
kernel.minimize1d = function(k,...)
	local t = os.time()
	_minimize1d(k, ...)
	gui.changed(k, bor(gui.ELEMENTS_CHANGED, gui.DATA_CHANGED))
	if os.time() - t > 2 then
		print(os.time() - t, "seconds")
	end
end
local _add_data = kernel.add_data
kernel.add_data = function(k, name)
	local ret = _add_data(k, name)
	if ret then
		k.names = k.names or {}
		
		if type(name) == "string" then
			name = {name}
		end
		
		for i = 1, #name do
			k.names[#k.names + 1] = name[i]:gsub("%!", "")
		end
	end

	gui.changed(k, gui.DATA_CHANGED)
end
local _remove_data = kernel.remove_data
kernel.remove_data = function(k, idx)
	local ret = _remove_data(k, idx)
	if idx == -1 then
		k.names = {}
	elseif ret then
		table.remove(k.names, idx)
	end
	gui.changed(k, gui.DATA_CHANGED)
end


local _new = kernel.new
kernel.new = function(props, ...)
	local ret = _new(props, ...)
	ret.names = {}
	if (type(props) == "table" and props.__class == "kernel" and props.names) then
		for i = 1, #props.names do
			ret.names[i] = props.names[i]
		end
	end
	
	ret._new = true
	return ret
end

local _flag = kernel.flag
kernel.flag = function(k, r, c, v)
	local r = _flag(k, r, c, v)
	if v then
		gui.stats(k)
	end
	return r
end

kernel.int_method = wrap_getset(kernel.int_method, bor(gui.ELEMENTS_CHANGED, gui.DATA_CHANGED))
kernel.mstar = wrap_getset(kernel.mstar, bor(gui.ELEMENTS_CHANGED, gui.DATA_CHANGED))
kernel.epoch = wrap_getset(kernel.epoch, bor(gui.ELEMENTS_CHANGED, gui.STATS_CHANGED))

local _c = kernel.calculate
kernel.calculate = function(k)
	_c(k)
	gui.stats(k)
end

-- Meooow
function cat()
	print("Meow!")
	print("@@@@,catz")
end

if not __gui_debug then
	gui.run()
end
