buf = buf or {}

function buf.buffer()
	local b = {}
	setmetatable(b, buf)
	return b
end

function buf.append(buf, ...)
	for i = 1, select('#', ...) do
		buf[#buf+1] = tostring(select(i, ...))
	end
	return buf
end

function buf.appendf(buf, fmt, ...)
	buf[#buf+1] = string.format(fmt, ...)
	return buf
end

function buf.read(buf, f, what, mode)
	what = what or "*l"
	local fid = f
	if type(f) == "string" then
		fid = io.open(f)
	end
	
	assert(fid)
	
	local c = fid:read(what)
	while c do
		if mode == "lower" then
			buf:append(c:lower())
		elseif mode == "upper" then
			buf:append(c:upper())
		elseif type(mode) == "function" then
			local ret = mode(c)
			if ret then buf:append(ret) end
		else
			buf:append(c)
		end
		c = fid:read(what)
	end
	
	if type(f) == "string" then
		fid:close()
	end
	
	return buf
end

function buf.tostring(buf, fmt, ...)
	return table.concat(buf)
end

function buf.clear(buf)
	for i = 1, #buf do buf[i] = nil end
end

function buf.__index(b, n)
	return buf[n]
end

ex = {}
function ex.capture(cmd)
	local pipe = io.popen(cmd, "r")
	local str = pipe:read("*a")
	return str
end

function ex.exists(fn)
	local fid = io.open(fn)
	if not fid then return false end
	fid:close()
	return true
end	

function ex.trim(str)
	return (string.gsub(str, "^%s*(.-)%s*$", "%1"))
end

function ex.loadfile(fn)
	local fid = io.open(fn)
	if not fid then return nil end
	local ret = fid:read("*a")
	fid:close()
	return ret
end

function ex.sandbox(...)
	local ts = {...}
	local fns = {}
	for i = 1, #ts do
		fns[ts] = {}
		for k, v in pairs(ts[i]) do
			if type(v) == "function" then
				fn[ts][k] = v
				ts[k] = function() error(string.format("Function %s not permitted.", tostring(k))) end
			end
		end
	end

	return function()
		for i = 1, #ts do
			for k, v in pairs(ts[i]) do
				if type(v) == "function" then
					ts[k] = fn[ts][k]
				end
			end
		end
	end
end

local function str_rep(v)
	if type(v) == "string" then
		return string.format("[==[%s]==]", v)
	else
		return tostring(v)
	end
end

function ex.save_table(t, sf, b, ks, depth)
	local b = b or buf.buffer()
	local ks = ks or {}
	local sf = sf or function() return false end
	local depth = depth or 0
	if depth == 0 then
		b:append("return function()\nlocal _0 = {}\n")
	end
	local up = up or "_"
	
	for k, v in pairs(t) do
		local ret = sf(v, k, depth+1)
		if ret then
			b:append(ret)
		elseif type(v) == "table" and not ks[v] then
			ks[v] = true
			if v.__serialize then
				sf(v, k, parent)
			else
				b:appendf("local _%d = {}\n", depth+1)
				ex.save_table(v, sf, b, ks, depth+1)
				b:appendf("_%d[ %s ] = _%d\n", depth, str_rep(k), depth + 1)
			end
		elseif type(v) == "string" or type(v) == "number" or type(v) == "boolean" then
			b:appendf("_%d[ %s ] = %s\n",
				depth, str_rep(k), str_rep(v))
		end
	end
	
	if depth == 0 then
		b:append("return _0\nend\n")
	end
	return b:tostring()
end
