#include "bootstrap.h"
#include "kernel.h"
#include "omp.h"

ok_list* K_bootstrap(ok_kernel* k, int trials, int warmup, int malgo, int miter, double mparams[]) {
    ok_list* kl = KL_alloc(trials);
    ok_list* wu = NULL;
    
    if (warmup > 0) {
        wu = KL_alloc(warmup);
        #pragma omp parallel for

        for (int i = 0; i < warmup; i++) {
            ok_kernel* k2 = K_clone(k);
            k2->flags |= NEEDS_COMPILE | BOOTSTRAP_DATA;
            K_minimize(k2, malgo, miter, mparams);
            wu->kernels[i] = k2;
        }
    }
    
    
    #pragma omp parallel for
    for (int i = 0; i < trials; i++) {
        ok_kernel * k2 = K_cloneFlags(k, SHARE_FLAGS | SHARE_STEPS);
        k2->flags |= NEEDS_COMPILE | BOOTSTRAP_DATA;
        
        K_calculate(k2);
        K_minimize(k2, malgo, miter, mparams);
        kl->kernels[i] = k2;
    }
    
    KL_free(wu);
    return kl;
}
